@@ -1,5 +1,5 @@
 #
-# demo.tcl,v 1.103 2006/08/20 19:00:07 jenglish Exp
+# demo.tcl,v 1.114 2007/12/13 20:14:02 jenglish Exp
 #
 # Tile widget set -- widget demo
 #
@@ -12,9 +12,9 @@ eval destroy [winfo children .]		;# in case script is re-sourced
 variable demodir [file dirname [info script]]
 lappend auto_path . $demodir
 package require tile
+package require keynav
 
 source [file join $demodir iconlib.tcl]
-source [file join $demodir toolbutton.tcl]
 source [file join $demodir repeater.tcl]
 
 # This forces an update of the available packages list.
@@ -58,24 +58,8 @@ if {![catch {package require Img 1.3}]} {
 ### Global data.
 #
 
-# The descriptive names of the builtin themes:
-#
-set ::THEMELIST {
-    default  	"Default"
-    classic  	"Classic"
-    alt      	"Revitalized"
-    winnative	"Windows native"
-    xpnative	"XP Native"
-    aqua	"Aqua"
-}
-array set ::THEMES $THEMELIST;
-
-# Add in any available loadable themes:
-#
-foreach name [tile::availableThemes] {
-    if {![info exists ::THEMES($name)]} {
-	lappend THEMELIST $name [set ::THEMES($name) [string totitle $name]]
-    }
+foreach theme [lsort [ttk::themes]] {
+    lappend THEMELIST $theme [string totitle $theme]
 }
 
 # Generate icons (see also: iconlib.tcl):
@@ -164,9 +148,9 @@ proc scrolled {class w args} {
     return $sf
 }
 
-## tile::scrolled -- create a widget with attached Tile scrollbars.
+## ttk::scrolled -- create a widget with attached Tile scrollbars.
 #
-proc tile::scrolled {class w args} {
+proc ttk::scrolled {class w args} {
     set sf "${w}_sf"
 
     ttk::frame $sf
@@ -178,7 +162,7 @@ proc tile::scrolled {class w args} {
     return $sf
 }
 
-## configure.scrolled -- common factor of [scrolled] and [tile::scrolled]
+## configure.scrolled -- common factor of [scrolled] and [ttk::scrolled]
 #
 proc configure.scrolled {sf w} {
     $w configure -xscrollcommand [list $sf.hsb set]
@@ -285,15 +269,12 @@ makeToolbars
 ### Theme control panel.
 #
 proc makeThemeControl {c} {
-    ttk::labelframe $c -text "Theme"
+    ttk::labelframe $c -text "Theme" -padding {6 0}
     foreach {theme name} $::THEMELIST {
 	set b [ttk::radiobutton $c.s$theme -text $name \
-		   -variable ::tile::currentTheme -value $theme \
-		   -command [list tile::setTheme $theme]]
+		   -variable ::ttk::currentTheme -value $theme \
+		   -command [list ttk::setTheme $theme]]
 	pack $b -side top -expand false -fill x
-	if {[lsearch -exact [package names] tile::theme::$theme] == -1} {
-	    $c.s$theme state disabled
-	}
     }
     return $c
 }
@@ -310,7 +291,7 @@ ttk::notebook::enableTraversal $nb
 #
 
 
-set pw [ttk::paned $nb.client -orient horizontal]
+set pw [ttk::panedwindow $nb.client -orient horizontal]
 $nb add $pw -text "Demo" -underline 0 -padding 6
 set l [ttk::labelframe $pw.l -text "Themed" -padding 6 -underline 1]
 set r [labelframe $pw.r -text "Standard" -padx 6 -pady 6]
@@ -349,7 +330,7 @@ set ::entryText "Entry widget"
 ttk::entry $l.e -textvariable ::entryText
 $l.e selection range 6 end 
 
-set ltext [tile::scrolled text $l.t -width 12 -height 5 -wrap none]
+set ltext [ttk::scrolled text $l.t -width 12 -height 5 -wrap none]
 
 grid $l.cb  -sticky ew
 grid $l.rb1 -sticky ew
@@ -416,7 +397,7 @@ $r.t insert end $txt
 proc scales.pane {scales} {
     ttk::frame $scales
 
-    ttk::paned $scales.pw -orient horizontal
+    ttk::panedwindow $scales.pw -orient horizontal
     set l [ttk::labelframe $scales.styled -text "Themed" -padding 6]
     set r [labelframe $scales.orig -text "Standard" -padx 6 -pady 6]
 
@@ -503,8 +484,10 @@ proc combobox.pane {cbf} {
     pack \
 	[ttk::combobox $cbf.cb1 -values $values -textvariable ::COMBO] \
 	[ttk::combobox $cbf.cb2 -values $values -textvariable ::COMBO ] \
+	[ttk::combobox $cbf.cb3 -values $values -textvariable ::COMBO ] \
     -side top -padx 2 -pady 2 -expand false -fill x;
-    $cbf.cb2 configure -state readonly
+    $cbf.cb1 configure -state readonly
+    $cbf.cb3 configure -height 5
     $cbf.cb1 current 3
     return $cbf
 }
@@ -514,51 +497,50 @@ $nb add [combobox.pane $nb.combos] -text "Combobox" -underline 7
 #
 proc tree.pane {w} {
     ttk::frame $w
-    ttk::scrollbar $w.vsb -command [list $w.t yview]
-    ttk::treeview $w.t -columns [list Class] \
-	-padding 4 \
-	-yscrollcommand [list sbset $w.vsb] 
+    pack [ttk::scrolled ttk::treeview $w.t -columns [list Class]] \
+	-expand true -fill both
 
-    grid $w.t $w.vsb -sticky nwse
-    grid columnconfigure $w 0 -weight 1
-    grid rowconfigure $w 0 -weight 1
-    grid propagate $w 0
-
-    #
-    # Add initial tree node: 
-    # Later nodes will be added in <<TreeviewOpen>> binding.
-    #
-    $w.t insert {} 0 -id . -text "Main Window" -open 0 \
-	-values [list [winfo class .]]
-    $w.t heading \#0 -text "Widget"
+    $w.t heading #0 -text "Widget"
     $w.t heading Class -text "Class"
-    bind $w.t <<TreeviewOpen>> [list fillTree $w.t]
+
+    # Populate tree:
+    #
+    $w.t insert {} 0 -id . -text "Main Window" \
+	-values [list [winfo class .]]
+    populateTree $w.t .
+
+    bind $w.t <KeyPress-F5> [list refreshTree %W]
 
     return $w
 }
 
-# fillTree -- <<TreeviewOpen>> binding for tree widget. 
+# populateTree $tv $id --
+#	Recursively add child nodes to treeview widget.
 #
-proc fillTree {tv} {
-    set id [$tv focus]
+proc populateTree {tv id} {
     if {![winfo exists $id]} {
 	$tv delete $id
-	return
+	return ""
     }
 
-    #
     # Replace tree item children with current list of child windows.
     #
     $tv delete [$tv children $id]
-    set children [winfo children $id]
-    foreach child $children {
-	$tv insert $id end -id $child -text [winfo name $child] -open 0 \
+    foreach child [winfo children $id] {
+	$tv insert $id end -id $child -text [winfo name $child] \
 	    -values [list [winfo class $child]]
-	if {[llength [winfo children $child]]} {
-	    # insert dummy child to show [+] indicator
-	    $tv insert $child end
-	}
+	populateTree $tv $child
     }
+    return $id 
+}
+
+# refreshTree -- <KeyPress-F5> binding for treeview demo.
+#	Recompute children.
+#
+
+proc refreshTree {tv} {
+    set id [$tv focus]
+    $tv item [populateTree $tv $id] -open true
 }
 
 $nb add [tree.pane $nb.tree] -text "Tree" -sticky news
@@ -802,6 +784,8 @@ proc showHelp {} {
 	lappend detail "Tile version [package provide tile]"
 	lappend detail "Tile library: $::tile::library"
 	lappend detail "[package ifneeded tile [package provide tile]]"
+	lappend detail "Tk version: $::tk_patchLevel"
+	lappend detail "Tk library: $::tk_library"
 	ttk::dialog .helpDialog -type ok -icon info \
 	    -message "Tile demo" -detail [join $detail \n]
     }
@@ -820,7 +804,6 @@ pack [ttk::sizegrip $statusbar.grip] -side right -anchor se
 bind $::ROOT <KeyPress-Escape>	[list event generate $cmd.close <<Invoke>>]
 bind $::ROOT <<Help>>		[list event generate $cmd.help <<Invoke>>]
 keynav::enableMnemonics $::ROOT
-keynav::defaultButton $cmd.help
 
 ### Menubar.
 #
@@ -858,17 +841,20 @@ proc makeThemeMenu {menu} {
     menu $menu
     foreach {theme name} $::THEMELIST {
 	$menu add radiobutton -label $name \
-	    -variable ::tile::currentTheme -value $theme \
-	    -command [list tile::setTheme $theme]
-	if {[lsearch -exact [package names] tile::theme::$theme] == -1} {
-	    $menu entryconfigure end -state disabled
-	}
+	    -variable ::ttk::currentTheme -value $theme \
+	    -command [list ttk::setTheme $theme]
     }
     return $menu
 }
 
 $menu add cascade -label "Theme" -underline 3 -menu [makeThemeMenu $menu.theme]
 
+## Tools menu.
+#
+$menu add cascade -label "Tools" -underline 3 -menu [menu $menu.tools]
+$menu.tools add command -label "Track focus" -command trackFocus
+$menu.tools add command -label "Widget states" -command trackStates
+
 ### Main window layout.
 #
 
