@@ -1,5 +1,5 @@
 #
-# toolbutton.tcl,v 1.5 2006/09/19 20:40:41 jenglish Exp
+# toolbutton.tcl,v 1.6 2006/11/06 02:30:49 jenglish Exp
 #
 # Demonstration of custom widget styles.
 #
@@ -42,16 +42,16 @@
 # style; see demo.tcl.)
 #
 
-style theme settings "step" {
+ttk::style theme settings "step" {
 
 #
-# First, we use [style layout] to define what elements to
+# First, we use [ttk::style layout] to define what elements to
 # use and how they're arranged.  Toolbuttons are pretty
 # simple, consisting of a border, some internal padding,
 # and a label.  (See also the TScrollbar layout definition 
 # in demos/blue.tcl for a more complicated layout spec.)
 #
-    style layout Toolbutton {
+    ttk::style layout Toolbutton {
         Toolbutton.border -children {
             Toolbutton.padding -children {
                 Toolbutton.label
@@ -67,17 +67,18 @@ style theme settings "step" {
 # For many options (like -background), the defaults
 # inherited from the parent style are sufficient.
 #
-    style configure Toolbutton -width 0 -padding 1 -relief flat -borderwidth 2
+    ttk::style configure Toolbutton \
+    	-width 0 -padding 1 -relief flat -borderwidth 2
 
 #
-# Finally, use [style map] to specify state-specific 
+# Finally, use [ttk::style map] to specify state-specific 
 # resource values.  We want a flat relief if the widget is
 # disabled, sunken if it's selected (on) or pressed, 
 # and raised when it's active (the mouse pointer is
 # over the widget).  Each state-value pair is checked
 # in order, and the first matching state takes precedence.
 #
-    style map Toolbutton -relief {
+    ttk::style map Toolbutton -relief {
 	disabled 	flat
     	selected	sunken  
 	pressed 	sunken  
