@@ -1,4 +1,4 @@
-/* altTheme.c,v 1.62 2006/08/24 05:27:03 jenglish Exp
+/* altTheme.c,v 1.70 2007/11/08 01:39:25 jenglish Exp
  *
  * Copyright (c) 2003, Joe English
  *
@@ -21,13 +21,14 @@ static const int WIN32_XDRAWLINE_HACK = 1;
 static const int WIN32_XDRAWLINE_HACK = 0;
 #endif
 
-#define MIN(a,b) (a < b ? a : b)
-
 #define BORDERWIDTH  2
 #define SCROLLBAR_WIDTH 14
 #define MIN_THUMB_SIZE 8
-#define STR(x) StR(x)
+
+#ifndef STRINGIFY
 #define StR(x) #x
+#define STRINGIFY(x) StR(x)
+#endif
 
 /*
  *----------------------------------------------------------------------
@@ -56,7 +57,7 @@ static int shadowColors[6][4] =
 };
 
 /* top-left, bottom-right */
-int thinShadowColors[6][4] =
+static int thinShadowColors[6][4] =
 {
     { FLAT, FLAT },	/* TK_RELIEF_FLAT   = 0*/
     { DARK, LITE },	/* TK_RELIEF_GROOVE = 1*/
@@ -196,7 +197,7 @@ static void ArrowPoints(Ttk_Box b, ArrowDirection dir, XPoint points[4])
 }
 
 /*public*/
-void ArrowSize(int h, ArrowDirection dir, int *widthPtr, int *heightPtr)
+void TtkArrowSize(int h, ArrowDirection dir, int *widthPtr, int *heightPtr)
 {
     switch (dir) {
 	case ARROW_UP:
@@ -207,11 +208,11 @@ void ArrowSize(int h, ArrowDirection dir, int *widthPtr, int *heightPtr)
 }
 
 /*
- * DrawArrow, FillArrow --
+ * TtkDrawArrow, TtkFillArrow --
  * 	Draw an arrow in the indicated direction inside the specified box.
  */
 /*public*/ 
-void FillArrow(
+void TtkFillArrow(
     Display *display, Drawable d, GC gc, Ttk_Box b, ArrowDirection dir)
 {
     XPoint points[4];
@@ -221,7 +222,7 @@ void FillArrow(
 }
 
 /*public*/ 
-void DrawArrow(
+void TtkDrawArrow(
     Display *display, Drawable d, GC gc, Ttk_Box b, ArrowDirection dir)
 {
     XPoint points[4];
@@ -258,7 +259,7 @@ static Ttk_ElementOptionSpec BorderElementOptions[] =
     { "-default", TK_OPTION_ANY, Tk_Offset(BorderElement,defaultStateObj),
     	"disabled" },
     { "-borderwidth",TK_OPTION_PIXELS,Tk_Offset(BorderElement,borderWidthObj),
-    	STR(BORDERWIDTH) },
+    	STRINGIFY(BORDERWIDTH) },
     { "-relief", TK_OPTION_RELIEF, Tk_Offset(BorderElement,reliefObj),
     	"flat" },
     {NULL}
@@ -282,8 +283,7 @@ static void BorderElementSize(
     *paddingPtr = Ttk_UniformPadding((short)borderWidth);
 }
 
-static void
-BorderElementDraw(
+static void BorderElementDraw(
     void *clientData, void *elementRecord,
     Tk_Window tkwin, Drawable d, Ttk_Box b, unsigned int state)
 {
@@ -407,7 +407,7 @@ static char *button_images[] = {
 };
 #endif
 
-Ttk_StateTable checkbutton_states[] =
+static Ttk_StateTable checkbutton_states[] =
 {
     { 0, 0, TTK_STATE_SELECTED|TTK_STATE_DISABLED },
     { 1, TTK_STATE_SELECTED, TTK_STATE_DISABLED },
@@ -431,14 +431,14 @@ static char *checkbutton_pixels[] = {
     "BBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBBB",
 };
 
-IndicatorSpec checkbutton_spec =
+static IndicatorSpec checkbutton_spec =
 {
     13, 13, 4,		/* width, height, nimages */
     checkbutton_pixels,
     checkbutton_states
 };
 
-Ttk_StateTable radiobutton_states[] =
+static Ttk_StateTable radiobutton_states[] =
 {
     { 0, 0, TTK_STATE_SELECTED|TTK_STATE_DISABLED },
     { 1, TTK_STATE_SELECTED, TTK_STATE_DISABLED },
@@ -463,7 +463,7 @@ static char *radiobutton_pixels[] = {
     "FFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFFF",
 };
 
-IndicatorSpec radiobutton_spec =
+static IndicatorSpec radiobutton_spec =
 {
     13, 13, 4,		/* width, height, nimages */
     radiobutton_pixels,
@@ -503,9 +503,10 @@ static void IndicatorElementSize(
 {
     IndicatorSpec *spec = clientData;
     IndicatorElement *indicator = elementRecord;
-    Ttk_GetPaddingFromObj(NULL, tkwin, indicator->marginObj, paddingPtr);
-    *widthPtr = spec->width;
-    *heightPtr = spec->height;
+    Ttk_Padding margins;
+    Ttk_GetPaddingFromObj(NULL, tkwin, indicator->marginObj, &margins);
+    *widthPtr = spec->width + Ttk_PaddingWidth(margins);
+    *heightPtr = spec->height + Ttk_PaddingHeight(margins);
 }
 
 static void IndicatorElementDraw(
@@ -626,7 +627,7 @@ typedef struct
 static Ttk_ElementOptionSpec ArrowElementOptions[] =
 {
     { "-arrowsize", TK_OPTION_PIXELS,
-	Tk_Offset(ArrowElement,sizeObj), STR(SCROLLBAR_WIDTH) },
+	Tk_Offset(ArrowElement,sizeObj), STRINGIFY(SCROLLBAR_WIDTH) },
     { "-background", TK_OPTION_BORDER, 
 	Tk_Offset(ArrowElement,borderObj), DEFAULT_BACKGROUND },
     { "-bordercolor", TK_OPTION_COLOR, 
@@ -655,8 +656,9 @@ static void ArrowElementSize(
 
     Tk_GetPixelsFromObj(NULL, tkwin, arrow->sizeObj, &width);
     width -= Ttk_PaddingWidth(ArrowPadding);
-    ArrowSize(width/2, direction, widthPtr, heightPtr);
-    *paddingPtr = ArrowPadding;
+    TtkArrowSize(width/2, direction, widthPtr, heightPtr);
+    *widthPtr += Ttk_PaddingWidth(ArrowPadding);
+    *heightPtr += Ttk_PaddingHeight(ArrowPadding);
 }
 
 static void ArrowElementDraw(
@@ -677,7 +679,7 @@ static void ArrowElementDraw(
 	tkwin, d, border, b.x, b.y, b.width, b.height, 0, TK_RELIEF_FLAT);
     DrawBorder(tkwin,d,border,borderColor,b,borderWidth,relief);
 
-    FillArrow(Tk_Display(tkwin), d, Tk_GCForColor(arrowColor, d),
+    TtkFillArrow(Tk_Display(tkwin), d, Tk_GCForColor(arrowColor, d),
 	Ttk_PadBox(b, ArrowPadding), direction);
 }
 
@@ -713,7 +715,7 @@ static Ttk_ElementOptionSpec MenubuttonArrowElementOptions[] =
     { "-direction", TK_OPTION_STRING,
 	Tk_Offset(MenubuttonArrowElement,directionObj), "below" },
     { "-arrowsize", TK_OPTION_PIXELS,
-	Tk_Offset(MenubuttonArrowElement,sizeObj), STR(MENUBUTTON_ARROW_SIZE)},
+	Tk_Offset(MenubuttonArrowElement,sizeObj), STRINGIFY(MENUBUTTON_ARROW_SIZE)},
     { "-arrowcolor",TK_OPTION_COLOR,
 	Tk_Offset(MenubuttonArrowElement,colorObj), "black"},
     { NULL }
@@ -729,7 +731,8 @@ static void MenubuttonArrowElementSize(
     int size = MENUBUTTON_ARROW_SIZE;
     Tk_GetPixelsFromObj(NULL, tkwin, arrow->sizeObj, &size);
     *widthPtr = *heightPtr = 2 * size + 1;
-    *paddingPtr = MenubuttonArrowPadding;
+    *widthPtr += Ttk_PaddingWidth(MenubuttonArrowPadding);
+    *heightPtr += Ttk_PaddingHeight(MenubuttonArrowPadding);
 }
 
 static void MenubuttonArrowElementDraw(
@@ -742,7 +745,7 @@ static void MenubuttonArrowElementDraw(
     int size = MENUBUTTON_ARROW_SIZE;
     int postDirection = POST_BELOW;
     ArrowDirection arrowDirection = ARROW_DOWN;
-    int width, height;
+    int width = 0, height = 0;
 
     Tk_GetPixelsFromObj(NULL, tkwin, arrow->sizeObj, &size);
     Tcl_GetIndexFromObj(NULL, arrow->directionObj, directionStrings,
@@ -757,10 +760,10 @@ static void MenubuttonArrowElementDraw(
 	case POST_FLUSH:	arrowDirection = ARROW_DOWN; break;
     }
 
-    ArrowSize(size, arrowDirection, &width, &height);
+    TtkArrowSize(size, arrowDirection, &width, &height);
     b = Ttk_PadBox(b, MenubuttonArrowPadding);
     b = Ttk_AnchorBox(b, width, height, TK_ANCHOR_CENTER);
-    FillArrow(Tk_Display(tkwin), d, gc, b, arrowDirection);
+    TtkFillArrow(Tk_Display(tkwin), d, gc, b, arrowDirection);
 }
 
 static Ttk_ElementSpec MenubuttonArrowElementSpec =
@@ -880,7 +883,7 @@ typedef struct
 static Ttk_ElementOptionSpec ThumbElementOptions[] =
 {
     { "-width", TK_OPTION_PIXELS, Tk_Offset(ThumbElement,sizeObj),
-        STR(SCROLLBAR_WIDTH) },
+        STRINGIFY(SCROLLBAR_WIDTH) },
     { "-background", TK_OPTION_BORDER, Tk_Offset(ThumbElement,borderObj),
 	DEFAULT_BACKGROUND },
     { "-bordercolor", TK_OPTION_COLOR, Tk_Offset(ThumbElement,borderColorObj),
@@ -972,7 +975,7 @@ static Ttk_ElementOptionSpec SliderElementOptions[] =
     { "-sliderrelief", TK_OPTION_RELIEF, Tk_Offset(SliderElement,reliefObj),
 	"raised" },
     { "-borderwidth", TK_OPTION_PIXELS, Tk_Offset(SliderElement,borderWidthObj),
-	STR(BORDERWIDTH) },
+	STRINGIFY(BORDERWIDTH) },
     { "-background", TK_OPTION_BORDER, Tk_Offset(SliderElement,borderObj),
 	DEFAULT_BACKGROUND },
     { "-bordercolor", TK_OPTION_COLOR, Tk_Offset(ThumbElement,borderColorObj),
@@ -1053,9 +1056,9 @@ static Ttk_ElementOptionSpec TreeitemIndicatorOptions[] =
     { "-foreground", TK_OPTION_COLOR,
 	Tk_Offset(TreeitemIndicator,colorObj), DEFAULT_FOREGROUND },
     { "-diameter", TK_OPTION_PIXELS,
-	Tk_Offset(TreeitemIndicator,diameterObj), "6" },
+	Tk_Offset(TreeitemIndicator,diameterObj), "9" },
     { "-indicatormargins", TK_OPTION_STRING,
-	Tk_Offset(TreeitemIndicator,marginObj), "0 2 4 2" },
+	Tk_Offset(TreeitemIndicator,marginObj), "2 2 4 2" },
     {NULL}
 };
 
@@ -1065,9 +1068,12 @@ static void TreeitemIndicatorSize(
 {
     TreeitemIndicator *indicator = elementRecord;
     int diameter = 0;
-    Ttk_GetPaddingFromObj(NULL, tkwin, indicator->marginObj, paddingPtr);
+    Ttk_Padding margins;
+
+    Ttk_GetPaddingFromObj(NULL, tkwin, indicator->marginObj, &margins);
     Tk_GetPixelsFromObj(NULL, tkwin, indicator->diameterObj, &diameter);
-    *widthPtr = *heightPtr = diameter;
+    *widthPtr = diameter + Ttk_PaddingWidth(margins);
+    *heightPtr = diameter + Ttk_PaddingHeight(margins);
 }
 
 static void TreeitemIndicatorDraw(
@@ -1090,15 +1096,15 @@ static void TreeitemIndicatorDraw(
     b = Ttk_PadBox(b, padding);
 
     XDrawRectangle(Tk_Display(tkwin), d, gc,
-	    b.x, b.y, b.width, b.height);
+	    b.x, b.y, b.width - 1, b.height - 1);
 
-    cx = b.x + b.width / 2;
-    cy = b.y + b.height / 2;
-    XDrawLine(Tk_Display(tkwin), d, gc, b.x+2, cy, b.x+b.width-2+w, cy);
+    cx = b.x + (b.width - 1) / 2;
+    cy = b.y + (b.height - 1) / 2;
+    XDrawLine(Tk_Display(tkwin), d, gc, b.x+2, cy, b.x+b.width-3+w, cy);
 
     if (!(state & TTK_STATE_OPEN)) {
 	/* turn '-' into a '+' */
-	XDrawLine(Tk_Display(tkwin), d, gc, cx, b.y+2, cx, b.y+b.height-2+w);
+	XDrawLine(Tk_Display(tkwin), d, gc, cx, b.y+2, cx, b.y+b.height-3+w);
     }
 }
 
@@ -1115,10 +1121,10 @@ static Ttk_ElementSpec TreeitemIndicatorElementSpec =
 
 
 /*------------------------------------------------------------------------
- * AltTheme_Init --
+ * TtkAltTheme_Init --
  * 	Install alternate theme.
  */
-int AltTheme_Init(Tcl_Interp *interp)
+int TtkAltTheme_Init(Tcl_Interp *interp)
 {
     Ttk_Theme theme =  Ttk_CreateTheme(interp, "alt", NULL);
 
@@ -1158,7 +1164,7 @@ int AltTheme_Init(Tcl_Interp *interp)
     Ttk_RegisterElement(interp, theme, "Treeitem.indicator",
 	    &TreeitemIndicatorElementSpec, 0);
 
-    Tcl_PkgProvide(interp, "tile::theme::alt", TILE_VERSION);
+    Tcl_PkgProvide(interp, "ttk::theme::alt", TILE_VERSION);
 
     return TCL_OK;
 }
