@@ -1,8 +1,7 @@
-/* button.c,v 1.48 2006/08/04 23:22:50 patthoyts Exp
+/* button.c,v 1.52 2007/10/25 06:42:47 jenglish Exp
  * Copyright (c) 2003, Joe English
  *
- * Tile widget set: label, button, checkbutton, radiobutton, and 
- * menubutton widgets.
+ * label, button, checkbutton, radiobutton, and menubutton widgets.
  */
 
 #include <string.h>
@@ -29,7 +28,7 @@ typedef struct
     Tcl_Obj *widthObj;
 
     Ttk_TraceHandle	*textVariableTrace;
-    Tk_Image	*images;
+    Ttk_ImageSpec	*imageSpec;
 
     /*
      * Image element resources:
@@ -83,7 +82,7 @@ static Tk_OptionSpec BaseOptionSpecs[] =
      */
     {TK_OPTION_STRING_TABLE, "-compound", "compound", "Compound",
 	 "none", Tk_Offset(Base,base.compoundObj), -1,
-	 0,(ClientData)TTKCompoundStrings,GEOMETRY_CHANGED },
+	 0,(ClientData)ttkCompoundStrings,GEOMETRY_CHANGED },
     {TK_OPTION_STRING, "-padding", "padding", "Pad",
 	NULL, Tk_Offset(Base,base.paddingObj), -1,
 	TK_OPTION_NULL_OK,0,GEOMETRY_CHANGED},
@@ -95,7 +94,7 @@ static Tk_OptionSpec BaseOptionSpecs[] =
 	 "normal", Tk_Offset(Base,base.stateObj), -1,
 	 0,0,STATE_CHANGED },
 
-    WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
+    WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
 };
 
 /*
@@ -119,106 +118,12 @@ static void TextVariableChanged(void *clientData, const char *value)
     TtkResizeWidget(&basePtr->core);
 }
 
-/*
- * Tk_ImageChangedProc for -image option:
- */
-static void CoreImageChangedProc(ClientData clientData,
-    int x, int y, int width, int height, int imageWidth, int imageHeight)
-{
-    WidgetCore *corePtr = (WidgetCore *)clientData;
-    TtkRedisplayWidget(corePtr);
-}
-
-/* GetImageList --
- * 	ConfigureProc utility routine for handling -image option.
- * 	Verifies that -image is a valid image specification,
- * 	registers image-changed callbacks for each image (via Tk_GetImage).
- *
- * 	The -image option is a multi-element list; the first element
- * 	is the name of the default image to use, the remainder of the 
- * 	list is a sequence of statespec/imagename options as per 
- * 	[style map].
- *
- * Returns: TCL_OK if image specification is valid and sets *imageListPtr
- * 	to a NULL-terminated list of Tk_Images; otherwise TCL_ERROR
- * 	and leaves an error message in the interpreter result.
- */
-int GetImageList(
-    Tcl_Interp *interp, 
-    WidgetCore *corePtr, 
-    Tcl_Obj *imageOption, 
-    Tk_Image **imageListPtr)
-{
-    int i, mapCount, imageCount;
-    Tcl_Obj **mapList;
-    Tk_Image *images;
-
-    if (Tcl_ListObjGetElements(interp, 
-		imageOption, &mapCount, &mapList) != TCL_OK) 
-    {
-	return TCL_ERROR;
-    }
-
-    if (mapCount == 0) {
-	*imageListPtr = 0;
-	return TCL_OK;
-    }
-
-    if ((mapCount % 2) != 1) {
-	Tcl_SetResult(interp,
-	    "-image value must contain an odd number of elements", TCL_STATIC);
-	return TCL_ERROR;
-    }
-
-    /* Verify state specifications:
-     */
-    for (i = 1; i < mapCount -1; i += 2) {
-	Ttk_StateSpec spec;
-	if (Ttk_GetStateSpecFromObj(interp, mapList[i], &spec) != TCL_OK)
-	    return TCL_ERROR;
-    }
-    
-    /* Get images:
-     */
-    imageCount = (mapCount + 1) / 2;
-    images = (Tk_Image*)ckalloc((imageCount+1) * sizeof(Tk_Image));
-
-    for (i = 0; i < imageCount; ++i) {
-	const char *imageName = Tcl_GetString(mapList[i * 2]);
-	images[i] = Tk_GetImage(interp, corePtr->tkwin, 
-		imageName, CoreImageChangedProc, corePtr);
-
-	if (!images[i]) {
-	    while (i--)
-		Tk_FreeImage(images[i]);
-	    ckfree((ClientData)images);
-	    return TCL_ERROR;
-	}
-    }
-    images[i] = NULL;	/* Add null terminator */
-
-    *imageListPtr = images;
-    return TCL_OK;
-}
-
-/*
- * FreeImageList --
- * 	Release an image list obtained by GetImageList.
- */
-void FreeImageList(Tk_Image *imageList)
-{
-    Tk_Image *p;
-    for (p = imageList; *p; ++p)
-	Tk_FreeImage(*p);
-    ckfree((ClientData)imageList);
-}
-
 static int
 BaseInitialize(Tcl_Interp *interp, void *recordPtr)
 {
     Base *basePtr = recordPtr;
     basePtr->base.textVariableTrace = 0;
-    basePtr->base.images = NULL;
+    basePtr->base.imageSpec = NULL;
     return TCL_OK;
 }
 
@@ -228,8 +133,8 @@ BaseCleanup(void *recordPtr)
     Base *basePtr = recordPtr;
     if (basePtr->base.textVariableTrace)
 	Ttk_UntraceVariable(basePtr->base.textVariableTrace);
-    if (basePtr->base.images) 
-    	FreeImageList(basePtr->base.images);
+    if (basePtr->base.imageSpec) 
+    	TtkFreeImageSpec(basePtr->base.imageSpec);
 }
 
 static int BaseConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
@@ -237,22 +142,24 @@ static int BaseConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
     Base *basePtr = recordPtr;
     Tcl_Obj *textVarName = basePtr->base.textVariableObj;
     Ttk_TraceHandle *vt = 0;
-    Tk_Image *images = NULL;
+    Ttk_ImageSpec *imageSpec = NULL;
 
     if (textVarName != NULL && *Tcl_GetString(textVarName) != '\0') {
 	vt = Ttk_TraceVariable(interp,textVarName,TextVariableChanged,basePtr);
 	if (!vt) return TCL_ERROR;
     }
 
-    if (basePtr->base.imageObj && GetImageList(interp, 
-		&basePtr->core, basePtr->base.imageObj, &images) != TCL_OK)
-    {
-	goto error;
+    if (basePtr->base.imageObj) {
+	imageSpec = TtkGetImageSpec(
+	    interp, basePtr->core.tkwin, basePtr->base.imageObj);
+	if (!imageSpec) {
+	    goto error;
+	}
     }
 
-    if (CoreConfigure(interp, recordPtr, mask) != TCL_OK) {
+    if (TtkCoreConfigure(interp, recordPtr, mask) != TCL_OK) {
 error:
-	if (images) FreeImageList(images);
+	if (imageSpec) TtkFreeImageSpec(imageSpec);
 	if (vt) Ttk_UntraceVariable(vt);
 	return TCL_ERROR;
     }
@@ -262,13 +169,13 @@ error:
     }
     basePtr->base.textVariableTrace = vt;
 
-    if (basePtr->base.images) {
-	FreeImageList(basePtr->base.images);
+    if (basePtr->base.imageSpec) {
+	TtkFreeImageSpec(basePtr->base.imageSpec);
     }
-    basePtr->base.images = images;
+    basePtr->base.imageSpec = imageSpec;
 
     if (mask & STATE_CHANGED) {
-	CheckStateOption(&basePtr->core, basePtr->base.stateObj);
+	TtkCheckStateOption(&basePtr->core, basePtr->base.stateObj);
     }
 
     return TCL_OK;
@@ -287,9 +194,8 @@ BasePostConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
     return status;
 }
 
-
 /*------------------------------------------------------------------------
- * +++ Label widget:
+ * +++ Label widget.
  * Just a base widget that adds a few appearance-related options
  */
 
@@ -344,15 +250,15 @@ static Tk_OptionSpec LabelOptionSpecs[] =
 
 static WidgetCommandSpec LabelCommands[] =
 {
-    { "configure",	WidgetConfigureCommand },
-    { "cget",		WidgetCgetCommand },
-    { "instate",	WidgetInstateCommand },
-    { "state",  	WidgetStateCommand },
-    { "identify",	WidgetIdentifyCommand },
+    { "configure",	TtkWidgetConfigureCommand },
+    { "cget",		TtkWidgetCgetCommand },
+    { "instate",	TtkWidgetInstateCommand },
+    { "state",  	TtkWidgetStateCommand },
+    { "identify",	TtkWidgetIdentifyCommand },
     { NULL, NULL }
 };
 
-WidgetSpec LabelWidgetSpec =
+static WidgetSpec LabelWidgetSpec =
 {
     "TLabel",			/* className */
     sizeof(Label),		/* recordSize */
@@ -362,12 +268,18 @@ WidgetSpec LabelWidgetSpec =
     BaseCleanup,		/* cleanupProc */
     BaseConfigure,		/* configureProc */
     BasePostConfigure,		/* postConfigureProc */
-    WidgetGetLayout, 		/* getLayoutProc */
-    WidgetSize, 		/* sizeProc */
-    WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay		/* displayProc */
+    TtkWidgetGetLayout, 	/* getLayoutProc */
+    TtkWidgetSize, 		/* sizeProc */
+    TtkWidgetDoLayout,		/* layoutProc */
+    TtkWidgetDisplay		/* displayProc */
 };
 
+TTK_BEGIN_LAYOUT(LabelLayout)
+    TTK_GROUP("Label.border", TTK_FILL_BOTH|TTK_BORDER,
+	TTK_GROUP("Label.padding", TTK_FILL_BOTH|TTK_BORDER,
+	    TTK_NODE("Label.label", TTK_FILL_BOTH)))
+TTK_END_LAYOUT
+
 /*------------------------------------------------------------------------
  * +++ Button widget.
  * Adds a new subcommand "invoke", and options "-command" and "-default"
@@ -397,7 +309,7 @@ static Tk_OptionSpec ButtonOptionSpecs[] =
 	"", Tk_Offset(Button, button.commandObj), -1, 0,0,0},
     {TK_OPTION_STRING_TABLE, "-default", "default", "Default",
 	"normal", Tk_Offset(Button, button.defaultStateObj), -1,
-	0, (ClientData) TTKDefaultStrings, DEFAULTSTATE_CHANGED},
+	0, (ClientData) ttkDefaultStrings, DEFAULTSTATE_CHANGED},
 
     WIDGET_INHERIT_OPTIONS(BaseOptionSpecs)
 };
@@ -417,9 +329,9 @@ static int ButtonConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 	Ttk_GetButtonDefaultStateFromObj(
 	    NULL, buttonPtr->button.defaultStateObj, &defaultState);
 	if (defaultState == TTK_BUTTON_DEFAULT_ACTIVE) {
-	    WidgetChangeState(&buttonPtr->core, TTK_STATE_ALTERNATE, 0);
+	    TtkWidgetChangeState(&buttonPtr->core, TTK_STATE_ALTERNATE, 0);
 	} else {
-	    WidgetChangeState(&buttonPtr->core, 0, TTK_STATE_ALTERNATE);
+	    TtkWidgetChangeState(&buttonPtr->core, 0, TTK_STATE_ALTERNATE);
 	}
     }
     return TCL_OK;
@@ -445,16 +357,16 @@ ButtonInvokeCommand(
 
 static WidgetCommandSpec ButtonCommands[] =
 {
-    { "configure",	WidgetConfigureCommand },
-    { "cget",		WidgetCgetCommand },
+    { "configure",	TtkWidgetConfigureCommand },
+    { "cget",		TtkWidgetCgetCommand },
     { "invoke",		ButtonInvokeCommand },
-    { "instate",	WidgetInstateCommand },
-    { "state",  	WidgetStateCommand },
-    { "identify",	WidgetIdentifyCommand },
+    { "instate",	TtkWidgetInstateCommand },
+    { "state",  	TtkWidgetStateCommand },
+    { "identify",	TtkWidgetIdentifyCommand },
     { NULL, NULL }
 };
 
-WidgetSpec ButtonWidgetSpec =
+static WidgetSpec ButtonWidgetSpec =
 {
     "TButton",			/* className */
     sizeof(Button),		/* recordSize */
@@ -464,12 +376,19 @@ WidgetSpec ButtonWidgetSpec =
     BaseCleanup,		/* cleanupProc */
     ButtonConfigure,		/* configureProc */
     BasePostConfigure,		/* postConfigureProc */
-    WidgetGetLayout, 		/* getLayoutProc */
-    WidgetSize, 		/* sizeProc */
-    WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay		/* displayProc */
+    TtkWidgetGetLayout,		/* getLayoutProc */
+    TtkWidgetSize, 		/* sizeProc */
+    TtkWidgetDoLayout,		/* layoutProc */
+    TtkWidgetDisplay		/* displayProc */
 };
 
+TTK_BEGIN_LAYOUT(ButtonLayout)
+    TTK_GROUP("Button.border", TTK_FILL_BOTH|TTK_BORDER,
+	TTK_GROUP("Button.focus", TTK_FILL_BOTH,
+	    TTK_GROUP("Button.padding", TTK_FILL_BOTH,
+	        TTK_NODE("Button.label", TTK_FILL_BOTH))))
+TTK_END_LAYOUT
+
 /*------------------------------------------------------------------------
  * +++ Checkbutton widget.
  */
@@ -522,15 +441,15 @@ static void CheckbuttonVariableChanged(void *clientData, const char *value)
     }
 
     if (!value) {
-	WidgetChangeState(&checkPtr->core, TTK_STATE_ALTERNATE, 0);
+	TtkWidgetChangeState(&checkPtr->core, TTK_STATE_ALTERNATE, 0);
 	return;
     }
     /* else */
-    WidgetChangeState(&checkPtr->core, 0, TTK_STATE_ALTERNATE);
+    TtkWidgetChangeState(&checkPtr->core, 0, TTK_STATE_ALTERNATE);
     if (!strcmp(value, Tcl_GetString(checkPtr->checkbutton.onValueObj))) {
-	WidgetChangeState(&checkPtr->core, TTK_STATE_SELECTED, 0);
+	TtkWidgetChangeState(&checkPtr->core, TTK_STATE_SELECTED, 0);
     } else {
-	WidgetChangeState(&checkPtr->core, 0, TTK_STATE_SELECTED);
+	TtkWidgetChangeState(&checkPtr->core, 0, TTK_STATE_SELECTED);
     }
 }
 
@@ -636,17 +555,17 @@ CheckbuttonInvokeCommand(
 
 static WidgetCommandSpec CheckbuttonCommands[] =
 {
-    { "configure",	WidgetConfigureCommand },
-    { "cget",		WidgetCgetCommand },
+    { "configure",	TtkWidgetConfigureCommand },
+    { "cget",		TtkWidgetCgetCommand },
     { "invoke",		CheckbuttonInvokeCommand },
-    { "instate",	WidgetInstateCommand },
-    { "state",  	WidgetStateCommand },
-    { "identify",	WidgetIdentifyCommand },
+    { "instate",	TtkWidgetInstateCommand },
+    { "state",  	TtkWidgetStateCommand },
+    { "identify",	TtkWidgetIdentifyCommand },
     /* MISSING: select, deselect, toggle */
     { NULL, NULL }
 };
 
-WidgetSpec CheckbuttonWidgetSpec =
+static WidgetSpec CheckbuttonWidgetSpec =
 {
     "TCheckbutton",		/* className */
     sizeof(Checkbutton),	/* recordSize */
@@ -656,12 +575,19 @@ WidgetSpec CheckbuttonWidgetSpec =
     CheckbuttonCleanup,		/* cleanupProc */
     CheckbuttonConfigure,	/* configureProc */
     CheckbuttonPostConfigure,	/* postConfigureProc */
-    WidgetGetLayout, 		/* getLayoutProc */
-    WidgetSize, 		/* sizeProc */
-    WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay		/* displayProc */
+    TtkWidgetGetLayout, 	/* getLayoutProc */
+    TtkWidgetSize, 		/* sizeProc */
+    TtkWidgetDoLayout,		/* layoutProc */
+    TtkWidgetDisplay		/* displayProc */
 };
 
+TTK_BEGIN_LAYOUT(CheckbuttonLayout)
+     TTK_GROUP("Checkbutton.padding", TTK_FILL_BOTH,
+	 TTK_NODE("Checkbutton.indicator", TTK_PACK_LEFT)
+	 TTK_GROUP("Checkbutton.focus", TTK_PACK_LEFT | TTK_STICK_W,
+	     TTK_NODE("Checkbutton.label", TTK_FILL_BOTH)))
+TTK_END_LAYOUT
+
 /*------------------------------------------------------------------------
  * +++ Radiobutton widget.
  */
@@ -716,15 +642,15 @@ RadiobuttonVariableChanged(void *clientData, const char *value)
     }
 
     if (!value) {
-	WidgetChangeState(&radioPtr->core, TTK_STATE_ALTERNATE, 0);
+	TtkWidgetChangeState(&radioPtr->core, TTK_STATE_ALTERNATE, 0);
 	return;
     }
     /* else */
-    WidgetChangeState(&radioPtr->core, 0, TTK_STATE_ALTERNATE);
+    TtkWidgetChangeState(&radioPtr->core, 0, TTK_STATE_ALTERNATE);
     if (!strcmp(value, Tcl_GetString(radioPtr->radiobutton.valueObj))) {
-	WidgetChangeState(&radioPtr->core, TTK_STATE_SELECTED, 0);
+	TtkWidgetChangeState(&radioPtr->core, TTK_STATE_SELECTED, 0);
     } else {
-	WidgetChangeState(&radioPtr->core, 0, TTK_STATE_SELECTED);
+	TtkWidgetChangeState(&radioPtr->core, 0, TTK_STATE_SELECTED);
     }
 }
 
@@ -807,17 +733,17 @@ RadiobuttonInvokeCommand(
 
 static WidgetCommandSpec RadiobuttonCommands[] =
 {
-    { "configure",	WidgetConfigureCommand },
-    { "cget",		WidgetCgetCommand },
+    { "configure",	TtkWidgetConfigureCommand },
+    { "cget",		TtkWidgetCgetCommand },
     { "invoke",		RadiobuttonInvokeCommand },
-    { "instate",	WidgetInstateCommand },
-    { "state",  	WidgetStateCommand },
-    { "identify",	WidgetIdentifyCommand },
+    { "instate",	TtkWidgetInstateCommand },
+    { "state",  	TtkWidgetStateCommand },
+    { "identify",	TtkWidgetIdentifyCommand },
     /* MISSING: select, deselect */
     { NULL, NULL }
 };
 
-WidgetSpec RadiobuttonWidgetSpec =
+static WidgetSpec RadiobuttonWidgetSpec =
 {
     "TRadiobutton",		/* className */
     sizeof(Radiobutton),	/* recordSize */
@@ -827,12 +753,19 @@ WidgetSpec RadiobuttonWidgetSpec =
     RadiobuttonCleanup,		/* cleanupProc */
     RadiobuttonConfigure,	/* configureProc */
     RadiobuttonPostConfigure,	/* postConfigureProc */
-    WidgetGetLayout, 		/* getLayoutProc */
-    WidgetSize, 		/* sizeProc */
-    WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay		/* displayProc */
+    TtkWidgetGetLayout, 	/* getLayoutProc */
+    TtkWidgetSize, 		/* sizeProc */
+    TtkWidgetDoLayout,		/* layoutProc */
+    TtkWidgetDisplay		/* displayProc */
 };
 
+TTK_BEGIN_LAYOUT(RadiobuttonLayout)
+     TTK_GROUP("Radiobutton.padding", TTK_FILL_BOTH,
+	 TTK_NODE("Radiobutton.indicator", TTK_PACK_LEFT)
+	 TTK_GROUP("Radiobutton.focus", TTK_PACK_LEFT,
+	     TTK_NODE("Radiobutton.label", TTK_FILL_BOTH)))
+TTK_END_LAYOUT
+
 /*------------------------------------------------------------------------
  * +++ Menubutton widget.
  */
@@ -871,15 +804,15 @@ static Tk_OptionSpec MenubuttonOptionSpecs[] =
 
 static WidgetCommandSpec MenubuttonCommands[] =
 {
-    { "configure",	WidgetConfigureCommand },
-    { "cget",		WidgetCgetCommand },
-    { "instate",	WidgetInstateCommand },
-    { "state",  	WidgetStateCommand },
-    { "identify",	WidgetIdentifyCommand },
+    { "configure",	TtkWidgetConfigureCommand },
+    { "cget",		TtkWidgetCgetCommand },
+    { "instate",	TtkWidgetInstateCommand },
+    { "state",  	TtkWidgetStateCommand },
+    { "identify",	TtkWidgetIdentifyCommand },
     { NULL, NULL }
 };
 
-WidgetSpec MenubuttonWidgetSpec =
+static WidgetSpec MenubuttonWidgetSpec =
 {
     "TMenubutton",		/* className */
     sizeof(Menubutton), 	/* recordSize */
@@ -889,9 +822,37 @@ WidgetSpec MenubuttonWidgetSpec =
     BaseCleanup,		/* cleanupProc */
     BaseConfigure,		/* configureProc */
     BasePostConfigure,  	/* postConfigureProc */
-    WidgetGetLayout, 		/* getLayoutProc */
-    WidgetSize, 		/* sizeProc */
-    WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay		/* displayProc */
+    TtkWidgetGetLayout, 	/* getLayoutProc */
+    TtkWidgetSize, 		/* sizeProc */
+    TtkWidgetDoLayout,		/* layoutProc */
+    TtkWidgetDisplay		/* displayProc */
 };
 
+TTK_BEGIN_LAYOUT(MenubuttonLayout)
+    TTK_GROUP("Menubutton.border", TTK_FILL_BOTH,
+	TTK_GROUP("Menubutton.focus", TTK_FILL_BOTH,
+	    TTK_NODE("Menubutton.indicator", TTK_PACK_RIGHT)
+	    TTK_GROUP("Menubutton.padding", TTK_PACK_LEFT|TTK_EXPAND|TTK_FILL_X,
+	        TTK_NODE("Menubutton.label", TTK_PACK_LEFT))))
+TTK_END_LAYOUT
+
+/*------------------------------------------------------------------------
+ * +++ Initialization.
+ */
+
+void TtkButton_Init(Tcl_Interp *interp)
+{
+    Ttk_Theme theme = Ttk_GetDefaultTheme(interp);
+
+    Ttk_RegisterLayout(theme, "TLabel", LabelLayout);
+    Ttk_RegisterLayout(theme, "TButton", ButtonLayout);
+    Ttk_RegisterLayout(theme, "TCheckbutton", CheckbuttonLayout);
+    Ttk_RegisterLayout(theme, "TRadiobutton", RadiobuttonLayout);
+    Ttk_RegisterLayout(theme, "TMenubutton", MenubuttonLayout);
+
+    RegisterWidget(interp, "ttk::label", &LabelWidgetSpec);
+    RegisterWidget(interp, "ttk::button", &ButtonWidgetSpec);
+    RegisterWidget(interp, "ttk::checkbutton", &CheckbuttonWidgetSpec);
+    RegisterWidget(interp, "ttk::radiobutton", &RadiobuttonWidgetSpec);
+    RegisterWidget(interp, "ttk::menubutton", &MenubuttonWidgetSpec);
+}
