@@ -1,5 +1,5 @@
 /*
- * entry.c,v 1.41 2006/10/03 18:22:01 jenglish Exp
+ * entry.c,v 1.46 2007/05/18 21:50:49 jenglish Exp
  *
  * DERIVED FROM: tk/generic/tkEntry.c r1.35.
  *
@@ -73,7 +73,7 @@ static const char *validateReasonStrings[] = {
 
 /* Style parameters:
  */
-typedef struct 
+typedef struct
 {
     Tcl_Obj *foregroundObj;	/* Foreground color for normal text */
     Tcl_Obj *backgroundObj;	/* Entry widget background color */
@@ -156,6 +156,7 @@ typedef struct
 #define DEF_INSERT_BG	"black"
 #define DEF_ENTRY_WIDTH	"20"
 #define DEF_ENTRY_FONT	"TkTextFont"
+#define DEF_LIST_HEIGHT	"10"
 
 static Tk_OptionSpec EntryOptionSpecs[] =
 {
@@ -204,7 +205,7 @@ static Tk_OptionSpec EntryOptionSpecs[] =
 	NULL, Tk_Offset(Entry, entry.styleData.backgroundObj), -1,
 	TK_OPTION_NULL_OK,0,0},
 
-    WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
+    WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
 };
 
 /*------------------------------------------------------------------------
@@ -565,7 +566,7 @@ static int EntryNeedsValidation(VMODE vmode, VREASON reason)
  *	TCL_BREAK if the change is rejected
  *      TCL_ERROR if any errors occured
  *
- * The change will be rejected if -validatecommand returns 0, 
+ * The change will be rejected if -validatecommand returns 0,
  * or if -validatecommand or -invalidcommand modifies the value.
  */
 static int
@@ -642,9 +643,9 @@ static int EntryRevalidate(Tcl_Interp *interp, Entry *entryPtr, VREASON reason)
 		    entryPtr, entryPtr->entry.string, -1,0, reason);
 
     if (code == TCL_BREAK) {
-	WidgetChangeState(&entryPtr->core, TTK_STATE_INVALID, 0);
+	TtkWidgetChangeState(&entryPtr->core, TTK_STATE_INVALID, 0);
     } else if (code == TCL_OK) {
-	WidgetChangeState(&entryPtr->core, 0, TTK_STATE_INVALID);
+	TtkWidgetChangeState(&entryPtr->core, 0, TTK_STATE_INVALID);
     }
 
     return code;
@@ -749,13 +750,15 @@ EntryStoreValue(Entry *entryPtr, const char *value)
  * 	linked -textvariable, if any.  The write trace on the
  * 	text variable is temporarily disabled; however, other
  * 	write traces may change the value of the variable.
- * 	If so, the new value is used instead (bypassing validation).
+ * 	If so, the widget is updated again with the new value.
  *
  * Returns:
  * 	TCL_OK if successful, TCL_ERROR otherwise.
  */
 static int EntrySetValue(Entry *entryPtr, const char *value)
 {
+    EntryStoreValue(entryPtr, value);
+
     if (entryPtr->entry.textVariableObj) {
 	const char *textVarName =
 	    Tcl_GetString(entryPtr->entry.textVariableObj);
@@ -764,12 +767,16 @@ static int EntrySetValue(Entry *entryPtr, const char *value)
 	    value = Tcl_SetVar(entryPtr->core.interp, textVarName,
 		    value, TCL_GLOBAL_ONLY|TCL_LEAVE_ERR_MSG);
 	    entryPtr->core.flags &= ~SYNCING_VARIABLE;
-	    if (!value || WidgetDestroyed(&entryPtr->core))
+	    if (!value || WidgetDestroyed(&entryPtr->core)) {
 		return TCL_ERROR;
+	    } else if (strcmp(value, entryPtr->entry.string) != 0) {
+		/* Some write trace has changed the variable value.
+		 */
+		EntryStoreValue(entryPtr, value);
+	    }
 	}
     }
 
-    EntryStoreValue(entryPtr, value);
     return TCL_OK;
 }
 
@@ -927,7 +934,7 @@ EntryInitialize(Tcl_Interp *interp, void *recordPtr)
 	entryPtr->core.tkwin, EntryEventMask, EntryEventProc, entryPtr);
     Tk_CreateSelHandler(entryPtr->core.tkwin, XA_PRIMARY, XA_STRING,
 	EntryFetchSelection, (ClientData) entryPtr, XA_STRING);
-    BlinkCursor(&entryPtr->core);
+    TtkBlinkCursor(&entryPtr->core);
 
     entryPtr->entry.string		= ckalloc(1);
     *entryPtr->entry.string 		= '\0';
@@ -938,7 +945,7 @@ EntryInitialize(Tcl_Interp *interp, void *recordPtr)
     EntryInitStyleDefaults(&entryPtr->entry.styleDefaults);
 
     entryPtr->entry.xscrollHandle =
-	CreateScrollHandle(&entryPtr->core, &entryPtr->entry.xscroll);
+	TtkCreateScrollHandle(&entryPtr->core, &entryPtr->entry.xscroll);
 
     entryPtr->entry.insertPos		= 0;
     entryPtr->entry.selectFirst 	= -1;
@@ -955,7 +962,7 @@ EntryCleanup(void *recordPtr)
     if (entryPtr->entry.textVariableTrace)
 	Ttk_UntraceVariable(entryPtr->entry.textVariableTrace);
 
-    FreeScrollHandle(entryPtr->entry.xscrollHandle);
+    TtkFreeScrollHandle(entryPtr->entry.xscrollHandle);
 
     EntryFreeStyleDefaults(&entryPtr->entry.styleDefaults);
 
@@ -984,7 +991,7 @@ static int EntryConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 	}
     }
 
-    if (CoreConfigure(interp, recordPtr, mask) != TCL_OK) {
+    if (TtkCoreConfigure(interp, recordPtr, mask) != TCL_OK) {
 	if (vt) Ttk_UntraceVariable(vt);
 	return TCL_ERROR;
     }
@@ -1006,13 +1013,13 @@ static int EntryConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
     /* Handle -state compatibility option:
      */
     if (mask & STATE_CHANGED) {
-	CheckStateOption(&entryPtr->core, entryPtr->entry.stateObj);
+	TtkCheckStateOption(&entryPtr->core, entryPtr->entry.stateObj);
     }
 
     /* Force scrollbar update if needed:
      */
     if (mask & SCROLLCMD_CHANGED) {
-	ScrollbarUpdateRequired(entryPtr->entry.xscrollHandle);
+	TtkScrollbarUpdateRequired(entryPtr->entry.xscrollHandle);
     }
 
     /* Recompute the displayString, in case showChar changed:
@@ -1139,7 +1146,7 @@ EntryDoLayout(void *recordPtr)
 	entryPtr->entry.layoutX = textarea.x - leftX;
     }
 
-    Scrolled(entryPtr->entry.xscrollHandle,
+    TtkScrolled(entryPtr->entry.xscrollHandle,
 	    leftIndex, rightIndex, entryPtr->entry.numChars);
 }
 
@@ -1164,7 +1171,6 @@ static GC EntryGetGC(Entry *entryPtr, Tcl_Obj *colorObj)
     return Tk_GetGC(entryPtr->core.tkwin, mask, &gcValues);
 }
 
-
 /* EntryDisplay --
  *	Redraws the contents of an entry window.
  */
@@ -1182,7 +1188,7 @@ static void EntryDisplay(void *clientData, Drawable d)
 
     EntryInitStyleData(entryPtr, &es);
 
-    showCursor = 
+    showCursor =
 	   (entryPtr->core.flags & CURSOR_ON) != 0
 	&& EntryEditable(entryPtr)
 	&& entryPtr->entry.insertPos >= leftIndex
@@ -1244,7 +1250,7 @@ static void EntryDisplay(void *clientData, Drawable d)
 	Tk_SetCaretPos(tkwin, cursorX, cursorY, cursorHeight);
 
 	gc = EntryGetGC(entryPtr, es.insertColorObj);
-	XFillRectangle(Tk_Display(tkwin), d, gc, 
+	XFillRectangle(Tk_Display(tkwin), d, gc,
 	    cursorX-cursorWidth/2, cursorY, cursorWidth, cursorHeight);
 	Tk_FreeGC(Tk_Display(tkwin), gc);
     }
@@ -1386,7 +1392,7 @@ EntryBBoxCommand(
     if ((index == entryPtr->entry.numChars) && (index > 0)) {
 	index--;
     }
-    Tk_CharBbox(entryPtr->entry.textLayout, index, 
+    Tk_CharBbox(entryPtr->entry.textLayout, index,
 	    &b.x, &b.y, &b.width, &b.height);
     b.x += entryPtr->entry.layoutX;
     b.y += entryPtr->entry.layoutY;
@@ -1581,7 +1587,7 @@ static int EntrySelectionCommand(
 	{ "range", EntrySelectionRangeCommand },
 	{0,0}
     };
-    return WidgetEnsembleCommand(
+    return TtkWidgetEnsembleCommand(
 	    EntrySelectionCommands, 2, interp, objc, objv, recordPtr);
 }
 
@@ -1630,23 +1636,23 @@ static int EntryXViewCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
 {
     Entry *entryPtr = recordPtr;
-    return ScrollviewCommand(interp, objc, objv, entryPtr->entry.xscrollHandle);
+    return TtkScrollviewCommand(interp, objc, objv, entryPtr->entry.xscrollHandle);
 }
 
 static WidgetCommandSpec EntryCommands[] =
 {
     { "bbox", 		EntryBBoxCommand },
-    { "cget", 		WidgetCgetCommand },
-    { "configure", 	WidgetConfigureCommand },
+    { "cget", 		TtkWidgetCgetCommand },
+    { "configure", 	TtkWidgetConfigureCommand },
     { "delete", 	EntryDeleteCommand },
     { "get", 		EntryGetCommand },
     { "icursor", 	EntryICursorCommand },
-    { "identify",	WidgetIdentifyCommand },
+    { "identify",	TtkWidgetIdentifyCommand },
     { "index", 		EntryIndexCommand },
     { "insert", 	EntryInsertCommand },
-    { "instate",	WidgetInstateCommand },
+    { "instate",	TtkWidgetInstateCommand },
     { "selection", 	EntrySelectionCommand },
-    { "state",  	WidgetStateCommand },
+    { "state",  	TtkWidgetStateCommand },
     { "validate", 	EntryValidateCommand },
     { "xview", 		EntryXViewCommand },
     {0,0}
@@ -1656,7 +1662,7 @@ static WidgetCommandSpec EntryCommands[] =
  * +++ Entry widget definition.
  */
 
-WidgetSpec EntryWidgetSpec =
+static WidgetSpec EntryWidgetSpec =
 {
     "TEntry",			/* className */
     sizeof(Entry), 		/* recordSize */
@@ -1666,8 +1672,8 @@ WidgetSpec EntryWidgetSpec =
     EntryCleanup,		/* cleanupProc */
     EntryConfigure,		/* configureProc */
     EntryPostConfigure,  	/* postConfigureProc */
-    WidgetGetLayout, 		/* getLayoutProc */
-    WidgetSize, 		/* sizeProc */
+    TtkWidgetGetLayout, 	/* getLayoutProc */
+    TtkWidgetSize, 		/* sizeProc */
     EntryDoLayout,		/* layoutProc */
     EntryDisplay		/* displayProc */
 };
@@ -1679,6 +1685,7 @@ WidgetSpec EntryWidgetSpec =
 typedef struct {
     Tcl_Obj	*postCommandObj;
     Tcl_Obj	*valuesObj;
+    Tcl_Obj	*heightObj;
     int 	currentIndex;
 } ComboboxPart;
 
@@ -1690,6 +1697,9 @@ typedef struct {
 
 static Tk_OptionSpec ComboboxOptionSpecs[] =
 {
+    {TK_OPTION_STRING, "-height", "height", "Height",
+        DEF_LIST_HEIGHT, Tk_Offset(Combobox, combobox.heightObj), -1,
+	0,0,0 },
     {TK_OPTION_STRING, "-postcommand", "postCommand", "PostCommand",
         "", Tk_Offset(Combobox, combobox.postCommandObj), -1,
 	0,0,0 },
@@ -1707,7 +1717,7 @@ ComboboxInitialize(Tcl_Interp *interp, void *recordPtr)
 {
     Combobox *cb = recordPtr;
     cb->combobox.currentIndex = -1;
-    TrackElementState(&cb->core);
+    TtkTrackElementState(&cb->core);
     return EntryInitialize(interp, recordPtr);
 }
 
@@ -1732,7 +1742,7 @@ ComboboxConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
  * 	Setting the current index updates the combobox value,
  * 	but the value and -values may be changed independently
  * 	of the index.  Instead of trying to keep currentIndex
- * 	in sync at all times, [$cb current] double-checks 
+ * 	in sync at all times, [$cb current] double-checks
  */
 static int ComboboxCurrentCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], void *recordPtr)
@@ -1748,7 +1758,7 @@ static int ComboboxCurrentCommand(
     if (objc == 2) {
 	/* Check if currentIndex still valid:
 	 */
-	if (    currentIndex < 0 
+	if (    currentIndex < 0
 	     || currentIndex >= nValues
 	     || strcmp(currentValue,Tcl_GetString(values[currentIndex]))
 	   )
@@ -1773,7 +1783,7 @@ static int ComboboxCurrentCommand(
 	    return TCL_ERROR;
 	}
 	if (currentIndex < 0 || currentIndex >= nValues) {
-	    Tcl_AppendResult(interp, 
+	    Tcl_AppendResult(interp,
 		    "Index ", Tcl_GetString(objv[2]), " out of range",
 		    NULL);
 	    return TCL_ERROR;
@@ -1795,24 +1805,24 @@ static int ComboboxCurrentCommand(
 static WidgetCommandSpec ComboboxCommands[] =
 {
     { "bbox", 		EntryBBoxCommand },
-    { "cget", 		WidgetCgetCommand },
-    { "configure", 	WidgetConfigureCommand },
+    { "cget", 		TtkWidgetCgetCommand },
+    { "configure", 	TtkWidgetConfigureCommand },
     { "current", 	ComboboxCurrentCommand },
     { "delete", 	EntryDeleteCommand },
     { "get", 		EntryGetCommand },
     { "icursor", 	EntryICursorCommand },
-    { "identify",	WidgetIdentifyCommand },
+    { "identify",	TtkWidgetIdentifyCommand },
     { "index", 		EntryIndexCommand },
     { "insert", 	EntryInsertCommand },
-    { "instate",	WidgetInstateCommand },
+    { "instate",	TtkWidgetInstateCommand },
     { "selection", 	EntrySelectionCommand },
-    { "state",  	WidgetStateCommand },
+    { "state",  	TtkWidgetStateCommand },
     { "set", 		EntrySetCommand },
     { "xview", 		EntryXViewCommand },
     {0,0}
 };
 
-WidgetSpec ComboboxWidgetSpec =
+static WidgetSpec ComboboxWidgetSpec =
 {
     "TCombobox",		/* className */
     sizeof(Combobox), 		/* recordSize */
@@ -1822,8 +1832,8 @@ WidgetSpec ComboboxWidgetSpec =
     EntryCleanup,		/* cleanupProc */
     ComboboxConfigure,		/* configureProc */
     EntryPostConfigure,  	/* postConfigureProc */
-    WidgetGetLayout, 		/* getLayoutProc */
-    WidgetSize, 		/* sizeProc */
+    TtkWidgetGetLayout, 	/* getLayoutProc */
+    TtkWidgetSize, 		/* sizeProc */
     EntryDoLayout,		/* layoutProc */
     EntryDisplay		/* displayProc */
 };
@@ -1848,7 +1858,7 @@ static Ttk_ElementOptionSpec TextareaElementOptions[] = {
     {0,0,0}
 };
 
-static void TextareaElementGeometry(
+static void TextareaElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -1871,10 +1881,14 @@ static Ttk_ElementSpec TextareaElementSpec = {
     TK_STYLE_VERSION_2,
     sizeof(TextareaElement),
     TextareaElementOptions,
-    TextareaElementGeometry,
-    NullElementDraw
+    TextareaElementSize,
+    TtkNullElementDraw
 };
 
+/*------------------------------------------------------------------------
+ * +++ Widget layouts.
+ */
+
 TTK_BEGIN_LAYOUT(EntryLayout)
     TTK_GROUP("Entry.field", TTK_FILL_BOTH|TTK_BORDER,
 	TTK_GROUP("Entry.padding", TTK_FILL_BOTH,
@@ -1888,10 +1902,11 @@ TTK_BEGIN_LAYOUT(ComboboxLayout)
 	    TTK_NODE("Combobox.textarea", TTK_FILL_BOTH)))
 TTK_END_LAYOUT
 
-/* EntryWidget_Init --
- * 	Register entry-based widgets and related resources.
+/*------------------------------------------------------------------------
+ * +++ Initialization.
  */
-int EntryWidget_Init(Tcl_Interp *interp)
+
+void TtkEntry_Init(Tcl_Interp *interp)
 {
     Ttk_Theme themePtr =  Ttk_GetDefaultTheme(interp);
 
@@ -1902,8 +1917,6 @@ int EntryWidget_Init(Tcl_Interp *interp)
 
     RegisterWidget(interp, "ttk::entry", &EntryWidgetSpec);
     RegisterWidget(interp, "ttk::combobox", &ComboboxWidgetSpec);
-
-    return TCL_OK;
 }
 
 /*EOF*/
