@@ -1,7 +1,7 @@
-/* frame.c,v 1.49 2006/08/04 23:22:50 patthoyts Exp
+/* frame.c,v 1.56 2007/11/25 17:17:09 jenglish Exp
  * Copyright (c) 2004, Joe English
  *
- * Tile widget set: frame and labelframe widgets
+ * ttk::frame and ttk::labelframe widgets.
  */
 
 #include <tk.h>
@@ -14,8 +14,7 @@
  * +++ Frame widget:
  */
 
-typedef struct
-{
+typedef struct {
     Tcl_Obj	*borderWidthObj;
     Tcl_Obj	*paddingObj;
     Tcl_Obj	*reliefObj;
@@ -23,8 +22,7 @@ typedef struct
     Tcl_Obj 	*heightObj;
 } FramePart;
 
-typedef struct 
-{
+typedef struct {
     WidgetCore	core;
     FramePart	frame;
 } Frame;
@@ -47,16 +45,16 @@ static Tk_OptionSpec FrameOptionSpecs[] =
 	Tk_Offset(Frame,frame.heightObj), -1,
 	0,0,GEOMETRY_CHANGED },
 
-    WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
+    WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
 };
 
 static WidgetCommandSpec FrameCommands[] =
 {
-    { "configure",	WidgetConfigureCommand },
-    { "cget",		WidgetCgetCommand },
-    { "instate",	WidgetInstateCommand },
-    { "state",  	WidgetStateCommand },
-    { "identify",   WidgetIdentifyCommand },
+    { "configure",	TtkWidgetConfigureCommand },
+    { "cget",		TtkWidgetCgetCommand },
+    { "instate",	TtkWidgetInstateCommand },
+    { "state",  	TtkWidgetStateCommand },
+    { "identify",   TtkWidgetIdentifyCommand },
     { NULL, NULL }
 };
 
@@ -146,26 +144,29 @@ static int FrameConfigure(Tcl_Interp *interp, void *recordPtr, int mask)
 	Tk_GeometryRequest(framePtr->core.tkwin, width, height);
     }
 
-    return CoreConfigure(interp, recordPtr, mask);
+    return TtkCoreConfigure(interp, recordPtr, mask);
 }
 
-/* public */
-WidgetSpec FrameWidgetSpec =
+static WidgetSpec FrameWidgetSpec =
 {
     "TFrame",			/* className */
     sizeof(Frame),		/* recordSize */
     FrameOptionSpecs,		/* optionSpecs */
     FrameCommands,		/* subcommands */
-    NullInitialize,		/* initializeProc */
-    NullCleanup,		/* cleanupProc */
+    TtkNullInitialize,		/* initializeProc */
+    TtkNullCleanup,		/* cleanupProc */
     FrameConfigure,		/* configureProc */
-    NullPostConfigure,		/* postConfigureProc */
-    WidgetGetLayout, 		/* getLayoutProc */
+    TtkNullPostConfigure,	/* postConfigureProc */
+    TtkWidgetGetLayout, 	/* getLayoutProc */
     FrameSize,			/* sizeProc */
-    WidgetDoLayout,		/* layoutProc */
-    WidgetDisplay		/* displayProc */
+    TtkWidgetDoLayout,		/* layoutProc */
+    TtkWidgetDisplay		/* displayProc */
 };
 
+TTK_BEGIN_LAYOUT(FrameLayout)
+    TTK_NODE("Frame.border", TTK_FILL_BOTH)
+TTK_END_LAYOUT
+
 /* ======================================================================
  * +++ Labelframe widget:
  */
@@ -235,15 +236,14 @@ typedef struct {
     Tcl_Obj 	*labelAnchorObj;
     Tcl_Obj	*textObj;
     Tcl_Obj 	*underlineObj;
-    Tcl_Obj	*labelWidgetObj;
+    Tk_Window	labelWidget;
 
     Ttk_Manager	*mgr;
-    Tk_Window	labelWidget;	/* Set in configureProc */
+    Ttk_Layout	labelLayout;	/* Sublayout for label */
     Ttk_Box	labelParcel;	/* Set in layoutProc */
 } LabelframePart;
 
-typedef struct
-{
+typedef struct {
     WidgetCore  	core;
     FramePart   	frame;
     LabelframePart	label;
@@ -263,7 +263,7 @@ static Tk_OptionSpec LabelframeOptionSpecs[] =
 	"-1", Tk_Offset(Labelframe,label.underlineObj), -1, 
 	0,0,0 },
     {TK_OPTION_WINDOW, "-labelwidget", "labelWidget", "LabelWidget", NULL,
-	Tk_Offset(Labelframe,label.labelWidgetObj), -1,
+	-1, Tk_Offset(Labelframe,label.labelWidget),
 	TK_OPTION_NULL_OK,0,LABELWIDGET_CHANGED|GEOMETRY_CHANGED },
 
     WIDGET_INHERIT_OPTIONS(FrameOptionSpecs)
@@ -272,8 +272,7 @@ static Tk_OptionSpec LabelframeOptionSpecs[] =
 /*
  * Labelframe style parameters:
  */
-typedef struct
-{
+typedef struct {
     int 		borderWidth;	/* border width */
     Ttk_Padding 	padding;	/* internal padding */
     Ttk_PositionSpec	labelAnchor;	/* corner/side to place label */
@@ -289,8 +288,6 @@ static void LabelframeStyleOptions(Labelframe *lf, LabelframeStyle *style)
     style->borderWidth = DEFAULT_BORDERWIDTH;
     style->padding = Ttk_UniformPadding(0);
     style->labelAnchor = TTK_PACK_TOP | TTK_STICK_W;
-    style->labelMargins = 
-	Ttk_MakePadding(DEFAULT_LABELINSET,0,DEFAULT_LABELINSET,0);
     style->labelOutside = 0;
 
     if ((objPtr = Ttk_QueryOption(layout, "-borderwidth", 0)) != NULL) {
@@ -304,6 +301,14 @@ static void LabelframeStyleOptions(Labelframe *lf, LabelframeStyle *style)
     }
     if ((objPtr = Ttk_QueryOption(layout,"-labelmargins", 0)) != NULL) {
 	Ttk_GetBorderFromObj(NULL, objPtr, &style->labelMargins);
+    } else {
+	if (style->labelAnchor & (TTK_PACK_TOP|TTK_PACK_BOTTOM)) {
+	    style->labelMargins = 
+		Ttk_MakePadding(DEFAULT_LABELINSET,0,DEFAULT_LABELINSET,0);
+	} else {
+	    style->labelMargins = 
+		Ttk_MakePadding(0,DEFAULT_LABELINSET,0,DEFAULT_LABELINSET);
+	}
     }
     if ((objPtr = Ttk_QueryOption(layout,"-labeloutside", 0)) != NULL) {
 	Tcl_GetBooleanFromObj(NULL, objPtr, &style->labelOutside);
@@ -319,17 +324,13 @@ static void LabelframeStyleOptions(Labelframe *lf, LabelframeStyle *style)
 static void 
 LabelframeLabelSize(Labelframe *lframePtr, int *widthPtr, int *heightPtr)
 {
-    WidgetCore *corePtr = &lframePtr->core;
     Tk_Window labelWidget = lframePtr->label.labelWidget;
-    Ttk_LayoutNode *textNode = Ttk_LayoutFindNode(corePtr->layout, "text");
 
     if (labelWidget) {
 	*widthPtr = Tk_ReqWidth(labelWidget);
 	*heightPtr = Tk_ReqHeight(labelWidget);
-    } else if (textNode) { 
-	Ttk_LayoutNodeReqSize(corePtr->layout, textNode, widthPtr, heightPtr);
     } else {
-	*widthPtr = *heightPtr = 0;
+	Ttk_LayoutSize(lframePtr->label.labelLayout, 0, widthPtr, heightPtr);
     }
 }
 
@@ -377,6 +378,36 @@ static int LabelframeSize(void *recordPtr, int *widthPtr, int *heightPtr)
     return 0;
 }
 
+/* 
+ * LabelframeGetLayout --
+ * 	Getlayout widget hook.
+ */
+
+static Ttk_Layout LabelframeGetLayout(
+    Tcl_Interp *interp, Ttk_Theme theme, void *recordPtr)
+{
+    Labelframe *lf = recordPtr;
+    Ttk_Layout frameLayout = TtkWidgetGetLayout(interp, theme, recordPtr);
+    Ttk_Layout labelLayout;
+
+    if (!frameLayout) {
+	return NULL;
+    }
+
+    labelLayout = Ttk_CreateSublayout(
+	interp, theme, frameLayout, ".Label", lf->core.optionTable);
+
+    if (labelLayout) {
+	if (lf->label.labelLayout) {
+	    Ttk_FreeLayout(lf->label.labelLayout);
+	}
+	Ttk_RebindSublayout(labelLayout, recordPtr);
+	lf->label.labelLayout = labelLayout;
+    }
+
+    return frameLayout;
+}
+
 /*
  * LabelframeDoLayout --
  * 	Labelframe layout hook.  
@@ -388,19 +419,11 @@ static void LabelframeDoLayout(void *recordPtr)
 {
     Labelframe *lframePtr = recordPtr;
     WidgetCore *corePtr = &lframePtr->core;
-    Ttk_Box borderParcel = Ttk_WinBox(corePtr->tkwin);
-    Ttk_LayoutNode
-	*textNode = Ttk_LayoutFindNode(corePtr->layout, "text"),
-	*borderNode = Ttk_LayoutFindNode(corePtr->layout, "border");
     int lw, lh;			/* Label width and height */
     LabelframeStyle style;
+    Ttk_Box borderParcel = Ttk_WinBox(lframePtr->core.tkwin);
     Ttk_Box labelParcel;
 
-    /*
-     * Do base layout:
-     */
-    Ttk_PlaceLayout(corePtr->layout,corePtr->state,borderParcel);
-
     /*
      * Compute label parcel:
      */
@@ -427,16 +450,22 @@ static void LabelframeDoLayout(void *recordPtr)
     /* 
      * Place border and label:
      */
-    if (borderNode) {
-	Ttk_PlaceLayoutNode(corePtr->layout, borderNode, borderParcel);
-    }
-    if (textNode) {
-	Ttk_PlaceLayoutNode(corePtr->layout, textNode, labelParcel);
-    }
+    Ttk_PlaceLayout(corePtr->layout, corePtr->state, borderParcel);
+    Ttk_PlaceLayout(lframePtr->label.labelLayout, corePtr->state, labelParcel);
     /* labelWidget placed in LabelframePlaceSlaves GM hook */
     lframePtr->label.labelParcel = labelParcel;
 }
 
+static void LabelframeDisplay(void *recordPtr, Drawable d)
+{
+    Labelframe *lframePtr = recordPtr;
+    Ttk_DrawLayout(lframePtr->core.layout, lframePtr->core.state, d);
+    Ttk_DrawLayout(lframePtr->label.labelLayout, lframePtr->core.state, d);
+}
+
+/* +++ Labelframe geometry manager hooks.
+ */
+
 /* LabelframePlaceSlaves --
  * 	Sets the position and size of the labelwidget.  
  */
@@ -453,49 +482,30 @@ static void LabelframePlaceSlaves(void *recordPtr)
     }
 }
 
-/* Labelframe geometry manager:
- */
-static void LabelAdded(Ttk_Manager *mgr, int slaveIndex) { /* no-op */ }
-static void LabelRemoved(Ttk_Manager *mgr, int slaveIndex) { /* no-op */ }
-static int LabelConfigured(
-    Tcl_Interp *interp, Ttk_Manager *mgr, Ttk_Slave *slave, unsigned mask)
-    { return TCL_OK; }
-
-/* LabelframeLostSlave --
- * 	Called when the labelWidget slave is involuntarily lost;
- * 	unset the -labelwidget option.
- * Notes:
- * 	Do this here instead of in the SlaveRemoved hook, 
- * 	since the latter is also called when the widget voluntarily
- * 	forgets the slave.  The latter happens in the ConfigureProc
- * 	at a time when the widget is in an inconsistent state.
- */
-static void LabelframeLostSlave(ClientData clientData, Tk_Window slaveWindow)
+static int LabelRequest(void *managerData, int index, int width, int height)
 {
-    Ttk_Slave *slave = clientData;
-    Labelframe *lframePtr = slave->manager->managerData;
-
-    Tcl_DecrRefCount(lframePtr->label.labelWidgetObj);
-    lframePtr->label.labelWidgetObj = 0;
-    lframePtr->label.labelWidget = 0;
-    Ttk_LostSlaveProc(clientData, slaveWindow);
+    return 1;
 }
 
-static Tk_OptionSpec LabelOptionSpecs[] = {
-    {TK_OPTION_END, 0,0,0, NULL, -1,-1, 0, 0,0}
-};
-
-static Ttk_ManagerSpec LabelframeManagerSpec = 
+/* LabelRemoved --
+ * 	Unset the -labelwidget option.
+ *
+ * <<NOTE-LABELREMOVED>>:
+ * 	This routine is also called when the widget voluntarily forgets
+ * 	the slave in LabelframeConfigure.
+ */
+static void LabelRemoved(void *managerData, int slaveIndex)
 {
-    { "labelframe", Ttk_GeometryRequestProc, LabelframeLostSlave },
-    LabelOptionSpecs, 0,
+    Labelframe *lframe = managerData;
+    lframe->label.labelWidget = 0;
+}
 
+static Ttk_ManagerSpec LabelframeManagerSpec = {
+    { "labelframe", Ttk_GeometryRequestProc, Ttk_LostSlaveProc },
     LabelframeSize,
     LabelframePlaceSlaves,
-
-    LabelAdded,
-    LabelRemoved,
-    LabelConfigured
+    LabelRequest,
+    LabelRemoved
 };
 
 /* LabelframeInitialize --
@@ -508,6 +518,7 @@ static int LabelframeInitialize(Tcl_Interp *interp, void *recordPtr)
     lframe->label.mgr = Ttk_CreateManager(
 	&LabelframeManagerSpec, lframe, lframe->core.tkwin);
     lframe->label.labelWidget = 0;
+    lframe->label.labelLayout = 0;
     lframe->label.labelParcel = Ttk_MakeBox(-1,-1,-1,-1);
 
     return TCL_OK;
@@ -547,22 +558,14 @@ static void RaiseLabelWidget(Labelframe *lframe)
 static int LabelframeConfigure(Tcl_Interp *interp,void *recordPtr,int mask)
 {
     Labelframe *lframePtr = recordPtr;
-    Tk_Window labelWidget = NULL;
+    Tk_Window labelWidget = lframePtr->label.labelWidget;
     Ttk_PositionSpec unused;
 
-    /* Validate -labelwidget option:
+    /* Validate options:
      */
-    if (lframePtr->label.labelWidgetObj) {
-	const char *pathName = Tcl_GetString(lframePtr->label.labelWidgetObj);
-	if (pathName && *pathName) {
-	    labelWidget =
-		Tk_NameToWindow(interp, pathName, lframePtr->core.tkwin);
-	    if (!labelWidget) {
-		return TCL_ERROR;
-	    }
-	    if (!Ttk_Maintainable(interp, labelWidget, lframePtr->core.tkwin)) {
-		return TCL_ERROR;
-	    }
+    if (mask & LABELWIDGET_CHANGED && labelWidget != NULL) {
+	if (!Ttk_Maintainable(interp, labelWidget, lframePtr->core.tkwin)) {
+	    return TCL_ERROR;
 	}
     }
 
@@ -583,12 +586,13 @@ static int LabelframeConfigure(Tcl_Interp *interp,void *recordPtr,int mask)
     if (mask & LABELWIDGET_CHANGED) {
 	if (Ttk_NumberSlaves(lframePtr->label.mgr) == 1) {
 	    Ttk_ForgetSlave(lframePtr->label.mgr, 0);
+	    /* Restore labelWidget field (see <<NOTE-LABELREMOVED>>)
+	     */
+	    lframePtr->label.labelWidget = labelWidget;
 	}
 
-	lframePtr->label.labelWidget = labelWidget;
-
 	if (labelWidget) {
-	    Ttk_AddSlave(interp, lframePtr->label.mgr, labelWidget, 0,  0,0);
+	    Ttk_InsertSlave(lframePtr->label.mgr, 0, labelWidget, NULL);
 	    RaiseLabelWidget(lframePtr);
 	}
     }
@@ -601,8 +605,7 @@ static int LabelframeConfigure(Tcl_Interp *interp,void *recordPtr,int mask)
     return TCL_OK;
 }
 
-/* public */
-WidgetSpec LabelframeWidgetSpec =
+static WidgetSpec LabelframeWidgetSpec =
 {
     "TLabelframe",		/* className */
     sizeof(Labelframe),		/* recordSize */
@@ -611,10 +614,35 @@ WidgetSpec LabelframeWidgetSpec =
     LabelframeInitialize,	/* initializeProc */
     LabelframeCleanup,		/* cleanupProc */
     LabelframeConfigure,	/* configureProc */
-    NullPostConfigure,  	/* postConfigureProc */
-    WidgetGetLayout, 		/* getLayoutProc */
+    TtkNullPostConfigure,  	/* postConfigureProc */
+    LabelframeGetLayout, 	/* getLayoutProc */
     LabelframeSize,		/* sizeProc */
     LabelframeDoLayout,		/* layoutProc */
-    WidgetDisplay		/* displayProc */
+    LabelframeDisplay		/* displayProc */
 };
 
+TTK_BEGIN_LAYOUT(LabelframeLayout)
+    TTK_NODE("Labelframe.border", TTK_FILL_BOTH)
+TTK_END_LAYOUT
+
+TTK_BEGIN_LAYOUT(LabelSublayout)
+    TTK_GROUP("Label.fill", TTK_FILL_BOTH,
+	TTK_NODE("Label.text", TTK_FILL_BOTH))
+TTK_END_LAYOUT
+
+/* ======================================================================
+ * +++ Initialization.
+ */
+
+void TtkFrame_Init(Tcl_Interp *interp)
+{
+    Ttk_Theme theme =  Ttk_GetDefaultTheme(interp);
+
+    Ttk_RegisterLayout(theme, "TFrame", FrameLayout);
+    Ttk_RegisterLayout(theme, "TLabelframe", LabelframeLayout);
+    Ttk_RegisterLayout(theme, "TLabelframe.Label", LabelSublayout);
+
+    RegisterWidget(interp, "ttk::frame", &FrameWidgetSpec);
+    RegisterWidget(interp, "ttk::labelframe", &LabelframeWidgetSpec);
+}
+
