@@ -1,6 +1,6 @@
-/* label.c,v 1.16 2006/04/17 01:53:13 jenglish Exp
+/* label.c,v 1.22 2007/11/19 01:57:21 jenglish Exp
  *
- * Tile widget set: text, image, and label elements.
+ * text, image, and label elements.
  *
  * The label element combines text and image elements,
  * with layout determined by the "-compound" option.
@@ -11,8 +11,7 @@
 #include <tk.h>
 #include "tkTheme.h"
 
-/*
- *----------------------------------------------------------------------
+/*----------------------------------------------------------------------
  * +++ Text element.
  *
  * This element displays a textual label in the foreground color.
@@ -21,15 +20,13 @@
  * is present and >= 0.
  */
 
-typedef struct
-{
+typedef struct {
     /*
      * Element options:
      */
     Tcl_Obj	*textObj;
     Tcl_Obj	*fontObj;
     Tcl_Obj	*foregroundObj;
-    Tcl_Obj	*backgroundObj;
     Tcl_Obj	*underlineObj;
     Tcl_Obj	*widthObj;
     Tcl_Obj	*anchorObj;
@@ -51,22 +48,19 @@ typedef struct
 /* Text element options table.
  * NB: Keep in sync with label element option table.
  */
-static Ttk_ElementOptionSpec TextElementOptions[] =
-{
+static Ttk_ElementOptionSpec TextElementOptions[] = {
     { "-text", TK_OPTION_STRING,
 	Tk_Offset(TextElement,textObj), "" },
     { "-font", TK_OPTION_FONT,
 	Tk_Offset(TextElement,fontObj), DEFAULT_FONT },
     { "-foreground", TK_OPTION_COLOR,
 	Tk_Offset(TextElement,foregroundObj), "black" },
-    { "-background", TK_OPTION_BORDER,
-	Tk_Offset(TextElement,backgroundObj), DEFAULT_BACKGROUND },
     { "-underline", TK_OPTION_INT,
 	Tk_Offset(TextElement,underlineObj), "-1"},
     { "-width", TK_OPTION_INT,
 	Tk_Offset(TextElement,widthObj), "-1"},
     { "-anchor", TK_OPTION_ANCHOR,
-	Tk_Offset(TextElement,anchorObj), "center"},
+	Tk_Offset(TextElement,anchorObj), "w"},
     { "-justify", TK_OPTION_JUSTIFY,
 	Tk_Offset(TextElement,justifyObj), "left" },
     { "-wraplength", TK_OPTION_PIXELS,
@@ -209,8 +203,7 @@ static void TextElementDraw(
     }
 }
 
-/*public*/ Ttk_ElementSpec TextElementSpec =
-{
+static Ttk_ElementSpec TextElementSpec = {
     TK_STYLE_VERSION_2,
     sizeof(TextElement),
     TextElementOptions,
@@ -218,63 +211,25 @@ static void TextElementDraw(
     TextElementDraw
 };
 
-/*
- * ImageTextElement --
- * 	Same as TextElement, but erases the background area first.
- */
-static void ImageTextElementDraw(
-    void *clientData, void *elementRecord, Tk_Window tkwin,
-    Drawable d, Ttk_Box b, Ttk_State state)
-{
-    TextElement *text = elementRecord;
-    Tk_3DBorder bd = Tk_Get3DBorderFromObj(tkwin,text->backgroundObj);
-
-    if (!TextSetup(text, tkwin))
-	return;
-
-    XFillRectangle(Tk_Display(tkwin), d,
-	Tk_3DBorderGC(tkwin, bd, TK_3D_FLAT_GC), b.x, b.y, b.width, b.height);
-
-    TextDraw(text, tkwin, d, b);
-    TextCleanup(text);
-}
-
-/*public*/ Ttk_ElementSpec ImageTextElementSpec =
-{
-    TK_STYLE_VERSION_2,
-    sizeof(TextElement),
-    TextElementOptions,
-    TextElementSize,
-    ImageTextElementDraw
-};
-
-/*
- *----------------------------------------------------------------------
+/*----------------------------------------------------------------------
  * +++ Image element.
- *
  * Draws an image.
- *
- * The clientData parameter is a Tcl_Interp, which is needed for
- * the call to Tk_GetImage().
  */
 
-typedef struct
-{
+typedef struct {
     Tcl_Obj	*imageObj;
-
     Tcl_Obj 	*stippleObj;	/* For TTK_STATE_DISABLED */
     Tcl_Obj 	*backgroundObj;	/* " " */
 
+    Ttk_ImageSpec *imageSpec;
     Tk_Image	tkimg;
     int 	width;
     int		height;
-    int		doStipple;
 } ImageElement;
 
 /* ===> NB: Keep in sync with label element option table.  <===
  */
-static Ttk_ElementOptionSpec ImageElementOptions[] =
-{
+static Ttk_ElementOptionSpec ImageElementOptions[] = {
     { "-image", TK_OPTION_STRING,
 	Tk_Offset(ImageElement,imageObj), "" },
     { "-stipple", TK_OPTION_STRING, 	/* Really: TK_OPTION_BITMAP */
@@ -284,13 +239,6 @@ static Ttk_ElementOptionSpec ImageElementOptions[] =
     {NULL}
 };
 
-/* NullImageChanged --
- * 	No-op Tk_ImageChangedProc for Tk_GetImage.
- */
-static void NullImageChanged(ClientData clientData,
-    int x, int y, int width, int height, int imageWidth, int imageHeight)
-{ }
-
 /*
  * ImageSetup() --
  * 	Look up the Tk_Image from the image element's imageObj resource.
@@ -302,53 +250,21 @@ static void NullImageChanged(ClientData clientData,
  */
 
 static int ImageSetup(
-    ImageElement *image, Tk_Window tkwin, Tcl_Interp *interp, Ttk_State state)
+    ImageElement *image, Tk_Window tkwin, Ttk_State state)
 {
-    const char *imageName;
-    Tcl_Obj *imageObj = image->imageObj;
-    Tcl_Obj **mapList = NULL;
-    int i, mapCnt = 0;
 
-    if (!imageObj)		/* No -image option specified */
+    if (!image->imageObj) {
 	return 0;
-
-    if (Tcl_ListObjGetElements(interp,imageObj,&mapCnt,&mapList) == TCL_ERROR)
-	return 0;
-
-    if (mapCnt == 0)		/* -image is an empty list */
-	return 0;
-
-    /* Only enable disabled-stippling if there's no state map:
-     * @@@ Possibly: Don't do disabled-stippling at all;
-     * @@@ it's ugly and out of fashion.
-     */
-    image->doStipple = mapCnt == 1;
-
-    /* Locate which image to use based on current state:
-     */
-    imageObj = mapList[0];
-    for (i = 1; i < mapCnt - 1; i += 2) {
-	Ttk_StateSpec stateSpec;
-
-	if (Ttk_GetStateSpecFromObj(interp,mapList[i],&stateSpec) != TCL_OK) {
-	    /* shouldn't happen, but can */
-	    break;
-	}
-
-	if (Ttk_StateMatches(state, &stateSpec)) {
-	    imageObj = mapList[i+1];
-	    break;
-	}
     }
-
-    imageName = Tcl_GetString(imageObj);
-    if (!imageName || !*imageName)	/* Empty string. */
+    image->imageSpec = TtkGetImageSpec(NULL, tkwin, image->imageObj);
+    if (!image->imageSpec) {
 	return 0;
-
-    image->tkimg = Tk_GetImage(interp, tkwin, imageName, NullImageChanged, 0);
-    if (!image->tkimg)			/* No such image */
+    }
+    image->tkimg = TtkSelectImage(image->imageSpec, state);
+    if (!image->tkimg) {
+	TtkFreeImageSpec(image->imageSpec);
 	return 0;
-
+    }
     Tk_SizeOfImage(image->tkimg, &image->width, &image->height);
 
     return 1;
@@ -356,7 +272,7 @@ static int ImageSetup(
 
 static void ImageCleanup(ImageElement *image)
 {
-    Tk_FreeImage(image->tkimg);
+    TtkFreeImageSpec(image->imageSpec);
 }
 
 /*
@@ -400,8 +316,15 @@ static void ImageDraw(
 
     Tk_RedrawImage(image->tkimg, 0,0, width, height, d, b.x, b.y);
 
-    if (image->doStipple && (state & TTK_STATE_DISABLED)) {
-	StippleOver(image, tkwin, d, b.x,b.y);
+    /* If we're disabled there's no state-specific 'disabled' image, 
+     * stipple the image.
+     * @@@ Possibly: Don't do disabled-stippling at all;
+     * @@@ it's ugly and out of fashion.
+     */
+    if (state & TTK_STATE_DISABLED) {
+	if (TtkSelectImage(image->imageSpec, 0ul) == image->tkimg) {
+	    StippleOver(image, tkwin, d, b.x,b.y);
+	}
     }
 }
 
@@ -410,9 +333,8 @@ static void ImageElementSize(
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     ImageElement *image = elementRecord;
-    Tcl_Interp *interp = clientData;
 
-    if (ImageSetup(image, tkwin, interp, 0)) {
+    if (ImageSetup(image, tkwin, 0)) {
 	*widthPtr = image->width;
 	*heightPtr = image->height;
 	ImageCleanup(image);
@@ -424,16 +346,14 @@ static void ImageElementDraw(
     Drawable d, Ttk_Box b, Ttk_State state)
 {
     ImageElement *image = elementRecord;
-    Tcl_Interp *interp = clientData;
 
-    if (ImageSetup(image, tkwin, interp, state)) {
+    if (ImageSetup(image, tkwin, state)) {
 	ImageDraw(image, tkwin, d, b, state);
 	ImageCleanup(image);
     }
 }
 
-/*public*/ Ttk_ElementSpec ImageElementSpec =
-{
+static Ttk_ElementSpec ImageElementSpec = {
     TK_STYLE_VERSION_2,
     sizeof(ImageElement),
     ImageElementOptions,
@@ -446,9 +366,6 @@ static void ImageElementDraw(
  *
  * Displays an image and/or text, as determined by the -compound option.
  *
- * The clientData parameter is a Tcl_Interp; this is needed for the
- * image part.
- *
  * Differences from Tk 8.4 compound elements:
  *
  * This adds two new values for the -compound option, "text"
@@ -475,8 +392,7 @@ static void ImageElementDraw(
  * Here, padding is handled by a different element.
  */
 
-typedef struct
-{
+typedef struct {
     /*
      * Element options:
      */
@@ -493,8 +409,7 @@ typedef struct
     int 		totalWidth, totalHeight;
 } LabelElement;
 
-static Ttk_ElementOptionSpec LabelElementOptions[] =
-{
+static Ttk_ElementOptionSpec LabelElementOptions[] = {
     { "-compound", TK_OPTION_ANY,
 	Tk_Offset(LabelElement,compoundObj), "none" },
     { "-space", TK_OPTION_PIXELS,
@@ -509,14 +424,12 @@ static Ttk_ElementOptionSpec LabelElementOptions[] =
 	Tk_Offset(LabelElement,text.fontObj), DEFAULT_FONT },
     { "-foreground", TK_OPTION_COLOR,
 	Tk_Offset(LabelElement,text.foregroundObj), "black" },
-    { "-background", TK_OPTION_BORDER,
-	Tk_Offset(LabelElement,text.backgroundObj), DEFAULT_BACKGROUND },
     { "-underline", TK_OPTION_INT,
 	Tk_Offset(LabelElement,text.underlineObj), "-1"},
     { "-width", TK_OPTION_INT,
 	Tk_Offset(LabelElement,text.widthObj), ""},
     { "-anchor", TK_OPTION_ANCHOR,
-	Tk_Offset(LabelElement,text.anchorObj), "center"},
+	Tk_Offset(LabelElement,text.anchorObj), "w"},
     { "-justify", TK_OPTION_JUSTIFY,
 	Tk_Offset(LabelElement,text.justifyObj), "left" },
     { "-wraplength", TK_OPTION_PIXELS,
@@ -546,7 +459,7 @@ static Ttk_ElementOptionSpec LabelElementOptions[] =
 
 #define MAX(a,b) ((a) > (b) ? a : b);
 static void LabelSetup(
-    LabelElement *c, Tk_Window tkwin, Tcl_Interp *interp, Ttk_State state)
+    LabelElement *c, Tk_Window tkwin, Ttk_State state)
 {
     Tk_GetPixelsFromObj(NULL,tkwin,c->spaceObj,&c->space);
     Ttk_GetCompoundFromObj(NULL,c->compoundObj,(int*)&c->compound);
@@ -555,13 +468,13 @@ static void LabelSetup(
      * Deal with TTK_COMPOUND_NONE.
      */
     if (c->compound == TTK_COMPOUND_NONE) {
-	if (ImageSetup(&c->image, tkwin, interp, state)) {
+	if (ImageSetup(&c->image, tkwin, state)) {
 	    c->compound = TTK_COMPOUND_IMAGE;
 	} else {
 	    c->compound = TTK_COMPOUND_TEXT;
 	}
     } else if (c->compound != TTK_COMPOUND_TEXT) {
-    	if (!ImageSetup(&c->image, tkwin, interp, state)) {
+    	if (!ImageSetup(&c->image, tkwin, state)) {
 	    c->compound = TTK_COMPOUND_TEXT;
 	}
     }
@@ -619,10 +532,9 @@ static void LabelElementSize(
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     LabelElement *label = elementRecord;
-    Tcl_Interp *interp = clientData;
     int textReqWidth = 0;
 
-    LabelSetup(label, tkwin, interp, 0);
+    LabelSetup(label, tkwin, 0);
 
     *heightPtr = label->totalHeight;
 
@@ -677,10 +589,9 @@ static void LabelElementDraw(
     Drawable d, Ttk_Box b, Ttk_State state)
 {
     LabelElement *l = elementRecord;
-    Tcl_Interp *interp = clientData;
     Tk_Anchor anchor = TK_ANCHOR_CENTER;
 
-    LabelSetup(l, tkwin, interp, state);
+    LabelSetup(l, tkwin, state);
 
     /*
      * Adjust overall parcel based on -anchor:
@@ -729,8 +640,7 @@ static void LabelElementDraw(
     LabelCleanup(l);
 }
 
-/*public*/ Ttk_ElementSpec LabelElementSpec =
-{
+static Ttk_ElementSpec LabelElementSpec = {
     TK_STYLE_VERSION_2,
     sizeof(LabelElement),
     LabelElementOptions,
@@ -738,3 +648,16 @@ static void LabelElementDraw(
     LabelElementDraw
 };
 
+/*------------------------------------------------------------------------
+ * +++ Initialization.
+ */
+
+void TtkLabel_Init(Tcl_Interp *interp)
+{
+    Ttk_Theme theme =  Ttk_GetDefaultTheme(interp);
+
+    Ttk_RegisterElement(interp, theme, "text", &TextElementSpec, NULL);
+    Ttk_RegisterElement(interp, theme, "image", &ImageElementSpec, NULL);
+    Ttk_RegisterElement(interp, theme, "label", &LabelElementSpec, NULL);
+}
+
