@@ -1,11 +1,11 @@
 /*
- * layout.c --
+ * ttkLayout.c --
  *
  * Generic layout processing.
  *
  * Copyright (c) 2003 Joe English.  Freely redistributable.
  *
- * layout.c,v 1.70 2006/09/12 22:02:18 jenglish Exp
+ * layout.c,v 1.76 2007/12/02 04:34:30 jenglish Exp
  */
 
 #include <string.h>
@@ -277,6 +277,7 @@ static void TTKInitPadding(int padc, int pixels[4], Ttk_Padding *pad)
 {
     switch (padc)
     {
+	case 0: pixels[0] = 0; /*FALLTHRU*/
 	case 1:	pixels[1] = pixels[0]; /*FALLTHRU*/
 	case 2:	pixels[2] = pixels[0]; /*FALLTHRU*/
 	case 3:	pixels[3] = pixels[1]; /*FALLTHRU*/
@@ -513,13 +514,13 @@ Tcl_Obj *Ttk_NewStickyObj(Ttk_Sticky sticky)
 struct Ttk_LayoutNode_
 {
     unsigned		flags;		/* Packing and sticky flags */
-    Ttk_Element 	element;	/* Element implementation */
+    Ttk_ElementImpl 	element;	/* Element implementation */
     Ttk_State 	 	state;		/* Current state */
     Ttk_Box 		parcel;		/* allocated parcel */
     Ttk_LayoutNode	*next, *child;
 };
 
-static Ttk_LayoutNode *Ttk_NewLayoutNode(unsigned flags, Ttk_Element element)
+static Ttk_LayoutNode *Ttk_NewLayoutNode(unsigned flags, Ttk_ElementImpl element)
 {
     Ttk_LayoutNode *node = (Ttk_LayoutNode*)ckalloc(sizeof(Ttk_LayoutNode));
 
@@ -578,7 +579,7 @@ void Ttk_FreeLayoutTemplate(Ttk_LayoutTemplate op)
 static Ttk_LayoutNode *
 Ttk_InstantiateLayout(Ttk_Theme theme, Ttk_TemplateNode *op)
 {
-    Ttk_Element elementImpl = Ttk_GetElement(theme, op->name);
+    Ttk_ElementImpl elementImpl = Ttk_GetElement(theme, op->name);
     Ttk_LayoutNode *node = Ttk_NewLayoutNode(op->flags, elementImpl);
 
     if (op->next) {
@@ -718,7 +719,7 @@ Ttk_LayoutTemplate Ttk_BuildLayoutTemplate(Ttk_LayoutSpec spec)
 {
     Ttk_TemplateNode *first = 0, *last = 0;
 
-    for ( ; !(spec->opcode & TTK_LAYOUT_END) ; ++spec) {
+    for ( ; !(spec->opcode & _TTK_LAYOUT_END) ; ++spec) {
 	if (spec->elementName) {
 	    Ttk_TemplateNode *node =
 		Ttk_NewTemplateNode(spec->elementName, spec->opcode);
@@ -731,17 +732,39 @@ Ttk_LayoutTemplate Ttk_BuildLayoutTemplate(Ttk_LayoutSpec spec)
 	    last = node;
 	}
 
-	if (spec->opcode & TTK_CHILDREN) {
+	if (spec->opcode & _TTK_CHILDREN) {
+	    int depth = 1;
 	    last->child = Ttk_BuildLayoutTemplate(spec+1);
-	    while (!(spec->opcode & TTK_LAYOUT_END)) {
+
+	    /* Skip to end of group:
+	     */
+	    while (depth) {
 		++spec;
+		if (spec->opcode & _TTK_CHILDREN) {
+		    ++depth;
+		}
+		if (spec->opcode & _TTK_LAYOUT_END) {
+		    --depth;
+		}
 	    }
 	}
+
     } /* for */
 
     return first;
 }
 
+void Ttk_RegisterLayouts(Ttk_Theme theme, Ttk_LayoutSpec spec)
+{
+    while (!(spec->opcode & _TTK_LAYOUT_END)) {
+	Ttk_LayoutTemplate layoutTemplate = Ttk_BuildLayoutTemplate(spec+1);
+	Ttk_RegisterLayoutTemplate(theme, spec->elementName, layoutTemplate);
+	do {
+	    ++spec;
+	} while (!(spec->opcode & _TTK_LAYOUT));
+    }
+}
+
 Tcl_Obj *Ttk_UnparseLayoutTemplate(Ttk_TemplateNode *node) 
 {
     Tcl_Obj *result = Tcl_NewListObj(0,0);
@@ -846,7 +869,7 @@ Ttk_Layout Ttk_CreateLayout(
     Ttk_Style style = Ttk_GetStyle(themePtr, styleName);
     Ttk_LayoutTemplate layoutTemplate =
 	Ttk_FindLayoutTemplate(themePtr,styleName);
-    Ttk_Element bgelement = Ttk_GetElement(themePtr, "background");
+    Ttk_ElementImpl bgelement = Ttk_GetElement(themePtr, "background");
     Ttk_LayoutNode *bgnode;
 
     if (!layoutTemplate) {
@@ -1175,11 +1198,6 @@ Ttk_Box Ttk_LayoutNodeParcel(Ttk_LayoutNode *node)
     return node->parcel;
 }
 
-void Ttk_LayoutNodeSetParcel(Ttk_LayoutNode *node, Ttk_Box parcel)
-{
-    node->parcel = parcel;
-}
-
 void Ttk_PlaceLayoutNode(Ttk_Layout layout, Ttk_LayoutNode *node, Ttk_Box b)
 {
     node->parcel = b;
