@@ -1,8 +1,8 @@
-/* paned.c,v 1.13 2006/04/06 19:34:11 jenglish Exp
+/* paned.c,v 1.21 2007/11/25 17:17:10 jenglish Exp
  *
  * Copyright (c) 2005, Joe English.  Freely redistributable.
  *
- * Tile Widget Set: ttk::paned widget.
+ * ttk::panedwindow widget implementation.
  *
  * TODO: track active/pressed sash.
  */
@@ -26,7 +26,7 @@
  *
  * Pane geometry is determined by the sash positions.
  * When resizing, sash positions are computed from the request sizes,
- * the available space, and pane weights (see ComputePositions()).
+ * the available space, and pane weights (see PlaceSashes()).
  * This ensures continuous resize behavior (that is: changing
  * the size by X pixels then changing the size by Y pixels
  * gives the same result as changing the size by X+Y pixels
@@ -61,9 +61,12 @@
  */
 
 typedef struct {
-    Tcl_Obj 	*orientObj;
+    Tcl_Obj	*orientObj;
     int 	orient;
+    int 	width;
+    int 	height;
     Ttk_Manager	*mgr;
+    Tk_OptionTable paneOptionTable;
     Ttk_Layout	sashLayout;
     int 	sashThickness;
 } PanedPart;
@@ -78,9 +81,15 @@ typedef struct {
 static Tk_OptionSpec PanedOptionSpecs[] = {
     {TK_OPTION_STRING_TABLE, "-orient", "orient", "Orient", "vertical",
 	Tk_Offset(Paned,paned.orientObj), Tk_Offset(Paned,paned.orient),
-	0,(ClientData)TTKOrientStrings,READONLY_OPTION|STYLE_CHANGED },
+	0,(ClientData)ttkOrientStrings,READONLY_OPTION|STYLE_CHANGED },
+    {TK_OPTION_INT, "-width", "width", "Width", "0",
+	-1,Tk_Offset(Paned,paned.width),
+	0,0,GEOMETRY_CHANGED },
+    {TK_OPTION_INT, "-height", "height", "Height", "0",
+	-1,Tk_Offset(Paned,paned.height),
+	0,0,GEOMETRY_CHANGED },
 
-    WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
+    WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
 };
 
 /*------------------------------------------------------------------------
@@ -98,8 +107,76 @@ static Tk_OptionSpec PaneOptionSpecs[] = {
     {TK_OPTION_END, 0,0,0, NULL, -1,-1, 0,0,0}
 };
 
+/* CreatePane --
+ * 	Create a new pane record.
+ */
+static Pane *CreatePane(Tcl_Interp *interp, Paned *pw, Tk_Window slaveWindow)
+{
+    Tk_OptionTable optionTable = pw->paned.paneOptionTable;
+    void *record = ckalloc(sizeof(Pane));
+    Pane *pane = record;
+
+    memset(record, 0, sizeof(Pane));
+    if (Tk_InitOptions(interp, record, optionTable, slaveWindow) != TCL_OK) {
+	ckfree(record);
+	return NULL;
+    }
+
+    pane->reqSize
+	= pw->paned.orient == TTK_ORIENT_HORIZONTAL
+	? Tk_ReqWidth(slaveWindow) : Tk_ReqHeight(slaveWindow);
+
+    return pane;
+}
+
+/* DestroyPane --
+ * 	Free pane record.
+ */
+static void DestroyPane(Paned *pw, Pane *pane)
+{
+    void *record = pane;
+    Tk_FreeConfigOptions(record, pw->paned.paneOptionTable, pw->core.tkwin);
+    ckfree(record);
+}
+
+/* ConfigurePane --
+ * 	Set pane options.
+ */
+static int ConfigurePane(
+    Tcl_Interp *interp, Paned *pw, Pane *pane, Tk_Window slaveWindow,
+    int objc, Tcl_Obj *CONST objv[])
+{
+    Ttk_Manager *mgr = pw->paned.mgr;
+    Tk_SavedOptions savedOptions;
+    int mask = 0;
+
+    if (Tk_SetOptions(interp, (void*)pane, pw->paned.paneOptionTable,
+	    objc, objv, slaveWindow, &savedOptions, &mask) != TCL_OK)
+    {
+	return TCL_ERROR;
+    }
+
+    /* Sanity-check:
+     */
+    if (pane->weight < 0) {
+	Tcl_AppendResult(interp, "-weight must be nonnegative", NULL);
+	goto error;
+    }
+
+    /* Done.
+     */
+    Tk_FreeSavedOptions(&savedOptions);
+    Ttk_ManagerSizeChanged(mgr);
+    return TCL_OK;
+
+error:
+    Tk_RestoreSavedOptions(&savedOptions);
+    return TCL_ERROR;
+}
+
+
 /*------------------------------------------------------------------------
- * +++ Layout algorithm.
+ * +++ Sash adjustment.
  */
 
 /* ShoveUp --
@@ -135,7 +212,7 @@ static int ShoveDown(Paned *pw, int i, int pos)
     Pane *pane = Ttk_SlaveData(pw->paned.mgr,i);
     int sashThickness = pw->paned.sashThickness;
 
-    if (i == pw->paned.mgr->nSlaves - 1) {
+    if (i == Ttk_NumberSlaves(pw->paned.mgr) - 1) {
 	pos = pane->sashPos; /* Sentinel value == master window size */
     } else {
 	Pane *nextPane = Ttk_SlaveData(pw->paned.mgr,i+1);
@@ -146,8 +223,10 @@ static int ShoveDown(Paned *pw, int i, int pos)
 }
 
 /* PanedSize --
- * 	Compute the requested size of the paned widget.
- * 	Used as both the WidgetSpec sizeProc and the ManagerSpec sizeProc.
+ * 	Compute the requested size of the paned widget
+ * 	from the individual pane request sizes.
+ *
+ * 	Used as the WidgetSpec sizeProc and the ManagerSpec sizeProc.
  */
 static int PanedSize(void *recordPtr, int *widthPtr, int *heightPtr)
 {
@@ -180,8 +259,8 @@ static int PanedSize(void *recordPtr, int *widthPtr, int *heightPtr)
 	height += nSashes * sashThickness;
     }
 
-    *widthPtr = width;
-    *heightPtr = height;
+    *widthPtr = pw->paned.width > 0 ? pw->paned.width : width;
+    *heightPtr = pw->paned.height > 0 ? pw->paned.height : height;
     return 1;
 }
 
@@ -189,7 +268,7 @@ static int PanedSize(void *recordPtr, int *widthPtr, int *heightPtr)
  * 	Set pane request sizes from sash positions.
  *
  * NOTE:
- * 	AdjustPanes followed by ComputePositions (called during relayout)
+ * 	AdjustPanes followed by PlaceSashes (called during relayout)
  * 	will leave the sashes in the same place, as long as available size
  * 	remains contant.
  */
@@ -207,8 +286,9 @@ static void AdjustPanes(Paned *pw)
     }
 }
 
-/* ComputePositions --
+/* PlaceSashes --
  *	Set sash positions from pane request sizes and available space.
+ *	The sentinel sash position is set to the available space.
  *
  *	Allocate pane->reqSize pixels to each pane, and distribute
  *	the difference = available size - requested size according
@@ -222,14 +302,12 @@ static void AdjustPanes(Paned *pw)
  * 	This doesn't distribute the remainder pixels as evenly as it could
  * 	when more than one pane has weight > 1.
  */
-static void ComputePositions(Paned *pw)
+static void PlaceSashes(Paned *pw, int width, int height)
 {
     Ttk_Manager *mgr = pw->paned.mgr;
     int nPanes = Ttk_NumberSlaves(mgr);
     int sashThickness = pw->paned.sashThickness;
-    int available
-	= pw->paned.orient == TTK_ORIENT_HORIZONTAL
-	? Tk_Width(pw->core.tkwin) : Tk_Height(pw->core.tkwin);
+    int available = pw->paned.orient == TTK_ORIENT_HORIZONTAL ? width : height;
     int reqSize = 0, totalWeight = 0;
     int difference, delta, remainder, pos, i;
 
@@ -322,74 +400,77 @@ static void PlacePanes(Paned *pw)
 static void PanedPlaceSlaves(void *managerData)
 {
     Paned *pw = managerData;
-    ComputePositions(pw);
+    PlaceSashes(pw, Tk_Width(pw->core.tkwin), Tk_Height(pw->core.tkwin));
     PlacePanes(pw);
 }
 
-static void PaneAdded(Ttk_Manager *mgr, int index)
+static void PaneRemoved(void *managerData, int index)
 {
-    Pane *pane = Ttk_SlaveData(mgr, index);
-    Tk_Window slaveWindow = Ttk_SlaveWindow(mgr, index);
-    Paned *pw = mgr->managerData;
-
-    /* See also: PanedGeometryRequestProc */
-    pane->reqSize
-	= pw->paned.orient == TTK_ORIENT_HORIZONTAL
-	? Tk_ReqWidth(slaveWindow) : Tk_ReqHeight(slaveWindow);
+    Paned *pw = managerData;
+    Pane *pane = Ttk_SlaveData(pw->paned.mgr, index);
+    DestroyPane(pw, pane);
 }
 
-static void PaneRemoved(Ttk_Manager *mgr, int i) { /*no-op*/ }
-
-static int PaneConfigured(
-    Tcl_Interp *interp, Ttk_Manager *mgr, Ttk_Slave *slave, unsigned mask)
+static int AddPane(
+    Tcl_Interp *interp, Paned *pw,
+    int destIndex, Tk_Window slaveWindow,
+    int objc, Tcl_Obj *const objv[])
 {
-    Pane *pane = slave->slaveData;
-    if (pane->weight < 0) {
-	Tcl_AppendResult(interp, "-weight must be nonnegative", NULL);
-	pane->weight = 0;
+    Pane *pane;
+    if (!Ttk_Maintainable(interp, slaveWindow, pw->core.tkwin)) {
 	return TCL_ERROR;
     }
+    if (Ttk_SlaveIndex(pw->paned.mgr, slaveWindow) >= 0) {
+	Tcl_AppendResult(interp,
+	    Tk_PathName(slaveWindow), " already added",
+	    NULL);
+	return TCL_ERROR;
+    }
+
+    pane = CreatePane(interp, pw, slaveWindow);
+    if (!pane) {
+	return TCL_ERROR;
+    }
+    if (ConfigurePane(interp, pw, pane, slaveWindow, objc, objv) != TCL_OK) {
+	DestroyPane(pw, pane);
+	return TCL_ERROR;
+    }
+
+    Ttk_InsertSlave(pw->paned.mgr, destIndex, slaveWindow, pane);
     return TCL_OK;
 }
 
-/* PanedGeometryRequestProc --
- * 	Update pane request size, but only if slave is currently unmapped.
- * 	Geometry requests from mapped slaves are not directly honored,
+/* PaneRequest --
+ * 	Only update pane request size if slave is currently unmapped.
+ * 	Geometry requests from mapped slaves are not directly honored
  * 	in order to avoid unexpected pane resizes (esp. while the
  * 	user is dragging a sash [#1325286]).
  */
-
-static void PanedGeometryRequestProc(
-    ClientData clientData, Tk_Window slaveWindow)
+static int PaneRequest(void *managerData, int index, int width, int height)
 {
-    Ttk_Slave *slave = clientData;
-    Pane *pane = slave->slaveData;
-    Paned *pw = slave->manager->managerData;
+    Paned *pw = managerData;
+    Pane *pane = Ttk_SlaveData(pw->paned.mgr, index);
+    Tk_Window slaveWindow = Ttk_SlaveWindow(pw->paned.mgr, index);
+    int horizontal = pw->paned.orient == TTK_ORIENT_HORIZONTAL;
 
     if (!Tk_IsMapped(slaveWindow)) {
-	pane->reqSize
-	    = pw->paned.orient == TTK_ORIENT_HORIZONTAL
-	    ? Tk_ReqWidth(slaveWindow) : Tk_ReqHeight(slaveWindow);
+	pane->reqSize = horizontal ? width : height;
     }
-
-    /* Continue with default GeometryRequestProc:
-     */
-    Ttk_GeometryRequestProc(clientData, slaveWindow);
+    return 1;
 }
 
 static Ttk_ManagerSpec PanedManagerSpec = {
-    { "paned", PanedGeometryRequestProc, Ttk_LostSlaveProc },
-    PaneOptionSpecs, sizeof(Pane),
+    { "panedwindow", Ttk_GeometryRequestProc, Ttk_LostSlaveProc },
     PanedSize,
     PanedPlaceSlaves,
-    PaneAdded,
-    PaneRemoved,
-    PaneConfigured,
+    PaneRequest,
+    PaneRemoved
 };
 
 /*------------------------------------------------------------------------
  * +++ Event handler.
  *
+ * <<NOTE-PW-LEAVE-NOTIFYINFERIOR>>
  * Tk does not execute binding scripts for <Leave> events when
  * the pointer crosses from a parent to a child.  This widget
  * needs to know when that happens, though, so it can reset
@@ -406,7 +487,7 @@ static void PanedEventProc(ClientData clientData, XEvent *eventPtr)
     if (   eventPtr->type == LeaveNotify
 	&& eventPtr->xcrossing.detail == NotifyInferior)
     {
-	SendVirtualEvent(corePtr->tkwin, "EnteredChild");
+	TtkSendVirtualEvent(corePtr->tkwin, "EnteredChild");
     }
 }
 
@@ -421,6 +502,7 @@ static int PanedInitialize(Tcl_Interp *interp, void *recordPtr)
     Tk_CreateEventHandler(pw->core.tkwin,
 	PanedEventMask, PanedEventProc, recordPtr);
     pw->paned.mgr = Ttk_CreateManager(&PanedManagerSpec, pw, pw->core.tkwin);
+    pw->paned.paneOptionTable = Tk_CreateOptionTable(interp,PaneOptionSpecs);
     pw->paned.sashLayout = 0;
     pw->paned.sashThickness = 1;
 
@@ -438,6 +520,25 @@ static void PanedCleanup(void *recordPtr)
     Ttk_DeleteManager(pw->paned.mgr);
 }
 
+/* Post-configuration hook.
+ */
+static int PanedPostConfigure(Tcl_Interp *interp, void *clientData, int mask)
+{
+    Paned *pw = clientData;
+
+    if (mask & GEOMETRY_CHANGED) {
+	/* User has changed -width or -height.
+	 * Recalculate sash positions based on requested size.
+	 */
+	Tk_Window tkwin = pw->core.tkwin;
+	PlaceSashes(pw, 
+	    pw->paned.width > 0 ? pw->paned.width : Tk_Width(tkwin),
+	    pw->paned.height > 0 ? pw->paned.height : Tk_Height(tkwin));
+    }
+
+    return TCL_OK;
+}
+
 /*------------------------------------------------------------------------
  * +++ Layout management hooks.
  */
@@ -445,63 +546,71 @@ static Ttk_Layout PanedGetLayout(
     Tcl_Interp *interp, Ttk_Theme themePtr, void *recordPtr)
 {
     Paned *pw = recordPtr;
-    Ttk_Layout panedLayout = WidgetGetLayout(interp, themePtr, recordPtr);
-    int horizontal = pw->paned.orient == TTK_ORIENT_HORIZONTAL;
-    const char *layoutName = horizontal ? ".Vertical.Sash" : ".Horizontal.Sash";
-    Ttk_Layout sashLayout = Ttk_CreateSublayout(interp, themePtr, panedLayout,
-	    layoutName, pw->core.optionTable);
+    Ttk_Layout panedLayout = TtkWidgetGetLayout(interp, themePtr, recordPtr);
 
-    if (sashLayout) {
-	int sashWidth, sashHeight;
+    if (panedLayout) {
+	int horizontal = pw->paned.orient == TTK_ORIENT_HORIZONTAL;
+	const char *layoutName =
+	    horizontal ? ".Vertical.Sash" : ".Horizontal.Sash";
+	Ttk_Layout sashLayout = Ttk_CreateSublayout(
+	    interp, themePtr, panedLayout, layoutName, pw->core.optionTable);
 
-	if (pw->paned.sashLayout)
-	    Ttk_FreeLayout(pw->paned.sashLayout);
-	pw->paned.sashLayout = sashLayout;
+	if (sashLayout) {
+	    int sashWidth, sashHeight;
 
-	Ttk_LayoutSize(sashLayout, 0, &sashWidth, &sashHeight);
+	    Ttk_LayoutSize(sashLayout, 0, &sashWidth, &sashHeight);
+	    pw->paned.sashThickness = horizontal ? sashWidth : sashHeight;
 
-	pw->paned.sashThickness = horizontal ? sashWidth : sashHeight;
-
-	/* Sanity-check:
-	 */
-	if (pw->paned.sashThickness < MIN_SASH_THICKNESS)
-	    pw->paned.sashThickness = MIN_SASH_THICKNESS;
-
-	Ttk_ManagerSizeChanged(pw->paned.mgr);
+	    if (pw->paned.sashLayout)
+		Ttk_FreeLayout(pw->paned.sashLayout);
+	    pw->paned.sashLayout = sashLayout;
+	} else {
+	    Ttk_FreeLayout(panedLayout);
+	    return 0;
+	}
     }
 
+    /* Sanity-check:
+     */
+    if (pw->paned.sashThickness < MIN_SASH_THICKNESS)
+	pw->paned.sashThickness = MIN_SASH_THICKNESS;
+
     return panedLayout;
 }
 
+/*------------------------------------------------------------------------
+ * +++ Drawing routines.
+ */
+
+static void DrawSash(Paned *pw, Drawable d, Ttk_Box b)
+{
+    Ttk_Layout sashLayout = pw->paned.sashLayout;
+    Ttk_State state = pw->core.state;
+
+    Ttk_PlaceLayout(sashLayout, state, b);
+    Ttk_DrawLayout(sashLayout, state, d);
+}
+
 static void PanedDisplay(void *recordPtr, Drawable d)
 {
     Paned *pw = recordPtr;
-    int horizontal = pw->paned.orient == TTK_ORIENT_HORIZONTAL;
-    Ttk_Layout sashLayout = pw->paned.sashLayout;
-    int sashThickness = pw->paned.sashThickness;
-    Ttk_State state = pw->core.state;
-    int nPanes = Ttk_NumberSlaves(pw->paned.mgr);
+    int nPanes = Ttk_NumberSlaves(pw->paned.mgr),
+	horizontal =  pw->paned.orient == TTK_ORIENT_HORIZONTAL,
+	thickness = pw->paned.sashThickness,
+	height = Tk_Height(pw->core.tkwin),
+	width = Tk_Width(pw->core.tkwin);
     int i;
 
-    WidgetDisplay(recordPtr, d);
+    TtkWidgetDisplay(recordPtr, d);
 
     /* Draw sashes:
      */
-    if (horizontal) {
-	int height = Tk_Height(pw->core.tkwin);
-	for (i = 0; i < nPanes; ++i) {
-	    Pane *pane = Ttk_SlaveData(pw->paned.mgr, i);
-	    Ttk_PlaceLayout(sashLayout, state,
-		Ttk_MakeBox(pane->sashPos, 0, sashThickness, height));
-	    Ttk_DrawLayout(sashLayout, state, d);
-	}
-    } else {
-	int width = Tk_Width(pw->core.tkwin);
-	for (i = 0; i < nPanes; ++i) {
-	    Pane *pane = Ttk_SlaveData(pw->paned.mgr, i);
-	    Ttk_PlaceLayout(sashLayout, state,
-		Ttk_MakeBox(0, pane->sashPos, width, sashThickness));
-	    Ttk_DrawLayout(sashLayout, state, d);
+    for (i = 0; i < nPanes; ++i) {
+	Pane *pane = Ttk_SlaveData(pw->paned.mgr, i);
+	if (horizontal) {
+	    DrawSash(pw, d, Ttk_MakeBox(pane->sashPos, 0, thickness, height));
+	} else {
+	    DrawSash(pw, d, Ttk_MakeBox(0, pane->sashPos, width, thickness));
 	}
     }
 }
@@ -530,8 +639,8 @@ static int PanedAddCommand(
 	return TCL_ERROR;
     }
 
-    return Ttk_AddSlave(interp, pw->paned.mgr, slaveWindow,
-	    Ttk_NumberSlaves(pw->paned.mgr), objc - 3, objv + 3);
+    return AddPane(interp, pw, Ttk_NumberSlaves(pw->paned.mgr), slaveWindow,
+	    objc - 3, objv + 3);
 }
 
 /* $pw insert $index $slave ?options...?
@@ -541,6 +650,7 @@ static int PanedInsertCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     Paned *pw = recordPtr;
+    int nSlaves = Ttk_NumberSlaves(pw->paned.mgr);
     int srcIndex, destIndex;
     Tk_Window slaveWindow;
 
@@ -557,23 +667,26 @@ static int PanedInsertCommand(
 
     if (!strcmp(Tcl_GetString(objv[2]), "end")) {
 	destIndex = Ttk_NumberSlaves(pw->paned.mgr);
-    } else if (!Ttk_GetSlaveFromObj(interp,pw->paned.mgr,objv[2],&destIndex)) {
+    } else if (TCL_OK != Ttk_GetSlaveIndexFromObj(
+		interp,pw->paned.mgr,objv[2],&destIndex)) 
+    {
 	return TCL_ERROR;
     }
 
     srcIndex = Ttk_SlaveIndex(pw->paned.mgr, slaveWindow);
     if (srcIndex < 0) { /* New slave: */
-	return Ttk_AddSlave(interp, pw->paned.mgr, slaveWindow,
-		destIndex, objc - 4, objv + 4);
+	return AddPane(interp, pw, destIndex, slaveWindow, objc-4, objv+4);
     } /* else -- move existing slave: */
 
-    if (destIndex >= pw->paned.mgr->nSlaves)
-	destIndex  = pw->paned.mgr->nSlaves - 1;
+    if (destIndex >= nSlaves)
+	destIndex  = nSlaves - 1;
     Ttk_ReorderSlave(pw->paned.mgr, srcIndex, destIndex);
 
     return objc == 4 ? TCL_OK :
-	Ttk_ConfigureSlave(interp, pw->paned.mgr,
-	     pw->paned.mgr->slaves[destIndex], objc-4,objv+4);
+	ConfigurePane(interp, pw,
+		Ttk_SlaveData(pw->paned.mgr, destIndex),
+		Ttk_SlaveWindow(pw->paned.mgr, destIndex),
+		objc-4,objv+4);
 }
 
 /* $pw forget $pane
@@ -589,7 +702,9 @@ static int PanedForgetCommand(
 	return TCL_ERROR;
     }
 
-    if (!Ttk_GetSlaveFromObj(interp, pw->paned.mgr, objv[2], &paneIndex)) {
+    if (TCL_OK != Ttk_GetSlaveIndexFromObj(
+		    interp, pw->paned.mgr, objv[2], &paneIndex)) 
+    {
 	return TCL_ERROR;
     }
     Ttk_ForgetSlave(pw->paned.mgr, paneIndex);
@@ -597,14 +712,15 @@ static int PanedForgetCommand(
     return TCL_OK;
 }
 
-/* $pw identify $x $y
- * @@@ TODO: implement as documented, or change documentation.
+/* $pw identify $x $y --
+ * 	Return index of sash at $x,$y
  */
 static int PanedIdentifyCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     Paned *pw = recordPtr;
     int sashThickness = pw->paned.sashThickness;
+    int nSashes = Ttk_NumberSlaves(pw->paned.mgr) - 1;
     int x, y, pos;
     int index;
 
@@ -619,7 +735,7 @@ static int PanedIdentifyCommand(
     }
 
     pos = pw->paned.orient == TTK_ORIENT_HORIZONTAL ? x : y;
-    for (index = 0; index < pw->paned.mgr->nSlaves - 1; ++index) {
+    for (index = 0; index < nSashes; ++index) {
 	Pane *pane = Ttk_SlaveData(pw->paned.mgr, index);
 	if (pane->sashPos <= pos && pos <= pane->sashPos + sashThickness) {
 	    Tcl_SetObjResult(interp, Tcl_NewIntObj(index));
@@ -627,7 +743,7 @@ static int PanedIdentifyCommand(
 	}
     }
 
-    return TCL_OK;	/* empty list */
+    return TCL_OK;	/* return empty string */
 }
 
 /* $pw pane $pane ?-option ?value -option value ...??
@@ -638,31 +754,62 @@ static int PanedPaneCommand(
 {
     Paned *pw = recordPtr;
     int paneIndex;
-    Ttk_Slave *slave;
+    Tk_Window slaveWindow;
+    Pane *pane;
 
     if (objc < 3) {
 	Tcl_WrongNumArgs(interp, 2,objv, "pane ?-option value...?");
 	return TCL_ERROR;
     }
 
-    slave = Ttk_GetSlaveFromObj(interp,pw->paned.mgr,objv[2],&paneIndex);
-    if (!slave) {
+    if (TCL_OK != Ttk_GetSlaveIndexFromObj(
+		    interp,pw->paned.mgr,objv[2],&paneIndex))
+    {
 	return TCL_ERROR;
     }
 
+    pane = Ttk_SlaveData(pw->paned.mgr, paneIndex);
+    slaveWindow = Ttk_SlaveWindow(pw->paned.mgr, paneIndex);
+
     switch (objc) {
 	case 3:
-	    return EnumerateOptions(interp, slave->slaveData, PaneOptionSpecs,
-			pw->paned.mgr->slaveOptionTable, slave->slaveWindow);
+	    return TtkEnumerateOptions(interp, pane, PaneOptionSpecs,
+			pw->paned.paneOptionTable, slaveWindow);
 	case 4:
-	    return GetOptionValue(interp, slave->slaveData,objv[3],
-			pw->paned.mgr->slaveOptionTable, slave->slaveWindow);
+	    return TtkGetOptionValue(interp, pane, objv[3],
+			pw->paned.paneOptionTable, slaveWindow);
 	default:
-	    return Ttk_ConfigureSlave(
-		    interp, pw->paned.mgr, slave, objc-3,objv+3);
+	    return ConfigurePane(interp, pw, pane, slaveWindow, objc-3,objv+3);
     }
 }
 
+/* $pw panes --
+ * 	Return list of managed panes.
+ */
+static int PanedPanesCommand(
+    Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
+{
+    Paned *pw = recordPtr;
+    Ttk_Manager *mgr = pw->paned.mgr;
+    Tcl_Obj *panes;
+    int i;
+
+    if (objc != 2) {
+	Tcl_WrongNumArgs(interp, 2, objv, "");
+	return TCL_ERROR;
+    }
+
+    panes = Tcl_NewListObj(0, NULL);
+    for (i = 0; i < Ttk_NumberSlaves(mgr); ++i) {
+	const char *pathName = Tk_PathName(Ttk_SlaveWindow(mgr,i));
+	Tcl_ListObjAppendElement(interp, panes, Tcl_NewStringObj(pathName,-1));
+    }
+    Tcl_SetObjResult(interp, panes);
+
+    return TCL_OK;
+}
+
+
 /* $pw sashpos $index ?$newpos?
  * 	Query or modify sash position.
  */
@@ -715,15 +862,16 @@ static int PanedSashposCommand(
 static WidgetCommandSpec PanedCommands[] =
 {
     { "add", 		PanedAddCommand },
-    { "configure",	WidgetConfigureCommand },
-    { "cget",		WidgetCgetCommand },
+    { "configure",	TtkWidgetConfigureCommand },
+    { "cget",		TtkWidgetCgetCommand },
     { "forget", 	PanedForgetCommand },
     { "identify", 	PanedIdentifyCommand },
     { "insert", 	PanedInsertCommand },
-    { "instate",	WidgetInstateCommand },
+    { "instate",	TtkWidgetInstateCommand },
     { "pane",   	PanedPaneCommand },
+    { "panes",   	PanedPanesCommand },
     { "sashpos",  	PanedSashposCommand },
-    { "state",  	WidgetStateCommand },
+    { "state",  	TtkWidgetStateCommand },
     { 0,0 }
 };
 
@@ -733,17 +881,17 @@ static WidgetCommandSpec PanedCommands[] =
 
 static WidgetSpec PanedWidgetSpec =
 {
-    "TPaned",			/* className */
+    "TPanedwindow",		/* className */
     sizeof(Paned),		/* recordSize */
     PanedOptionSpecs,		/* optionSpecs */
     PanedCommands,		/* subcommands */
     PanedInitialize,		/* initializeProc */
     PanedCleanup,		/* cleanupProc */
-    CoreConfigure,		/* configureProc */
-    NullPostConfigure,		/* postConfigureProc */
+    TtkCoreConfigure,		/* configureProc */
+    PanedPostConfigure, 	/* postConfigureProc */
     PanedGetLayout,		/* getLayoutProc */
     PanedSize, 			/* sizeProc */
-    WidgetDoLayout,		/* layoutProc */
+    TtkWidgetDoLayout,		/* layoutProc */
     PanedDisplay		/* displayProc */
 };
 
@@ -776,11 +924,11 @@ static Ttk_ElementSpec SashElementSpec = {
     sizeof(SashElement),
     SashElementOptions,
     SashElementSize,
-    NullElementDraw
+    TtkNullElementDraw
 };
 
 TTK_BEGIN_LAYOUT(PanedLayout)
-    TTK_NODE("Paned.background", 0)	/* @@@ BUG: empty layouts don't work */
+    TTK_NODE("Panedwindow.background", 0)/* @@@ BUG: empty layouts don't work */
 TTK_END_LAYOUT
 
 TTK_BEGIN_LAYOUT(HorizontalSashLayout)
@@ -794,15 +942,16 @@ TTK_END_LAYOUT
 /*------------------------------------------------------------------------
  * +++ Registration routine.
  */
-void Paned_Init(Tcl_Interp *interp)
+
+void TtkPanedwindow_Init(Tcl_Interp *interp)
 {
     Ttk_Theme themePtr = Ttk_GetDefaultTheme(interp);
-    RegisterWidget(interp, "ttk::paned", &PanedWidgetSpec);
+    RegisterWidget(interp, "ttk::panedwindow", &PanedWidgetSpec);
 
     Ttk_RegisterElement(interp, themePtr, "hsash", &SashElementSpec, 0);
     Ttk_RegisterElement(interp, themePtr, "vsash", &SashElementSpec, 0);
 
-    Ttk_RegisterLayout(themePtr, "TPaned", PanedLayout);
+    Ttk_RegisterLayout(themePtr, "TPanedwindow", PanedLayout);
     Ttk_RegisterLayout(themePtr, "Horizontal.Sash", HorizontalSashLayout);
     Ttk_RegisterLayout(themePtr, "Vertical.Sash", VerticalSashLayout);
 }
