@@ -1,4 +1,4 @@
-/* scroll.c,v 1.5 2006/08/27 16:01:00 jenglish Exp
+/* scroll.c,v 1.9 2006/12/18 15:05:58 jenglish Exp
  *
  * Copyright 2004, Joe English
  *
@@ -17,22 +17,22 @@
  *
  * 	1. User adjusts scrollbar, scrollbar widget calls its -command
  * 	2. Scrollbar -command invokes the scrollee [xy]view widget method
- * 	3. ScrollviewCommand calls ScrollTo(), which updates 
+ * 	3. TtkScrollviewCommand calls TtkScrollTo(), which updates 
  * 	   'first' and schedules a redisplay.
  * 	4. Once the scrollee knows 'total' and 'last' (typically in 
- * 	   the LayoutProc), call Scrolled(h,first,last,total) to 
+ * 	   the LayoutProc), call TtkScrolled(h,first,last,total) to 
  * 	   synchronize the scrollbar.
  * 	5. The scrollee -[xy]scrollcommand is called (in an idle callback)
  * 	6. Which calls the scrollbar 'set' method and redisplays the scrollbar.
  *
  * If the scrollee has internal scrolling (e.g., a 'see' method), 
- * it should ScrollTo() directly (step 2).
+ * it should TtkScrollTo() directly (step 2).
  *
- * If the widget value changes, it should call Scrolled() (step 4).
+ * If the widget value changes, it should call TtkScrolled() (step 4).
  * (This usually happens automatically when the widget is redisplayed).
  *
  * If the scrollee's -[xy]scrollcommand changes, it should call
- * ScrollbarUpdateRequired, which will invoke step (5) (@@@ Fix this)
+ * TtkScrollbarUpdateRequired, which will invoke step (5) (@@@ Fix this)
  */
 
 #include <tk.h>
@@ -51,10 +51,10 @@ struct ScrollHandleRec
     Scrollable	*scrollPtr;
 };
 
-/* CreateScrollHandle --
+/* TtkCreateScrollHandle --
  * 	Initialize scroll handle.
  */
-ScrollHandle CreateScrollHandle(WidgetCore *corePtr, Scrollable *scrollPtr)
+ScrollHandle TtkCreateScrollHandle(WidgetCore *corePtr, Scrollable *scrollPtr)
 {
     ScrollHandle h = (ScrollHandle)ckalloc(sizeof(*h));
 
@@ -68,11 +68,6 @@ ScrollHandle CreateScrollHandle(WidgetCore *corePtr, Scrollable *scrollPtr)
     return h;
 }
 
-void FreeScrollHandle(ScrollHandle h)
-{
-    Tcl_EventuallyFree((ClientData)h, TCL_DYNAMIC);
-}
-
 /* UpdateScrollbar --
  *	Call the -scrollcommand callback to sync the scrollbar.
  * 	Returns: Whatever the -scrollcommand does.
@@ -80,27 +75,29 @@ void FreeScrollHandle(ScrollHandle h)
 static int UpdateScrollbar(Tcl_Interp *interp, ScrollHandle h)
 {
     Scrollable *s = h->scrollPtr;
+    WidgetCore *corePtr = h->corePtr;
     char args[TCL_DOUBLE_SPACE * 2];
     int code;
 
-    h->flags &= ~(SCROLL_UPDATE_PENDING | SCROLL_UPDATE_REQUIRED);
+    h->flags &= ~SCROLL_UPDATE_REQUIRED;
 
-    if (s->scrollCmd == NULL)
+    if (s->scrollCmd == NULL) {
 	return TCL_OK;
+    }
 
     sprintf(args, " %g %g",
 		(double)s->first / s->total,
 		(double)s->last / s->total);
 
-    Tcl_Preserve(h->corePtr);
+    Tcl_Preserve(corePtr);
     code = Tcl_VarEval(interp, s->scrollCmd, args, NULL);
-    if (WidgetDestroyed(h->corePtr)) {
-	Tcl_Release(h->corePtr);
+    if (WidgetDestroyed(corePtr)) {
+	Tcl_Release(corePtr);
 	return TCL_ERROR;
     }
-    Tcl_Release(h->corePtr);
+    Tcl_Release(corePtr);
 
-    if (code != TCL_OK) {
+    if (code != TCL_OK && !Tcl_InterpDeleted(interp)) {
 	/* Disable the -scrollcommand, add to stack trace:
 	 */
 	ckfree(s->scrollCmd);
@@ -123,24 +120,19 @@ static void UpdateScrollbarBG(ClientData clientData)
     Tcl_Interp *interp = h->corePtr->interp;
     int code;
 
-    if (WidgetDestroyed(h->corePtr)) {
-	Tcl_Release(clientData);
-	return;
-    }
-
+    h->flags &= ~SCROLL_UPDATE_PENDING;
     Tcl_Preserve((ClientData) interp);
     code = UpdateScrollbar(interp, h);
     if (code == TCL_ERROR && !Tcl_InterpDeleted(interp)) {
 	Tcl_BackgroundError(interp);
     }
     Tcl_Release((ClientData) interp);
-    Tcl_Release(clientData);
 }
 
-/* Scrolled --
+/* TtkScrolled --
  * 	Update scroll info, schedule scrollbar update.
  */
-void Scrolled(ScrollHandle h, int first, int last, int total)
+void TtkScrolled(ScrollHandle h, int first, int last, int total)
 {
     Scrollable *s = h->scrollPtr;
     
@@ -152,6 +144,12 @@ void Scrolled(ScrollHandle h, int first, int last, int total)
 	total = 1;
     }
 
+    if (last > total) {
+	first -= (last - total);
+	if (first < 0) first = 0;
+	last = total;
+    }
+
     if (s->first != first || s->last != last || s->total != total
 	    || (h->flags & SCROLL_UPDATE_REQUIRED))
     {
@@ -160,25 +158,24 @@ void Scrolled(ScrollHandle h, int first, int last, int total)
 	s->total = total;
 
 	if (!(h->flags & SCROLL_UPDATE_PENDING)) {
-	    Tcl_Preserve((ClientData)h);
 	    Tcl_DoWhenIdle(UpdateScrollbarBG, (ClientData)h);
 	    h->flags |= SCROLL_UPDATE_PENDING;
 	}
     }
 }
 
-/* ScrollbarUpdateRequired --
- * 	Force a scrollbar update at the next call to Scrolled(),
+/* TtkScrollbarUpdateRequired --
+ * 	Force a scrollbar update at the next call to TtkScrolled(),
  * 	even if scroll parameters haven't changed (e.g., if
  * 	-yscrollcommand has changed).
  */
 
-void ScrollbarUpdateRequired(ScrollHandle h)
+void TtkScrollbarUpdateRequired(ScrollHandle h)
 {
     h->flags |= SCROLL_UPDATE_REQUIRED;
 }
 
-/* ScrollviewCommand --
+/* TtkScrollviewCommand --
  * 	Widget [xy]view command implementation.
  *
  *  $w [xy]view -- return current view region
@@ -186,7 +183,7 @@ void ScrollbarUpdateRequired(ScrollHandle h)
  *  $w [xy]view moveto $fraction
  *  $w [xy]view scroll $number $what -- scrollbar interface
  */
-int ScrollviewCommand(
+int TtkScrollviewCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *const objv[], ScrollHandle h)
 {
     Scrollable *s = h->scrollPtr;
@@ -224,12 +221,12 @@ int ScrollviewCommand(
 	}
     }
 
-    ScrollTo(h, newFirst);
+    TtkScrollTo(h, newFirst);
 
     return TCL_OK;
 }
 
-void ScrollTo(ScrollHandle h, int newFirst)
+void TtkScrollTo(ScrollHandle h, int newFirst)
 {
     Scrollable *s = h->scrollPtr;
 
@@ -246,3 +243,11 @@ void ScrollTo(ScrollHandle h, int newFirst)
     }
 }
 
+void TtkFreeScrollHandle(ScrollHandle h)
+{
+    if (h->flags & SCROLL_UPDATE_PENDING) {
+	Tcl_CancelIdleCall(UpdateScrollbarBG, (ClientData)h);
+    }
+    ckfree((ClientData)h);
+}
+
