@@ -1,9 +1,9 @@
-/* scrollbar.c,v 1.46 2006/09/26 12:45:10 jenglish Exp
+/* scrollbar.c,v 1.49 2007/10/25 06:42:47 jenglish Exp
  * Copyright (c) 2003, Joe English
- * Tile widget set: scrollbar widget implementation.
+ *
+ * ttk::scrollbar widget.
  */
 
-#include <string.h>
 #include <tk.h>
 
 #include "tkTheme.h"
@@ -40,9 +40,9 @@ static Tk_OptionSpec ScrollbarOptionSpecs[] =
     {TK_OPTION_STRING_TABLE, "-orient", "orient", "Orient", "vertical",
 	Tk_Offset(Scrollbar,scrollbar.orientObj),
 	Tk_Offset(Scrollbar,scrollbar.orient),
-	0,(ClientData)TTKOrientStrings,STYLE_CHANGED },
+	0,(ClientData)ttkOrientStrings,STYLE_CHANGED },
 
-    WIDGET_INHERIT_OPTIONS(CoreOptionSpecs)
+    WIDGET_INHERIT_OPTIONS(ttkCoreOptionSpecs)
 };
 
 /*------------------------------------------------------------------------
@@ -56,7 +56,7 @@ ScrollbarInitialize(Tcl_Interp *interp, void *recordPtr)
     sb->scrollbar.first = 0.0;
     sb->scrollbar.last = 1.0;
 
-    TrackElementState(&sb->core);
+    TtkTrackElementState(&sb->core);
 
     return TCL_OK;
 }
@@ -65,7 +65,7 @@ static Ttk_Layout ScrollbarGetLayout(
     Tcl_Interp *interp, Ttk_Theme theme, void *recordPtr)
 {
     Scrollbar *sb = recordPtr;
-    return WidgetGetOrientedLayout(
+    return TtkWidgetGetOrientedLayout(
 	interp, theme, recordPtr, sb->scrollbar.orientObj);
 }
 
@@ -282,35 +282,65 @@ ScrollbarFractionCommand(
 
 static WidgetCommandSpec ScrollbarCommands[] =
 {
-    { "configure",	WidgetConfigureCommand },
-    { "cget",		WidgetCgetCommand },
+    { "configure",	TtkWidgetConfigureCommand },
+    { "cget",		TtkWidgetCgetCommand },
     { "delta",    	ScrollbarDeltaCommand },
     { "fraction",    	ScrollbarFractionCommand },
     { "get",    	ScrollbarGetCommand },
-    { "identify",	WidgetIdentifyCommand },
-    { "instate",	WidgetInstateCommand },
+    { "identify",	TtkWidgetIdentifyCommand },
+    { "instate",	TtkWidgetInstateCommand },
     { "set",  		ScrollbarSetCommand },
-    { "state",  	WidgetStateCommand },
+    { "state",  	TtkWidgetStateCommand },
     { 0,0 }
 };
 
 /*------------------------------------------------------------------------
  * +++ Widget specification.
  */
-WidgetSpec ScrollbarWidgetSpec =
+static WidgetSpec ScrollbarWidgetSpec =
 {
     "TScrollbar",		/* className */
     sizeof(Scrollbar),		/* recordSize */
     ScrollbarOptionSpecs,	/* optionSpecs */
     ScrollbarCommands,		/* subcommands */
     ScrollbarInitialize,	/* initializeProc */
-    NullCleanup,		/* cleanupProc */
-    CoreConfigure,		/* configureProc */
-    NullPostConfigure,		/* postConfigureProc */
+    TtkNullCleanup,		/* cleanupProc */
+    TtkCoreConfigure,		/* configureProc */
+    TtkNullPostConfigure,	/* postConfigureProc */
     ScrollbarGetLayout,		/* getLayoutProc */
-    WidgetSize, 		/* sizeProc */
+    TtkWidgetSize, 		/* sizeProc */
     ScrollbarDoLayout,		/* layoutProc */
-    WidgetDisplay		/* displayProc */
+    TtkWidgetDisplay		/* displayProc */
 };
 
+TTK_BEGIN_LAYOUT(VerticalScrollbarLayout)
+    TTK_GROUP("Vertical.Scrollbar.trough", TTK_FILL_Y,
+	TTK_NODE("Vertical.Scrollbar.uparrow", TTK_PACK_TOP)
+	TTK_NODE("Vertical.Scrollbar.downarrow", TTK_PACK_BOTTOM)
+	TTK_NODE(
+	    "Vertical.Scrollbar.thumb", TTK_PACK_TOP|TTK_EXPAND|TTK_FILL_BOTH))
+TTK_END_LAYOUT
+
+TTK_BEGIN_LAYOUT(HorizontalScrollbarLayout)
+    TTK_GROUP("Horizontal.Scrollbar.trough", TTK_FILL_X,
+	TTK_NODE("Horizontal.Scrollbar.leftarrow", TTK_PACK_LEFT)
+	TTK_NODE("Horizontal.Scrollbar.rightarrow", TTK_PACK_RIGHT)
+	TTK_NODE(
+	"Horizontal.Scrollbar.thumb", TTK_PACK_LEFT|TTK_EXPAND|TTK_FILL_BOTH))
+TTK_END_LAYOUT
+
+/*------------------------------------------------------------------------
+ * +++ Initialization.
+ */
+
+void TtkScrollbar_Init(Tcl_Interp *interp)
+{
+    Ttk_Theme theme = Ttk_GetDefaultTheme(interp);
+
+    Ttk_RegisterLayout(theme,"Vertical.TScrollbar",VerticalScrollbarLayout);
+    Ttk_RegisterLayout(theme,"Horizontal.TScrollbar",HorizontalScrollbarLayout);
+
+    RegisterWidget(interp, "ttk::scrollbar", &ScrollbarWidgetSpec);
+}
+
 /*EOF*/
