@@ -1,5 +1,5 @@
 /*
- * tkTheme.c --
+ * ttkTheme.c --
  *
  *	This file implements the widget styles and themes support.
  *
@@ -9,7 +9,7 @@
  * See the file "license.terms" for information on usage and redistribution
  * of this file, and for a DISCLAIMER OF ALL WARRANTIES.
  *
- * tkTheme.c,v 1.91 2006/09/20 23:57:41 jenglish Exp
+ * tkTheme.c,v 1.101 2007/12/12 01:19:00 jenglish Exp
  */
 
 #include <stdlib.h>
@@ -504,7 +504,7 @@ void Ttk_RegisterCleanup(
  * 	Scheduled as an idle callback; clientData is a StylePackageData *.
  *
  * 	Sends a <<ThemeChanged>> event to every widget in the hierarchy.
- * 	Tile widgets respond to this by calling the  WorldChanged class proc,
+ * 	Widgets respond to this by calling the  WorldChanged class proc,
  * 	which in turn recreates the layout.
  *
  * 	The Tk C API doesn't doesn't provide an easy way to traverse
@@ -513,7 +513,7 @@ void Ttk_RegisterCleanup(
 
 static void ThemeChangedProc(ClientData clientData)
 {
-    static char ThemeChangedScript[] = "tile::ThemeChanged";
+    static char ThemeChangedScript[] = "ttk::ThemeChanged";
     StylePackageData *pkgPtr = (StylePackageData *)clientData;
 
     if (Tcl_GlobalEval(pkgPtr->interp, ThemeChangedScript) != TCL_OK) {
@@ -671,7 +671,8 @@ Ttk_GetResourceCache(Tcl_Interp *interp)
  * Register a new layout specification with a style.
  * @@@ TODO: Make sure layoutName is not ".", root style must not have a layout
  */
-static void Ttk_RegisterLayoutTemplate(
+MODULE_SCOPE
+void Ttk_RegisterLayoutTemplate(
     Ttk_Theme theme,			/* Target theme */
     const char *layoutName,		/* Name of new layout */
     Ttk_LayoutTemplate layoutTemplate)	/* Template */
@@ -753,7 +754,7 @@ const char *Ttk_StyleName(Ttk_Style stylePtr)
  *	repeat the lookups in the parent theme.
  *	If not found, return the null element.
  */
-Ttk_Element Ttk_GetElement(Ttk_Theme themePtr, const char *elementName)
+Ttk_ElementImpl Ttk_GetElement(Ttk_Theme themePtr, const char *elementName)
 {
     Tcl_HashEntry *entryPtr;
     const char *dot = elementName;
@@ -763,7 +764,7 @@ Ttk_Element Ttk_GetElement(Ttk_Theme themePtr, const char *elementName)
      */
     entryPtr = Tcl_FindHashEntry(&themePtr->elementTable, elementName);
     if (entryPtr) {
-	return (Ttk_Element)Tcl_GetHashValue(entryPtr);
+	return (Ttk_ElementImpl)Tcl_GetHashValue(entryPtr);
     }
 
     /*
@@ -790,7 +791,7 @@ Ttk_Element Ttk_GetElement(Ttk_Theme themePtr, const char *elementName)
      */
     entryPtr = Tcl_FindHashEntry(&themePtr->elementTable, "");
     /* ASSERT: entryPtr != 0 */
-    return (Ttk_Element)Tcl_GetHashValue(entryPtr);
+    return (Ttk_ElementImpl)Tcl_GetHashValue(entryPtr);
 }
 
 const char *Ttk_ElementName(ElementImpl *elementImpl)
@@ -870,7 +871,7 @@ static int Ttk_CloneElement(
  *	if interp is non-NULL.
  */
 
-Ttk_Element Ttk_RegisterElement(
+Ttk_ElementImpl Ttk_RegisterElement(
     Tcl_Interp *interp,		/* Where to leave error messages */
     Ttk_Theme theme,		/* Style engine providing the implementation. */
     const char *name,		/* Name of new element */
@@ -1095,8 +1096,6 @@ Ttk_ElementSize(
     element->specPtr->size(
 	element->clientData, element->elementRecord,
 	tkwin, widthPtr, heightPtr, paddingPtr);
-    *widthPtr += paddingPtr->left + paddingPtr->right;
-    *heightPtr += paddingPtr->top + paddingPtr->bottom;
 }
 
 /*
@@ -1236,8 +1235,9 @@ usage:
 		&stylePtr->settingsTable,optionName,&newEntry);
 
 	Tcl_IncrRefCount(stateMap);
-	if (!newEntry)
+	if (!newEntry) {
 	    Tcl_DecrRefCount((Tcl_Obj*)Tcl_GetHashValue(entryPtr));
+	}
 	Tcl_SetHashValue(entryPtr, stateMap);
     }
     ThemeChanged(pkgPtr);
@@ -1289,8 +1289,9 @@ usage:
 		&stylePtr->defaultsTable,optionName,&newEntry);
 
 	Tcl_IncrRefCount(value);
-	if (!newEntry)
+	if (!newEntry) {
 	    Tcl_DecrRefCount((Tcl_Obj*)Tcl_GetHashValue(entryPtr));
+	}
 	Tcl_SetHashValue(entryPtr, value);
     }
 
@@ -1460,7 +1461,7 @@ static int StyleElementCreateCmd(
     FactoryRec *recPtr;
 
     if (objc < 5) {
-	Tcl_WrongNumArgs(interp, 5, objv, "name type ?options...?");
+	Tcl_WrongNumArgs(interp, 3, objv, "name type ?options...?");
 	return TCL_ERROR;
     }
 
@@ -1496,22 +1497,24 @@ static int StyleElementNamesCmd(
 }
 
 /* + style element options $element --
+ * 	Return list of element options for specified element
  */
 static int StyleElementOptionsCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[])
 {
-    StylePackageData *pkgPtr = (StylePackageData *)clientData;
+    StylePackageData *pkgPtr = clientData;
     Ttk_Theme theme = pkgPtr->currentTheme;
-    Tcl_HashEntry *entryPtr;
+    const char *elementName;
+    ElementImpl *elementImpl;
 
     if (objc != 4) {
 	Tcl_WrongNumArgs(interp, 3, objv, "element");
 	return TCL_ERROR;
     }
 
-    entryPtr = Tcl_FindHashEntry(&theme->elementTable, Tcl_GetString(objv[3]));
-    if (entryPtr) {
-	ElementImpl *elementImpl = (ElementImpl *)Tcl_GetHashValue(entryPtr);
+    elementName = Tcl_GetString(objv[3]);
+    elementImpl = Ttk_GetElement(theme, elementName);
+    if (elementImpl) {
 	Ttk_ElementSpec *specPtr = elementImpl->specPtr;
 	Ttk_ElementOptionSpec *option = specPtr->options;
 	Tcl_Obj *result = Tcl_NewListObj(0,0);
@@ -1526,9 +1529,7 @@ static int StyleElementOptionsCmd(
 	return TCL_OK;
     }
 
-    Tcl_AppendResult(interp, 
-	"element ", Tcl_GetString(objv[3]), " not found",
-	NULL);
+    Tcl_AppendResult(interp, "element ", elementName, " not found", NULL);
     return TCL_ERROR;
 }
 
@@ -1604,7 +1605,7 @@ struct Ensemble {
     struct Ensemble *ensemble;	/* subcommand ensemble */
 };
 
-struct Ensemble StyleThemeEnsemble[] = {
+static struct Ensemble StyleThemeEnsemble[] = {
     { "create", StyleThemeCreateCmd, 0 },
     { "names", StyleThemeNamesCmd, 0 },
     { "settings", StyleThemeSettingsCmd, 0 },
@@ -1612,22 +1613,20 @@ struct Ensemble StyleThemeEnsemble[] = {
     { NULL, 0, 0 }
 };
 
-struct Ensemble StyleElementEnsemble[] = {
+static struct Ensemble StyleElementEnsemble[] = {
     { "create", StyleElementCreateCmd, 0 },
     { "names", StyleElementNamesCmd, 0 },
     { "options", StyleElementOptionsCmd, 0 },
     { NULL, 0, 0 }
 };
 
-struct Ensemble StyleEnsemble[] = {
+static struct Ensemble StyleEnsemble[] = {
     { "configure", StyleConfigureCmd, 0 },
     { "map", StyleMapCmd, 0 },
     { "lookup", StyleLookupCmd, 0 },
     { "layout", StyleLayoutCmd, 0 },
     { "theme", 0, StyleThemeEnsemble },
     { "element", 0, StyleElementEnsemble },
-
-    { "default", StyleConfigureCmd, 0 }, /* TEMP: for pre-0.7 compatibility */
     { NULL, 0, 0 }
 };
 
@@ -1696,12 +1695,13 @@ void Ttk_StylePkgInit(Tcl_Interp *interp)
     /*
      * Register null element, used as a last-resort fallback:
      */
-    Ttk_RegisterElement(interp, pkgPtr->defaultTheme, "", &NullElementSpec, 0);
+    Ttk_RegisterElement(interp, pkgPtr->defaultTheme, "", &ttkNullElementSpec, 0);
 
     /*
      * Register commands:
      */
-    Tcl_CreateObjCommand(interp, "style", StyleObjCmd, (ClientData)pkgPtr, 0);
+    Tcl_CreateObjCommand(interp, "::ttk::style", StyleObjCmd,
+	    (ClientData)pkgPtr, 0);
     Ttk_RegisterElementFactory(interp, "from", Ttk_CloneElement, 0);
 }
 
