@@ -1,14 +1,11 @@
-/*
- * tkTheme.h --
- *      Declarations for Tk style engine.
- *
+/* tkTheme.h,v 1.130 2007/12/16 18:20:55 jenglish Exp
  * Copyright (c) 2003 Joe English.  Freely redistributable.
  *
- * tkTheme.h,v 1.119 2006/10/03 17:53:46 jenglish Exp
+ * Declarations for Tk theme engine.
  */
 
-#ifndef TKTHEME_H
-#define TKTHEME_H 1
+#ifndef _TTKTHEME
+#define _TTKTHEME
 
 #ifdef __cplusplus
 extern "C" {
@@ -21,8 +18,11 @@ extern "C" {
 #	define TTKAPI DLLIMPORT
 #endif
 
-#define TILE_VERSION	"0.7.8"
-#define TILE_PATCHLEVEL	TILE_VERSION
+#ifndef MODULE_SCOPE
+#	define MODULE_SCOPE extern
+#endif
+
+#define TILE_VERSION	"0.8.2"
 
 /*------------------------------------------------------------------------
  * +++ Defaults for element option specifications.
@@ -180,13 +180,13 @@ typedef unsigned int Ttk_PositionSpec;	/* See below */
 /*
  * Extra bits for layout specifications
  */
-#define TTK_CHILDREN	(0x1000)/* for LayoutSpecs -- children follow */
-#define TTK_LAYOUT_END	(0x2000)/* for LayoutSpecs -- end of child list */
+#define _TTK_CHILDREN	(0x1000)/* for LayoutSpecs -- children follow */
+#define _TTK_LAYOUT_END	(0x2000)/* for LayoutSpecs -- end of child list */
+#define _TTK_LAYOUT	(0x4000)/* for LayoutSpec tables -- define layout */
 
 #define _TTK_MASK_STICK (0x0F)	/* See Ttk_UnparseLayout() */
 #define _TTK_MASK_PACK	(0xF0)	/* See Ttk_UnparseLayout(), packStrings */
 
-
 TTKAPI Ttk_Box Ttk_PackBox(Ttk_Box *cavity, int w, int h, Ttk_Side side);
 TTKAPI Ttk_Box Ttk_StickBox(Ttk_Box parcel, int w, int h, Ttk_Sticky sticky);
 TTKAPI Ttk_Box Ttk_AnchorBox(Ttk_Box parcel, int w, int h, Tk_Anchor anchor);
@@ -201,7 +201,7 @@ TTKAPI Ttk_Box Ttk_PositionBox(Ttk_Box *cavity, int w, int h, Ttk_PositionSpec);
 extern void Ttk_StylePkgInit(Tcl_Interp *);
 
 typedef struct Ttk_Theme_ *Ttk_Theme;
-typedef struct Ttk_ElementImpl_ *Ttk_Element;
+typedef struct Ttk_ElementImpl_ *Ttk_ElementImpl;
 typedef struct Ttk_Layout_ *Ttk_Layout;
 typedef struct Ttk_LayoutNode_ Ttk_LayoutNode;
 
@@ -232,7 +232,7 @@ typedef void (Ttk_ElementSizeProc)(void *clientData, void *elementRecord,
 typedef void (Ttk_ElementDrawProc)(void *clientData, void *elementRecord,
         Tk_Window tkwin, Drawable d, Ttk_Box b, Ttk_State state);
 
-typedef struct Ttk_ElementOptionSpec 
+typedef struct Ttk_ElementOptionSpec
 {
     char *optionName;		/* Command-line name of the widget option */
     Tk_OptionType type; 	/* Accepted option types */
@@ -244,13 +244,13 @@ typedef struct Ttk_ElementOptionSpec
 
 typedef struct Ttk_ElementSpec {
     enum TTKStyleVersion2 version;	/* Version of the style support. */
-    size_t elementSize;			/* Size of element record */ 
+    size_t elementSize;			/* Size of element record */
     Ttk_ElementOptionSpec *options;	/* List of options, NULL-terminated */
     Ttk_ElementSizeProc *size;		/* Compute min size and padding */
     Ttk_ElementDrawProc *draw;  	/* Draw the element */
 } Ttk_ElementSpec;
 
-TTKAPI Ttk_Element Ttk_RegisterElement(
+TTKAPI Ttk_ElementImpl Ttk_RegisterElement(
 	Tcl_Interp *interp, Ttk_Theme theme, const char *elementName,
 	Ttk_ElementSpec *, void *clientData);
 
@@ -270,12 +270,12 @@ typedef struct {
     Tcl_Obj	*unused;
 } NullElement;
 
-extern void NullElementGeometry
+extern void TtkNullElementSize
 	(void *, void *, Tk_Window, int *, int *, Ttk_Padding *);
-extern void NullElementDraw
+extern void TtkNullElementDraw
 	(void *, void *, Tk_Window, Drawable, Ttk_Box, Ttk_State);
-extern Ttk_ElementOptionSpec NullElementOptions[];
-extern Ttk_ElementSpec NullElementSpec;
+extern Ttk_ElementOptionSpec TtkNullElementOptions[];
+extern Ttk_ElementSpec ttkNullElementSpec;
 
 /*------------------------------------------------------------------------
  * +++ Layout templates.
@@ -285,17 +285,28 @@ typedef struct {
     unsigned		opcode;
 } TTKLayoutInstruction, *Ttk_LayoutSpec;
 
-#define TTK_BEGIN_LAYOUT(name)	static TTKLayoutInstruction name[] = {
+#define TTK_BEGIN_LAYOUT_TABLE(name) \
+				static TTKLayoutInstruction name[] = {
+#define TTK_LAYOUT(name, content) \
+				{ name, _TTK_CHILDREN|_TTK_LAYOUT }, \
+				content \
+				{ 0, _TTK_LAYOUT_END },
 #define TTK_GROUP(name, flags, children) \
-				{ name, flags | TTK_CHILDREN }, \
+				{ name, flags | _TTK_CHILDREN }, \
 				children \
-				{ 0, TTK_LAYOUT_END },
+				{ 0, _TTK_LAYOUT_END },
 #define TTK_NODE(name, flags)	{ name, flags },
-#define TTK_END_LAYOUT 		{ 0, TTK_LAYOUT_END } };
+#define TTK_END_LAYOUT_TABLE	{ 0, _TTK_LAYOUT | _TTK_LAYOUT_END } };
+
+#define TTK_BEGIN_LAYOUT(name)	static TTKLayoutInstruction name[] = {
+#define TTK_END_LAYOUT 		{ 0, _TTK_LAYOUT_END } };
 
 TTKAPI void Ttk_RegisterLayout(
     Ttk_Theme theme, const char *className, Ttk_LayoutSpec layoutSpec);
 
+TTKAPI void Ttk_RegisterLayouts(
+    Ttk_Theme theme, Ttk_LayoutSpec layoutTable);
+
 /*------------------------------------------------------------------------
  * +++ Layout instances.
  */
@@ -324,7 +335,6 @@ extern Ttk_Box Ttk_LayoutNodeInternalParcel(Ttk_Layout,Ttk_LayoutNode *);
 extern Ttk_Padding Ttk_LayoutNodeInternalPadding(Ttk_Layout,Ttk_LayoutNode *);
 extern void Ttk_LayoutNodeReqSize(Ttk_Layout, Ttk_LayoutNode *, int *w, int *h);
 
-extern void Ttk_LayoutNodeSetParcel(Ttk_LayoutNode *node, Ttk_Box parcel);
 extern void Ttk_PlaceLayoutNode(Ttk_Layout,Ttk_LayoutNode *, Ttk_Box);
 extern void Ttk_ChangeElementState(Ttk_LayoutNode *,unsigned set,unsigned clr);
 
@@ -347,6 +357,15 @@ extern Tk_Image Ttk_UseImage(Ttk_ResourceCache, Tk_Window, Tcl_Obj *);
 
 extern void Ttk_RegisterNamedColor(Ttk_ResourceCache, const char *, XColor *);
 
+/*------------------------------------------------------------------------
+ * +++ Image specifications.
+ */
+
+typedef struct TtkImageSpec Ttk_ImageSpec;
+TTKAPI Ttk_ImageSpec *TtkGetImageSpec(Tcl_Interp *, Tk_Window, Tcl_Obj *);
+TTKAPI void TtkFreeImageSpec(Ttk_ImageSpec *);
+TTKAPI Tk_Image TtkSelectImage(Ttk_ImageSpec *, Ttk_State);
+
 /*------------------------------------------------------------------------
  * +++ Miscellaneous enumerations.
  * 	Other stuff that element implementations need to know about.
@@ -358,7 +377,7 @@ typedef enum 			/* -default option values */
     TTK_BUTTON_DEFAULT_DISABLED	/* not defaultable */
 } Ttk_ButtonDefaultState;
 
-extern int Ttk_GetButtonDefaultStateFromObj(Tcl_Interp *, Tcl_Obj *, int *);
+TTKAPI int Ttk_GetButtonDefaultStateFromObj(Tcl_Interp *, Tcl_Obj *, int *);
 
 typedef enum 			/* -compound option values */
 {
@@ -372,11 +391,11 @@ typedef enum 			/* -compound option values */
     TTK_COMPOUND_RIGHT  	/* image to right of text */
 } Ttk_Compound;
 
-extern int Ttk_GetCompoundFromObj(Tcl_Interp *, Tcl_Obj *, int *);
+TTKAPI int Ttk_GetCompoundFromObj(Tcl_Interp *, Tcl_Obj *, int *);
 
 typedef enum { 		/* -orient option values */
     TTK_ORIENT_HORIZONTAL,
-    TTK_ORIENT_VERTICAL 
+    TTK_ORIENT_VERTICAL
 } Ttk_Orient;
 
 /*------------------------------------------------------------------------
@@ -390,11 +409,11 @@ typedef enum { 		/* -orient option values */
  * (@@@ find a better home for this)
  */
 typedef enum { ARROW_UP, ARROW_DOWN, ARROW_LEFT, ARROW_RIGHT } ArrowDirection;
-extern void ArrowSize(int h, ArrowDirection, int *widthPtr, int *heightPtr);
-extern void DrawArrow(Display *, Drawable, GC, Ttk_Box, ArrowDirection);
-extern void FillArrow(Display *, Drawable, GC, Ttk_Box, ArrowDirection);
+extern void TtkArrowSize(int h, ArrowDirection, int *widthPtr, int *heightPtr);
+extern void TtkDrawArrow(Display *, Drawable, GC, Ttk_Box, ArrowDirection);
+extern void TtkFillArrow(Display *, Drawable, GC, Ttk_Box, ArrowDirection);
 
 #ifdef __cplusplus
 }
 #endif
-#endif /* TKTHEME_H */
+#endif /* _TTKTHEME */
