@@ -1,12 +1,12 @@
-/* track.c,v 1.7 2005/10/08 14:02:47 jenglish Exp
+/* track.c,v 1.9 2007/10/22 03:13:34 jenglish Exp
  * Copyright (c) 2004, Joe English
  *
- * TrackElementState() -- helper routine for widgets
+ * TtkTrackElementState() -- helper routine for widgets
  * like scrollbars in which individual elements may
  * be active or pressed instead of the widget as a whole.
  *
  * Usage:
- * 	TrackElementState(&recordPtr->core);
+ * 	TtkTrackElementState(&recordPtr->core);
  *
  * Registers an event handler on the widget that tracks pointer
  * events and updates the state of the element under the
@@ -21,8 +21,6 @@
  * <ButtonRelease> event.
  *
  * TODO: Handle "chords" properly (e.g., <B1-ButtonPress-2>)
- * TODO: Deal with grabs -- possible to get a Press event w/no corresponding Release.
- *
  */
 
 #include <tk.h>
@@ -132,6 +130,8 @@ ElementStateEventProc(ClientData clientData, XEvent *ev)
 	    break;
 	case LeaveNotify:
 	    ActivateElement(es, 0);
+	    if (ev->xcrossing.mode == NotifyGrab)
+		PressElement(es, 0);
 	    break;
 	case EnterNotify:
 	    node = Ttk_LayoutIdentify(
@@ -152,18 +152,18 @@ ElementStateEventProc(ClientData clientData, XEvent *ev)
 	     */
 	    Tk_DeleteEventHandler(es->corePtr->tkwin,
 		    ElementStateMask, ElementStateEventProc, es);
-	    ckfree((ClientData)es);
+	    ckfree(clientData);
 	    break;
     }
 }
 
 /*
- * TrackElementState --
+ * TtkTrackElementState --
  * 	Register an event handler to manage the 'pressed'
  * 	and 'active' states of individual widget elements.
  */
 
-void TrackElementState(WidgetCore *corePtr)
+void TtkTrackElementState(WidgetCore *corePtr)
 {
     ElementStateTracker *es = (ElementStateTracker*)ckalloc(sizeof(*es));
     es->corePtr = corePtr;
@@ -172,4 +172,3 @@ void TrackElementState(WidgetCore *corePtr)
 	    ElementStateMask,ElementStateEventProc,es);
 }
 
-
