@@ -1,5 +1,5 @@
 /*
- * ttkStubLib.c,v 1.3 2005/07/13 17:47:39 jenglish Exp
+ * ttkStubLib.c,v 1.4 2006/11/11 22:16:41 jenglish Exp
  * SOURCE: tk/generic/tkStubLib.c, version 1.9 2004/03/17
  */
 
@@ -25,7 +25,8 @@ const TtkStubs *ttkStubsPtr;
  *	Sets the stub table pointer.
  *
  */
-const char *TtkInitializeStubs(
+const char *
+TtkInitializeStubs(
     Tcl_Interp *interp, const char *version, int epoch, int revision)
 {
     int exact = 0;
