@@ -1,8 +1,7 @@
-/* widget.c,v 1.47 2005/10/08 15:53:42 jenglish Exp
+/* widget.c,v 1.52 2007/10/30 14:58:59 jenglish Exp
  * Copyright (c) 2003, Joe English
  *
- * Tile widget implementation, core widget utilities.
- *
+ * Core widget utilities.
  */
 
 #include <string.h>
@@ -11,25 +10,35 @@
 #include "widget.h"
 
 /*------------------------------------------------------------------------
- * Helper routines.
+ * +++ Internal helper routines.
  */
 
-static void UpdateLayout(Tcl_Interp *interp, WidgetCore *corePtr)
+/* UpdateLayout --
+ * 	Call the widget's get-layout hook to recompute corePtr->layout.
+ * 	Returns TCL_OK if successful, returns TCL_ERROR and leaves
+ * 	the layout unchanged otherwise.
+ */
+static int UpdateLayout(Tcl_Interp *interp, WidgetCore *corePtr)
 {
     Ttk_Theme themePtr = Ttk_GetCurrentTheme(interp);
-    Ttk_Layout newLayout = 
+    Ttk_Layout newLayout =
     	corePtr->widgetSpec->getLayoutProc(interp, themePtr,corePtr);
 
-    /* TODO: @@@ Check for errors */
     if (newLayout) {
 	if (corePtr->layout) {
 	    Ttk_FreeLayout(corePtr->layout);
 	}
 	corePtr->layout = newLayout;
+	return TCL_OK;
     }
+    return TCL_ERROR;
 }
 
-static void UpdateGeometry(WidgetCore *corePtr)
+/* SizeChanged --
+ * 	Call the widget's sizeProc to compute new requested size
+ * 	and pass it to the geometry manager.
+ */
+static void SizeChanged(WidgetCore *corePtr)
 {
     int reqWidth = 1, reqHeight = 1;
 
@@ -38,93 +47,107 @@ static void UpdateGeometry(WidgetCore *corePtr)
     }
 }
 
-/*
- * RedisplayWidget --
- *	Redraw a widget.  Called as an idle handler.
- */
+#ifndef TK_NO_DOUBLE_BUFFERING
 
-static void RedisplayWidget(ClientData recordPtr)
+/* BeginDrawing --
+ * 	Returns a Drawable for drawing the widget contents.
+ *	This is normally an off-screen Pixmap, copied to
+ *	the window by EndDrawing().
+ */
+static Drawable BeginDrawing(Tk_Window tkwin)
+{
+    return Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
+	    Tk_Width(tkwin), Tk_Height(tkwin),
+	    DefaultDepthOfScreen(Tk_Screen(tkwin)));
+}
+
+/* EndDrawing --
+ *	Copy the drawable contents to the screen and release resources.
+ */
+static void EndDrawing(Tk_Window tkwin, Drawable d)
 {
-    WidgetCore *corePtr = (WidgetCore *)recordPtr;
-    Tk_Window tkwin = corePtr->tkwin;
-    Drawable d;
     XGCValues gcValues;
     GC gc;
 
-    corePtr->flags &= ~REDISPLAY_PENDING;
-    if (!Tk_IsMapped(tkwin)) {
-	return;
-    }
-
-    /*
-     * Get a Pixmap for drawing in the background:
-     */
-    d = Tk_GetPixmap(Tk_Display(tkwin), Tk_WindowId(tkwin),
-	    Tk_Width(tkwin), Tk_Height(tkwin),
-	    DefaultDepthOfScreen(Tk_Screen(tkwin)));
-
-    /*
-     * Get a GC for blitting the pixmap to the display:
-     */
     gcValues.function = GXcopy;
     gcValues.graphics_exposures = False;
-    gc = Tk_GetGC(corePtr->tkwin, GCFunction|GCGraphicsExposures, &gcValues);
+    gc = Tk_GetGC(tkwin, GCFunction|GCGraphicsExposures, &gcValues);
 
-    /*
-     * Recompute layout and draw widget contents:
-     */
-    corePtr->widgetSpec->layoutProc(recordPtr);
-    corePtr->widgetSpec->displayProc(recordPtr, d);
-
-    /*
-     * Copy to the screen.
-     */
     XCopyArea(Tk_Display(tkwin), d, Tk_WindowId(tkwin), gc,
 	    0, 0, (unsigned) Tk_Width(tkwin), (unsigned) Tk_Height(tkwin),
 	    0, 0);
 
-    /*
-     * Release resources
-     */
     Tk_FreePixmap(Tk_Display(tkwin), d);
     Tk_FreeGC(Tk_Display(tkwin), gc);
 }
+#else
+/* No double-buffering: draw directly into the window. */
+static Drawable BeginDrawing(Tk_Window tkwin) { return Tk_WindowId(tkwin); }
+static void EndDrawing(Tk_Window tkwin, Drawable d) { }
+#endif
 
-/* TtkRedisplayWidget -- 
+/* DrawWidget --
+ *	Redraw a widget.  Called as an idle handler.
+ */
+static void DrawWidget(ClientData recordPtr)
+{
+    WidgetCore *corePtr = recordPtr;
+
+    corePtr->flags &= ~REDISPLAY_PENDING;
+    if (Tk_IsMapped(corePtr->tkwin)) {
+	Drawable d = BeginDrawing(corePtr->tkwin);
+	corePtr->widgetSpec->layoutProc(recordPtr);
+	corePtr->widgetSpec->displayProc(recordPtr, d);
+	EndDrawing(corePtr->tkwin, d);
+    }
+}
+
+/* TtkRedisplayWidget --
  * 	Schedule redisplay as an idle handler.
  */
-
-void TtkRedisplayWidget(WidgetCore *corePtr) 
+void TtkRedisplayWidget(WidgetCore *corePtr)
 {
     if (corePtr->flags & WIDGET_DESTROYED) {
 	return;
     }
 
     if (!(corePtr->flags & REDISPLAY_PENDING)) {
-	Tcl_DoWhenIdle(RedisplayWidget, (ClientData) corePtr);
+	Tcl_DoWhenIdle(DrawWidget, (ClientData) corePtr);
 	corePtr->flags |= REDISPLAY_PENDING;
     }
 }
 
 /* TtkResizeWidget --
- * 	Recompute widget size, schedule geometry propagation and redisplay. 
+ * 	Recompute widget size, schedule geometry propagation and redisplay.
  */
-
 void TtkResizeWidget(WidgetCore *corePtr)
 {
     if (corePtr->flags & WIDGET_DESTROYED) {
 	return;
     }
 
-    UpdateGeometry(corePtr);
+    SizeChanged(corePtr);
     TtkRedisplayWidget(corePtr);
 }
 
-/* WidgetEnsembleCommand --
+/* TtkWidgetChangeState --
+ * 	Set / clear the specified bits in the 'state' flag,
+ */
+void TtkWidgetChangeState(WidgetCore *corePtr,
+    unsigned int setBits, unsigned int clearBits)
+{
+    Ttk_State oldState = corePtr->state;
+    corePtr->state = (oldState & ~clearBits) | setBits;
+    if (corePtr->state ^ oldState) {
+	TtkRedisplayWidget(corePtr);
+    }
+}
+
+/* TtkWidgetEnsembleCommand --
  * 	Invoke an ensemble defined by a WidgetCommandSpec.
  */
-int WidgetEnsembleCommand(
-    WidgetCommandSpec *commands,	/* Ensemble definition */
+int TtkWidgetEnsembleCommand(
+    const WidgetCommandSpec *commands,	/* Ensemble definition */
     int cmdIndex,			/* Index of command word */
     Tcl_Interp *interp,			/* Interpreter to use */
     int objc, Tcl_Obj *const objv[],	/* Argument vector */
@@ -153,14 +176,14 @@ WidgetInstanceObjCmd(
     ClientData clientData,		/* Widget record pointer */
     Tcl_Interp *interp,			/* Current interpreter. */
     int objc,				/* Number of arguments. */
-    Tcl_Obj * CONST objv[])		/* Argument objects. */
+    Tcl_Obj * const objv[])		/* Argument objects. */
 {
     WidgetCore *corePtr = (WidgetCore *)clientData;
-    WidgetCommandSpec *commands = corePtr->widgetSpec->commands;
+    const WidgetCommandSpec *commands = corePtr->widgetSpec->commands;
     int status = TCL_OK;
 
     Tcl_Preserve(clientData);
-    status = WidgetEnsembleCommand(commands, 1, interp, objc, objv, clientData);
+    status = TtkWidgetEnsembleCommand(commands,1, interp,objc,objv,clientData);
     Tcl_Release(clientData);
 
     return status;
@@ -207,7 +230,7 @@ WidgetCleanup(char *memPtr)
  *
  *	For Deactivate/Activate pseudo-events, clear/set the background state flag.
  *
- *	<<NOTE-REALIZED>> On the first ConfigureNotify event 
+ *	<<NOTE-REALIZED>> On the first ConfigureNotify event
  *	(which indicates that the window has just been created),
  *	update the layout.  This is to work around two problems:
  *	(1) Virtual events aren't delivered to unrealized widgets
@@ -235,8 +258,8 @@ static void CoreEventProc(ClientData clientData, XEvent *eventPtr)
 	case ConfigureNotify :
 	    if (!(corePtr->flags & WIDGET_REALIZED)) {
 		/* See <<NOTE-REALIZED>> */
-		UpdateLayout(corePtr->interp, corePtr);
-		UpdateGeometry(corePtr);
+		(void)UpdateLayout(corePtr->interp, corePtr);
+		SizeChanged(corePtr);
 		corePtr->flags |= WIDGET_REALIZED;
 	    }
 	    TtkRedisplayWidget(corePtr);
@@ -253,7 +276,7 @@ static void CoreEventProc(ClientData clientData, XEvent *eventPtr)
 		    CoreEventMask,CoreEventProc,clientData);
 
 	    if (corePtr->flags & REDISPLAY_PENDING) {
-		Tcl_CancelIdleCall(RedisplayWidget, clientData);
+		Tcl_CancelIdleCall(DrawWidget, clientData);
 	    }
 
 	    corePtr->widgetSpec->cleanupProc(corePtr);
@@ -299,8 +322,8 @@ static void CoreEventProc(ClientData clientData, XEvent *eventPtr)
 	    break;
 	case VirtualEvent:
 	    if (!strcmp("ThemeChanged", ((XVirtualEvent *)(eventPtr))->name)) {
-		UpdateLayout(corePtr->interp, corePtr);
-		UpdateGeometry(corePtr);
+		(void)UpdateLayout(corePtr->interp, corePtr);
+		SizeChanged(corePtr);
 		TtkRedisplayWidget(corePtr);
 	    }
 	default:
@@ -318,7 +341,7 @@ static void CoreEventProc(ClientData clientData, XEvent *eventPtr)
 static void WidgetWorldChanged(ClientData clientData)
 {
     WidgetCore *corePtr = (WidgetCore*)clientData;
-    UpdateGeometry(corePtr);
+    SizeChanged(corePtr);
     TtkRedisplayWidget(corePtr);
 }
 
@@ -328,11 +351,11 @@ static struct Tk_ClassProcs widgetClassProcs = {
 };
 
 /*
- * WidgetConstructorObjCmd --
+ * TtkWidgetConstructorObjCmd --
  *	General-purpose widget constructor command implementation.
  *	ClientData is a WidgetSpec *.
  */
-int WidgetConstructorObjCmd(
+int TtkWidgetConstructorObjCmd(
     ClientData clientData, Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[])
 {
     WidgetSpec *widgetSpec = (WidgetSpec *)clientData;
@@ -393,10 +416,10 @@ int WidgetConstructorObjCmd(
     Tk_SetClassProcs(tkwin, &widgetClassProcs, recordPtr);
 
     if (Tk_InitOptions(interp, recordPtr, optionTable, tkwin) != TCL_OK)
-    	goto error;
+    	goto error_nocleanup;
 
     if (widgetSpec->initializeProc(interp, recordPtr) != TCL_OK)
-	goto error;
+	goto error_nocleanup;
 
     if (Tk_SetOptions(interp, recordPtr, optionTable, objc - 2,
 	    objv + 2, tkwin, NULL/*savePtr*/, (int *)NULL/*maskPtr*/) != TCL_OK)
@@ -411,9 +434,10 @@ int WidgetConstructorObjCmd(
     if (WidgetDestroyed(corePtr))
 	goto error;
 
-    UpdateLayout(interp, corePtr);
-    UpdateGeometry(corePtr);
+    if (UpdateLayout(interp, corePtr) != TCL_OK)
+	goto error;
 
+    SizeChanged(corePtr);
     Tk_CreateEventHandler(tkwin, CoreEventMask, CoreEventProc, recordPtr);
 
     Tcl_SetObjResult(interp, Tcl_NewStringObj(Tk_PathName(tkwin), -1));
@@ -421,6 +445,8 @@ int WidgetConstructorObjCmd(
     return TCL_OK;
 
 error:
+    widgetSpec->cleanupProc(recordPtr);
+error_nocleanup:
     if (corePtr->layout) {
 	Ttk_FreeLayout(corePtr->layout);
 	corePtr->layout = 0;
@@ -433,49 +459,38 @@ error:
     return TCL_ERROR;
 }
 
-/*
- * WidgetChangeState --
- * 	Set / clear the specified bits in the 'state' flag,
+/*------------------------------------------------------------------------
+ * +++ Default implementations for widget hook procedures.
  */
-void WidgetChangeState(WidgetCore *corePtr,
-    unsigned int setBits, unsigned int clearBits)
-{
-    Ttk_State oldState = corePtr->state;
-    corePtr->state = (oldState & ~clearBits) | setBits;
-    if (corePtr->state ^ oldState) {
-	TtkRedisplayWidget(corePtr);
-    }
-}
 
-/*
- * WidgetGetLayout --
+/* TtkWidgetGetLayout --
  * 	Default getLayoutProc.
  *	Looks up the layout based on the -style resource (if specified),
  *	otherwise use the widget class.
  */
-Ttk_Layout WidgetGetLayout(
+Ttk_Layout TtkWidgetGetLayout(
     Tcl_Interp *interp, Ttk_Theme themePtr, void *recordPtr)
 {
     WidgetCore *corePtr = recordPtr;
     const char *styleName = 0;
 
-    if (corePtr->styleObj) 
+    if (corePtr->styleObj)
     	styleName = Tcl_GetString(corePtr->styleObj);
 
     if (!styleName || *styleName == '\0')
     	styleName = corePtr->widgetSpec->className;
 
-    return Ttk_CreateLayout(interp, themePtr,styleName, 
+    return Ttk_CreateLayout(interp, themePtr, styleName,
 	recordPtr, corePtr->optionTable, corePtr->tkwin);
 }
 
 /*
- * WidgetGetOrientedLayout --
- * 	Helper routine.  Same as WidgetGetLayout, but prefixes 
- * 	"Horizontal." or "Vertical." to the style name, depending 
+ * TtkWidgetGetOrientedLayout --
+ * 	Helper routine.  Same as TtkWidgetGetLayout, but prefixes
+ * 	"Horizontal." or "Vertical." to the style name, depending
  * 	on the value of the 'orient' option.
  */
-Ttk_Layout WidgetGetOrientedLayout(
+Ttk_Layout TtkWidgetGetOrientedLayout(
     Tcl_Interp *interp, Ttk_Theme themePtr, void *recordPtr, Tcl_Obj *orientObj)
 {
     WidgetCore *corePtr = recordPtr;
@@ -486,18 +501,17 @@ Ttk_Layout WidgetGetOrientedLayout(
 
     Tcl_DStringInit(&styleName);
 
-    /* Prefix: 
+    /* Prefix:
      */
     Ttk_GetOrientFromObj(NULL, orientObj, &orient);
     if (orient == TTK_ORIENT_HORIZONTAL)
 	Tcl_DStringAppend(&styleName, "Horizontal.", -1);
-    else 
+    else
 	Tcl_DStringAppend(&styleName, "Vertical.", -1);
 
-
     /* Add base style name:
      */
-    if (corePtr->styleObj) 
+    if (corePtr->styleObj)
     	baseStyleName = Tcl_GetString(corePtr->styleObj);
     if (!baseStyleName || *baseStyleName == '\0')
     	baseStyleName = corePtr->widgetSpec->className;
@@ -506,7 +520,7 @@ Ttk_Layout WidgetGetOrientedLayout(
 
     /* Create layout:
      */
-    layout= Ttk_CreateLayout(interp, themePtr, Tcl_DStringValue(&styleName), 
+    layout= Ttk_CreateLayout(interp, themePtr, Tcl_DStringValue(&styleName),
 	recordPtr, corePtr->optionTable, corePtr->tkwin);
 
     Tcl_DStringFree(&styleName);
@@ -514,97 +528,81 @@ Ttk_Layout WidgetGetOrientedLayout(
     return layout;
 }
 
-/*
- * NullInitialize --
+/* TtkNullInitialize --
  * 	Default widget initializeProc (no-op)
  */
-int  NullInitialize(Tcl_Interp *interp, void *recordPtr)
+int TtkNullInitialize(Tcl_Interp *interp, void *recordPtr)
 {
     return TCL_OK;
 }
 
-/*
- * NullPostConfigure --
+/* TtkNullPostConfigure --
  * 	Default widget postConfigureProc (no-op)
  */
-int NullPostConfigure(Tcl_Interp *interp, void *clientData, int mask)
+int TtkNullPostConfigure(Tcl_Interp *interp, void *clientData, int mask)
 {
     return TCL_OK;
 }
 
-/* CoreConfigure --
+/* TtkCoreConfigure --
  * 	Default widget configureProc.
+ * 	Handles -style option.
  */
-int CoreConfigure(Tcl_Interp *interp, void *clientData, int mask)
+int TtkCoreConfigure(Tcl_Interp *interp, void *clientData, int mask)
 {
     WidgetCore *corePtr = clientData;
-    
-    if (mask & STYLE_CHANGED) {
-	Ttk_Theme theme = Ttk_GetCurrentTheme(interp);
-	Ttk_Layout newLayout =
-	    corePtr->widgetSpec->getLayoutProc(interp,theme,corePtr);
+    int status = TCL_OK;
 
-	if (!newLayout) {
-	    return TCL_ERROR;
-	}
-	if (corePtr->layout) {
-	    Ttk_FreeLayout(corePtr->layout);
-	}
-	corePtr->layout = newLayout;
+    if (mask & STYLE_CHANGED) {
+	status = UpdateLayout(interp, corePtr);
     }
 
-    return TCL_OK;
+    return status;
 }
 
-/*
- * NullCleanup --
+/* TtkNullCleanup --
  * 	Default widget cleanupProc (no-op)
  */
-void NullCleanup(void *recordPtr)
+void TtkNullCleanup(void *recordPtr)
 {
     return;
 }
 
-/*
- * WidgetDoLayout --
+/* TtkWidgetDoLayout --
  * 	Default widget layoutProc.
  */
-void WidgetDoLayout(void *clientData)
+void TtkWidgetDoLayout(void *clientData)
 {
     WidgetCore *corePtr = clientData;
     Ttk_PlaceLayout(corePtr->layout,corePtr->state,Ttk_WinBox(corePtr->tkwin));
 }
 
-/*
- * WidgetDisplay --
+/* TtkWidgetDisplay --
  * 	Default widget displayProc.
  */
-void WidgetDisplay(void *recordPtr, Drawable d)
+void TtkWidgetDisplay(void *recordPtr, Drawable d)
 {
     WidgetCore *corePtr = recordPtr;
     Ttk_DrawLayout(corePtr->layout, corePtr->state, d);
 }
 
-/*
- * WidgetSize --
+/* TtkWidgetSize --
  * 	Default widget sizeProc()
  */
-int WidgetSize(void *recordPtr, int *widthPtr, int *heightPtr)
+int TtkWidgetSize(void *recordPtr, int *widthPtr, int *heightPtr)
 {
     WidgetCore *corePtr = recordPtr;
     Ttk_LayoutSize(corePtr->layout, corePtr->state, widthPtr, heightPtr);
     return 1;
-
-/* OR: (@@@)
-    return *widthPtr > Tk_Width(corePtr->tkwin) 
-	|| *heightPtr > Tk_Height(corePtr->tkwin);
-*/
 }
 
+/*------------------------------------------------------------------------
+ * +++ Default implementations for widget subcommands.
+ */
 
-/* Default implementations for widget subcommands:
-*/
-int WidgetCgetCommand(
+/* $w cget -option
+ */
+int TtkWidgetCgetCommand(
 Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[], void *recordPtr)
 {
     WidgetCore *corePtr = recordPtr;
@@ -622,7 +620,9 @@ Tcl_Interp *interp, int objc, Tcl_Obj * CONST objv[], void *recordPtr)
     return TCL_OK;
 }
 
-int WidgetConfigureCommand(
+/* $w configure ?-option ?value ....??
+ */
+int TtkWidgetConfigureCommand(
 Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     WidgetCore *corePtr = recordPtr;
@@ -665,7 +665,7 @@ Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 	}
 
 	if (mask & (STYLE_CHANGED | GEOMETRY_CHANGED)) {
-	    UpdateGeometry(corePtr);
+	    SizeChanged(corePtr);
 	}
 
 	TtkRedisplayWidget(corePtr);
@@ -679,8 +679,7 @@ Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
     return TCL_OK;
 }
 
-/* $w state $stateSpec
- * $w state
+/* $w state ? $stateSpec ?
  *
  * 	If $stateSpec is specified, modify the widget state accordingly,
  * 	return a new stateSpec representing the changed bits.
@@ -688,7 +687,7 @@ Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
  * 	Otherwise, return a statespec matching all the currently-set bits.
  */
 
-int WidgetStateCommand(
+int TtkWidgetStateCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     WidgetCore *corePtr = recordPtr;
@@ -728,7 +727,7 @@ int WidgetStateCommand(
  *	Otherwise, return true/false
  */
 
-int WidgetInstateCommand(
+int TtkWidgetInstateCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     WidgetCore *corePtr = recordPtr;
@@ -758,7 +757,7 @@ int WidgetInstateCommand(
 /* $w identify $x $y
  * 	Returns: name of element at $x, $y
  */
-int WidgetIdentifyCommand(
+int TtkWidgetIdentifyCommand(
     Tcl_Interp *interp, int objc, Tcl_Obj *CONST objv[], void *recordPtr)
 {
     WidgetCore *corePtr = recordPtr;
@@ -774,7 +773,7 @@ int WidgetIdentifyCommand(
 	|| Tcl_GetIntFromObj(interp, objv[3], &y) != TCL_OK)
 	return TCL_ERROR;
 
-    node = Ttk_LayoutIdentify(corePtr->layout, x, y); 
+    node = Ttk_LayoutIdentify(corePtr->layout, x, y);
     if (node) {
 	const char *elementName = Ttk_LayoutNodeName(node);
 	Tcl_SetObjResult(interp,Tcl_NewStringObj(elementName,-1));
