@@ -1,5 +1,5 @@
 #
-# cursors.tcl,v 1.2 2006/07/22 16:37:19 jenglish Exp
+# cursors.tcl,v 1.3 2006/11/05 18:40:48 jenglish Exp
 #
 # Tile package: Symbolic cursor names.
 #
@@ -7,9 +7,9 @@
 #	for the various functions.
 #
 
-namespace eval tile {
+namespace eval ttk {
 
-    variable Cursors;
+    variable Cursors
 
     switch -glob $::tcl_platform(platform) {
 
