@@ -1,5 +1,5 @@
 #
-# menubutton.tcl,v 1.6 2006/01/16 15:29:47 jenglish Exp
+# menubutton.tcl,v 1.8 2006/08/20 19:40:39 jenglish Exp
 #
 # Bindings for Menubuttons.
 #
@@ -37,26 +37,8 @@
 #
 
 namespace eval ttk {
-
-    bind TMenubutton <Enter>		{ %W state active }
-    bind TMenubutton <Leave>		{ %W state !active }
-    bind TMenubutton <<Invoke>> 	{ ttk::menubutton::Popdown %W }
-    bind TMenubutton <Key-space>	{ ttk::menubutton::Popdown %W }
-
-    if {[tk windowingsystem] eq "x11"} {
-	bind TMenubutton <ButtonPress-1>   { ttk::menubutton::Pulldown %W }
-	bind TMenubutton <ButtonRelease-1> { ttk::menubutton::TransferGrab %W }
-	bind TMenubutton <B1-Leave>  	   { ttk::menubutton::TransferGrab %W }
-    } else {
-    	bind TMenubutton <ButtonPress-1>  \
-	    { %W state pressed ; ttk::menubutton::Popdown %W }
-	bind TMenubutton <ButtonRelease-1>  \
-	    { %W state !pressed }
-    }
-
     namespace eval menubutton {
 	variable State
-
 	array set State {
 	    pulldown	0
 	    oldcursor	{}
@@ -64,6 +46,22 @@ namespace eval ttk {
     }
 }
 
+bind TMenubutton <Enter>	{ %W instate !disabled {%W state active } }
+bind TMenubutton <Leave>	{ %W state !active }
+bind TMenubutton <Key-space> 	{ ttk::menubutton::Popdown %W }
+bind TMenubutton <<Invoke>> 	{ ttk::menubutton::Popdown %W }
+
+if {[tk windowingsystem] eq "x11"} {
+    bind TMenubutton <ButtonPress-1>  	{ ttk::menubutton::Pulldown %W }
+    bind TMenubutton <ButtonRelease-1>	{ ttk::menubutton::TransferGrab %W }
+    bind TMenubutton <B1-Leave> 	{ ttk::menubutton::TransferGrab %W }
+} else {
+    bind TMenubutton <ButtonPress-1>  \
+	{ %W state pressed ; ttk::menubutton::Popdown %W }
+    bind TMenubutton <ButtonRelease-1>  \
+	{ %W state !pressed }
+}
+
 # PostPosition --
 #	Returns the x and y coordinates where the menu 
 #	should be posted, based on the menubutton and menu size
@@ -145,7 +143,9 @@ proc ttk::menubutton::TransferGrab {mb} {
 	$mb configure -cursor $State(oldcursor)
 	$mb state {!pressed !active}
 	set State(pulldown) 0
-	grab -global [$mb cget -menu]
+
+	set menu [$mb cget -menu]
+    	tk_popup $menu [winfo rootx $menu] [winfo rooty $menu]
     }
 }
 
