@@ -1,10 +1,10 @@
 #
-# notebook.tcl,v 1.19 2006/07/04 17:21:55 jenglish Exp
-# 
+# notebook.tcl,v 1.24 2007/02/26 18:06:26 hobbs Exp
+#
 # Bindings for TNotebook widget
 #
 
-namespace eval ttk::notebook { 
+namespace eval ttk::notebook {
     variable TLNotebooks ;# See enableTraversal
 }
 
@@ -22,9 +22,9 @@ bind TNotebook <Destroy>		{ ttk::notebook::Cleanup %W }
 #	Select the specified tab and set focus.
 #
 # 	If $tab was already the current tab, set the focus to the
-#	notebook widget.  Otherwise, set the focus to the first 
-#	traversable widget in the pane.  The behavior is that the 
-#	notebook takes focus when the user selects the same tab 
+#	notebook widget.  Otherwise, set the focus to the first
+#	traversable widget in the pane.  The behavior is that the
+#	notebook takes focus when the user selects the same tab
 #	a second time.  This mirrors Windows tab behavior.
 #
 proc ttk::notebook::ActivateTab {w tab} {
@@ -34,29 +34,11 @@ proc ttk::notebook::ActivateTab {w tab} {
     	$w select $tab
 	update ;# needed so focus logic sees correct mapped/unmapped states
 	if {[set f [ttk::focusFirst [$w select]]] ne ""} {
-	    keynav::traverseTo $f
+	    ttk::traverseTo $f
 	}
     }
 }
 
-# ttk::focusFirst $w --
-#	Return the first descendant of $w, in preorder traversal order,
-#	that can take keyboard focus, "" if none do.
-#
-# See also: tk_focusNext
-#
-proc ttk::focusFirst {w} {
-    if {[tile::takesFocus $w]} { 
-	return $w 
-    }
-    foreach child [winfo children $w] {
-	if {[set c [ttk::focusFirst $child]] ne ""} {
-	    return $c
-	}
-    }
-    return ""
-}
-
 # Press $nb $x $y --
 #	ButtonPress-1 binding for notebook widgets.
 #	Activate the tab under the mouse cursor, if any.
@@ -66,7 +48,7 @@ proc ttk::notebook::Press {w x y} {
     if {$index ne ""} {
 	ActivateTab $w $index
     }
-} 
+}
 
 # CycleTab --
 #	Select the next/previous tab in the list.
@@ -85,7 +67,7 @@ proc ttk::notebook::CycleTab {w dir} {
 }
 
 # MnemonicTab $nb $key --
-#	Scan all tabs in the specified notebook for one with the 
+#	Scan all tabs in the specified notebook for one with the
 #	specified mnemonic. If found, returns path name of tab;
 #	otherwise returns ""
 #
@@ -109,8 +91,8 @@ proc ttk::notebook::MnemonicTab {nb key} {
 #	Enable keyboard traversal for a notebook widget
 #	by adding bindings to the containing toplevel window.
 #
-#	TLNotebooks($top) keeps track of the list of all traversal-enabled 
-#	notebooks contained in the toplevel 
+#	TLNotebooks($top) keeps track of the list of all traversal-enabled
+#	notebooks contained in the toplevel
 #
 proc ttk::notebook::enableTraversal {nb} {
     variable TLNotebooks
@@ -125,8 +107,13 @@ proc ttk::notebook::enableTraversal {nb} {
 	catch {
 	bind $top <Control-Key-ISO_Left_Tab> {+ttk::notebook::TLCycleTab %W -1}
 	}
-	bind $top <Alt-KeyPress> \
-	    +[list ttk::notebook::MnemonicActivation $top %K]
+	if {[tk windowingsystem] eq "aqua"} {
+	    bind $top <Option-KeyPress> \
+		+[list ttk::notebook::MnemonicActivation $top %K]
+	} else {
+	    bind $top <Alt-KeyPress> \
+		+[list ttk::notebook::MnemonicActivation $top %K]
+	}
 	bind $top <Destroy> {+ttk::notebook::TLCleanup %W}
     }
 
@@ -153,7 +140,7 @@ proc ttk::notebook::Cleanup {nb} {
     }
 }
 
-# EnclosingNotebook $w -- 
+# EnclosingNotebook $w --
 #	Return the nearest traversal-enabled notebook widget
 #	that contains $w.
 #
@@ -179,7 +166,7 @@ proc ttk::notebook::EnclosingNotebook {w} {
 
 # TLCycleTab --
 #	toplevel binding procedure for Control-Tab / Shift-Control-Tab
-#	Select the next/previous tab in the nearest ancestor notebook. 
+#	Select the next/previous tab in the nearest ancestor notebook.
 #
 proc ttk::notebook::TLCycleTab {w dir} {
     set nb [EnclosingNotebook $w]
@@ -204,9 +191,3 @@ proc ttk::notebook::MnemonicActivation {top key} {
     }
 }
 
-# +++ Backwards-compatibility (0.7.2)
-
-namespace eval tile::notebook { }
-ttk::deprecated ::tile::notebook::enableTraversal ttk::notebook::enableTraversal
-ttk::deprecated ::tile::enableNotebookTraversal   ttk::notebook::enableTraversal
-
