@@ -2,7 +2,7 @@
 #
 # Bindings for the TScale widget
 #
-# scale.tcl,v 1.10 2006/01/16 15:29:47 jenglish Exp
+# scale.tcl,v 1.11 2006/11/05 18:40:48 jenglish Exp
 
 namespace eval ttk::scale {
     variable State
@@ -23,9 +23,9 @@ proc ttk::scale::Press {w x y} {
 	*track -
         *trough {
 	    if {[$w get $x $y] <= [$w get]} {
-		tile::Repeatedly Increment $w -1
+		ttk::Repeatedly Increment $w -1
 	    } else {
-		tile::Repeatedly Increment $w 1
+		ttk::Repeatedly Increment $w 1
 	    }
         }
         *slider {
@@ -45,7 +45,7 @@ proc ttk::scale::Drag {w x y} {
 proc ttk::scale::Release {w x y} {
     variable State
     set State(dragging) 0
-    tile::CancelRepeat
+    ttk::CancelRepeat
 }
 
 proc ttk::scale::Increment {w delta} {
