@@ -1,39 +1,57 @@
 #
-# scrollbar.tcl,v 1.15 2005/07/18 14:53:26 jenglish Exp
+# scrollbar.tcl,v 1.19 2007/12/12 16:53:10 jenglish Exp
 #
 # Bindings for TScrollbar widget
 #
 
-namespace eval tile::scrollbar {
+# Still don't have a working ttk::scrollbar under OSX -
+# Swap in a [tk::scrollbar] on that platform,
+# unless user specifies -class or -style.
+#
+if {[tk windowingsystem] eq "aqua"} {
+    rename ::ttk::scrollbar ::ttk::_scrollbar_
+    proc ttk::scrollbar {w args} {
+	set constructor ::scrollbar
+	foreach {option _} $args {
+	    if {$option eq "-class" || $option eq "-style"} {
+		set constructor ::ttk::_scrollbar_
+		break
+	    }
+	}
+	return [eval [linsert $args 0 $constructor $w]]
+    }
+}
+
+namespace eval ttk::scrollbar {
     variable State
     # State(xPress)	--
     # State(yPress)	-- initial position of mouse at start of drag.
     # State(first)	-- value of -first at start of drag.
 }
 
-bind TScrollbar <ButtonPress-1> 	{ tile::scrollbar::Press %W %x %y }
-bind TScrollbar <B1-Motion>		{ tile::scrollbar::Drag %W %x %y }
-bind TScrollbar <ButtonRelease-1>	{ tile::scrollbar::Release %W %x %y }
+bind TScrollbar <ButtonPress-1> 	{ ttk::scrollbar::Press %W %x %y }
+bind TScrollbar <B1-Motion>		{ ttk::scrollbar::Drag %W %x %y }
+bind TScrollbar <ButtonRelease-1>	{ ttk::scrollbar::Release %W %x %y }
 
-bind TScrollbar <ButtonPress-2> 	{ tile::scrollbar::Jump %W %x %y }
-bind TScrollbar <B2-Motion>		{ tile::scrollbar::Drag %W %x %y }
-bind TScrollbar <ButtonRelease-2>	{ tile::scrollbar::Release %W %x %y }
+bind TScrollbar <ButtonPress-2> 	{ ttk::scrollbar::Jump %W %x %y }
+bind TScrollbar <B2-Motion>		{ ttk::scrollbar::Drag %W %x %y }
+bind TScrollbar <ButtonRelease-2>	{ ttk::scrollbar::Release %W %x %y }
 
-proc tile::scrollbar::Scroll {w n units} {
+proc ttk::scrollbar::Scroll {w n units} {
     set cmd [$w cget -command]
     if {$cmd ne ""} {
 	uplevel #0 $cmd scroll $n $units
     }
 }
 
-proc tile::scrollbar::Moveto {w fraction} {
+proc ttk::scrollbar::Moveto {w fraction} {
     set cmd [$w cget -command]
     if {$cmd ne ""} {
 	uplevel #0 $cmd moveto $fraction
     }
 }
 
-proc tile::scrollbar::Press {w x y} {
+proc ttk::scrollbar::Press {w x y} {
     variable State
 
     set State(xPress) $x
@@ -42,11 +60,11 @@ proc tile::scrollbar::Press {w x y} {
     switch -glob -- [$w identify $x $y] {
     	*uparrow -
 	*leftarrow {
-	    tile::Repeatedly Scroll $w -1 units
+	    ttk::Repeatedly Scroll $w -1 units
 	}
 	*downarrow -
 	*rightarrow {
-	    tile::Repeatedly Scroll $w  1 units
+	    ttk::Repeatedly Scroll $w  1 units
 	}
 	*thumb {
 	    set State(first) [lindex [$w get] 0]
@@ -55,10 +73,10 @@ proc tile::scrollbar::Press {w x y} {
 	    set f [$w fraction $x $y]
 	    if {$f < [lindex [$w get] 0]} {
 		# Clicked in upper/left trough
-		tile::Repeatedly Scroll $w -1 pages
+		ttk::Repeatedly Scroll $w -1 pages
 	    } elseif {$f > [lindex [$w get] 1]} {
 		# Clicked in lower/right trough
-		tile::Repeatedly Scroll $w 1 pages
+		ttk::Repeatedly Scroll $w 1 pages
 	    } else {
 		# Clicked on thumb (???)
 		set State(first) [lindex [$w get] 0]
@@ -67,7 +85,7 @@ proc tile::scrollbar::Press {w x y} {
     }
 }
 
-proc tile::scrollbar::Drag {w x y} {
+proc ttk::scrollbar::Drag {w x y} {
     variable State
     if {![info exists State(first)]} {
     	# Initial buttonpress was not on the thumb, 
@@ -79,17 +97,17 @@ proc tile::scrollbar::Drag {w x y} {
     Moveto $w [expr {$State(first) + [$w delta $xDelta $yDelta]}]
 }
 
-proc tile::scrollbar::Release {w x y} {
+proc ttk::scrollbar::Release {w x y} {
     variable State
     unset -nocomplain State(xPress) State(yPress) State(first)
-    tile::CancelRepeat
+    ttk::CancelRepeat
 }
 
 # scrollbar::Jump -- ButtonPress-2 binding for scrollbars.
 # 	Behaves exactly like scrollbar::Press, except that
 #	clicking in the trough jumps to the the selected position.
 #
-proc tile::scrollbar::Jump {w x y} {
+proc ttk::scrollbar::Jump {w x y} {
     variable State
 
     switch -glob -- [$w identify $x $y] {
