@@ -1,5 +1,5 @@
 #
-# tile.tcl,v 1.90 2006/10/03 18:23:03 jenglish Exp
+# tile.tcl,v 1.104 2006/12/13 17:05:25 jenglish Exp
 #
 # Tile widget set initialization script.
 #
@@ -22,14 +22,6 @@ source [file join $tile::library cursors.tcl]
 source [file join $tile::library icons.tcl]
 source [file join $tile::library utils.tcl]
 
-### Forward-compatibility.
-#
-# In 0.8.0, ttk::paned will be renamed ttk::panedwindow.
-#
-# Alias new name to current name to ease the transition:
-#
-interp alias {} ::ttk::panedwindow {} ::ttk::paned
-
 ### Deprecated aliases.
 #
 
@@ -47,7 +39,7 @@ proc ttk::deprecated {old new} {
 proc ttk::do'deprecate {old new args} {
     deprecated'warning $old $new
     interp alias {} $old {} $new
-    eval [linsert $args 0 $new]
+    uplevel 1 [linsert $args 0 $new]
 }
 
 ## deprecated'warning --
@@ -57,37 +49,48 @@ proc ttk::deprecated'warning {old new} {
     puts stderr "$old deprecated -- use $new instead"
 }
 
-### Widgets.
-#	Widgets are all defined in the ::ttk namespace.
-#
-#	For compatibility with earlier Tile releases, we temporarily 
-#	create aliases ::tile::widget, and ::t$widget.
-#	Using any of the aliases will issue a warning.
+### Backward compatibility:
 #
 
+## 0.7.X compatibility: renamed in 0.8.0 
+#
+ttk::deprecated ::ttk::paned		::ttk::panedwindow
+ttk::deprecated ::tile::availableThemes	::ttk::themes
+ttk::deprecated ::tile::setTheme	::ttk::setTheme
+#Not yet: ttk::deprecated ::style	::ttk::style
+interp alias {} ::style {} ::ttk::style
+
+ttk::deprecated ::tile::defineImage	::ttk::defineImage
+ttk::deprecated ::tile::stockIcon	::ttk::stockIcon
+
+ttk::deprecated ::tile::CopyBindings	::ttk::copyBindings
+
+### Exported routines:
+#
 namespace eval ttk {
+
+    namespace export style
+
+    # All widget constructor commands are exported:
     variable widgets {
 	button checkbutton radiobutton menubutton label entry
 	frame labelframe scrollbar
 	notebook progressbar combobox separator 
+	panedwindow treeview sizegrip
 	scale
     }
 
     variable wc
-    foreach wc $widgets {
+    foreach wc $widgets  {
 	namespace export $wc
-
-	deprecated ::t$wc ::ttk::$wc
-	deprecated ::tile::$wc ::ttk::$wc
-	namespace eval ::tile [list namespace export $wc]
     }
 }
 
-### tile::ThemeChanged --
+### ttk::ThemeChanged --
 #	Called from [style theme use].
 #	Sends a <<ThemeChanged>> virtual event to all widgets.
 #
-proc tile::ThemeChanged {} {
+proc ttk::ThemeChanged {} {
     set Q .
     while {[llength $Q]} {
 	set QN [list]
@@ -104,26 +107,26 @@ proc tile::ThemeChanged {} {
 ### Public API.
 #
 
-## tile::availableThemes --
+## ttk::themes --
 #	Return list of themes registered in the package database.
 #
-proc tile::availableThemes {} {
+proc ttk::themes {} {
     set themes [list]
 
-    foreach pkg [lsearch -inline -all -glob [package names] tile::theme::*] {
+    foreach pkg [lsearch -inline -all -glob [package names] ttk::theme::*] {
 	lappend themes [lindex [split $pkg :] end]
     }
 
     return $themes
 }
 
-## tile::setTheme $theme --
+## ttk::setTheme $theme --
 #	Set the current theme to $theme, loading it if necessary.
 #
-proc tile::setTheme {theme} {
-    variable currentTheme	;# @@@ Temp -- [style theme use] doesn't work
+proc ttk::setTheme {theme} {
+    variable currentTheme
     if {[lsearch [style theme names] $theme] < 0} {
-	package require tile::theme::$theme
+	package require ttk::theme::$theme
     }
     style theme use $theme
     set currentTheme $theme
@@ -147,50 +150,62 @@ source [file join $tile::library dialog.tcl]
 ## Label and Labelframe bindings:
 #  (not enough to justify their own file...)
 #
-bind TLabelframe <<Invoke>>	{ keynav::traverseTo [tk_focusNext %W] }
-bind TLabel <<Invoke>>		{ keynav::traverseTo [tk_focusNext %W] }
+bind TLabelframe <<Invoke>>	{ ttk::traverseTo [tk_focusNext %W] }
+bind TLabel <<Invoke>>		{ ttk::traverseTo [tk_focusNext %W] }
 
-### Load themes.
+### Load settings for built-in themes:
 #
-source [file join $tile::library defaults.tcl]
-source [file join $tile::library classicTheme.tcl]
-source [file join $tile::library altTheme.tcl]
-source [file join $tile::library stepTheme.tcl]
-source [file join $tile::library clamTheme.tcl]
+proc ttk::LoadThemes {} {
+    variable ::tile::library
 
+    # "default" always present:
+    uplevel #0 [list source [file join $library defaults.tcl]] 
 
-### Choose platform-specific default theme.
-#
-# Notes: 
-#	+ xpnative takes precedence over winnative if available.
-#	+ On X11, users can use the X resource database to
-#	  specify a preferred theme (*TkTheme: themeName)
-#
-
-set ::tile::defaultTheme "default"
-
-if {[package provide tile::theme::winnative] != {}} {
-    source [file join $tile::library winTheme.tcl]
-    set ::tile::defaultTheme "winnative"
-}
-if {[package provide tile::theme::xpnative] != {}} {
-    source [file join $tile::library xpTheme.tcl]
-    set ::tile::defaultTheme "xpnative"
-}
-if {[package provide tile::theme::aqua] != {}} {
-    source [file join $tile::library aquaTheme.tcl]
-    set ::tile::defaultTheme "aqua"
-}
-
-set tile::userTheme [option get . tkTheme TkTheme]
-if {$tile::userTheme != {}} {
-    if {    [lsearch [style theme names] $tile::userTheme] >= 0
-        || ![catch [list package require tile::theme::$tile::userTheme]]
+    set builtinThemes [style theme names]
+    foreach {theme script} {
+	classic 	classicTheme.tcl
+	alt 		altTheme.tcl
+	clam 		clamTheme.tcl
+	winnative	winTheme.tcl
+	xpnative	xpTheme.tcl
+	aqua 		aquaTheme.tcl
     } {
-	set ::tile::defaultTheme $tile::userTheme
+	if {[lsearch -exact $builtinThemes $theme] >= 0} {
+	    uplevel #0 [list source [file join $library $script]]
+	}
     }
 }
 
-tile::setTheme $::tile::defaultTheme
+ttk::LoadThemes; rename ::ttk::LoadThemes {}
+
+### Select platform-specific default theme:
+#
+# Notes: 
+#	+ On OSX, aqua theme is the default
+#	+ On Windows, xpnative takes precedence over winnative if available.
+#	+ On X11, users can use the X resource database to
+#	  specify a preferred theme (*TkTheme: themeName);
+#	  otherwise "default" is used.
+#
+
+proc ttk::DefaultTheme {} {
+    set preferred [list aqua xpnative winnative]
+
+    set userTheme [option get . tkTheme TkTheme]
+    if {$userTheme != {} && ![catch {
+	uplevel #0 [list package require ttk::theme::$userTheme]
+    }]} {
+	return $userTheme
+    }
+
+    foreach theme $preferred {
+	if {[package provide ttk::theme::$theme] != ""} {
+	    return $theme
+	}
+    }
+    return "default"
+}
+
+ttk::setTheme [ttk::DefaultTheme] ; rename ttk::DefaultTheme {}
 
 #*EOF*
