@@ -1,31 +1,43 @@
 #
-# utils.tcl,v 1.5 2006/07/04 17:21:55 jenglish Exp
+# utils.tcl,v 1.10 2007/10/22 03:21:25 jenglish Exp
 #
-# Tile widget set: utilities for widget implementations.
+# Utilities for widget implementations.
 #
 
 ### Focus management.
 #
 
-## tile::takefocus --
+## ttk::takefocus --
 #	This is the default value of the "-takefocus" option
 #	for widgets that participate in keyboard navigation.
 #
 # See also: tk::FocusOK
 #
-proc tile::takefocus {w} {
+proc ttk::takefocus {w} {
     expr {[$w instate !disabled] && [winfo viewable $w]}
 }
 
-## tile::clickToFocus $w --
+## ttk::traverseTo $w --
+# 	Set the keyboard focus to the specified window.
+#
+proc ttk::traverseTo {w} {
+    set focus [focus]
+    if {$focus ne ""} {
+	event generate $focus <<TraverseOut>>
+    }
+    focus $w
+    event generate $w <<TraverseIn>>
+}
+
+## ttk::clickToFocus $w --
 #	Utility routine, used in <ButtonPress-1> bindings --
 #	Assign keyboard focus to the specified widget if -takefocus is enabled.
 #
-proc tile::clickToFocus {w} {
-    if {[tile::takesFocus $w]} { focus $w }
+proc ttk::clickToFocus {w} {
+    if {[ttk::takesFocus $w]} { focus $w }
 }
 
-## tile::takesFocus w --
+## ttk::takesFocus w --
 #	Test if the widget can take keyboard focus:
 #
 #	+ widget is viewable, AND:
@@ -39,7 +51,7 @@ proc tile::clickToFocus {w} {
 # Note: This routine doesn't implement the same fallback heuristics 
 #	as tk::FocusOK.
 #
-proc tile::takesFocus {w} {
+proc ttk::takesFocus {w} {
 
     if {![winfo viewable $w]} { return 0 }
 
@@ -58,6 +70,25 @@ proc tile::takesFocus {w} {
     return 0
 }
 
+## ttk::focusFirst $w --
+#	Return the first descendant of $w, in preorder traversal order,
+#	that can take keyboard focus, "" if none do.
+#
+# See also: tk_focusNext
+#
+
+proc ttk::focusFirst {w} {
+    if {[ttk::takesFocus $w]} { 
+	return $w 
+    }
+    foreach child [winfo children $w] {
+	if {[set c [ttk::focusFirst $child]] ne ""} {
+	    return $c
+	}
+    }
+    return ""
+}
+
 ### Grabs.
 #
 # Rules:
@@ -69,7 +100,7 @@ proc tile::takesFocus {w} {
 #
 #	See #1239190 and #1411983 for more discussion.
 #
-namespace eval tile {
+namespace eval ttk {
     variable Grab 		;# map: window name -> grab token
 
     # grab token details:
@@ -81,9 +112,16 @@ namespace eval tile {
 ## SaveGrab --
 #	Record current grab and focus windows.
 #
-proc tile::SaveGrab {w} {
+proc ttk::SaveGrab {w} {
     variable Grab
 
+    if {[info exists Grab($w)]} {
+	# $w is already on the grab stack.
+	# This should not happen, but bail out in case it does anyway:
+	#
+	return
+    }
+
     set restoreGrab [set restoreFocus ""]
 
     set grabbed [grab current $w]
@@ -108,7 +146,7 @@ proc tile::SaveGrab {w} {
 #	If called more than once without an intervening [SaveGrab $w],
 #	does nothing.
 #
-proc tile::RestoreGrab {w} {
+proc ttk::RestoreGrab {w} {
     variable Grab
 
     if {![info exists Grab($w)]} {	# Ignore
@@ -125,28 +163,28 @@ proc tile::RestoreGrab {w} {
     unset Grab($w)
 }
 
-## tile::grabWindow $w --
+## ttk::grabWindow $w --
 #	Records the current focus and grab windows, sets an application-modal
 #	grab on window $w.
 #
-proc tile::grabWindow {w} {
+proc ttk::grabWindow {w} {
     SaveGrab $w
     grab $w
 }
 
-## tile::globalGrab $w --
+## ttk::globalGrab $w --
 #	Same as grabWindow, but sets a global grab on $w.
 #
-proc tile::globalGrab {w} {
+proc ttk::globalGrab {w} {
     SaveGrab $w
     grab -global $w
 }
 
-## tile::releaseGrab --
-#	Release the grab previously set by [tile::grabWindow]
-#	or [tile::globalGrab].
+## ttk::releaseGrab --
+#	Release the grab previously set by [ttk::grabWindow]
+#	or [ttk::globalGrab].
 #
-proc tile::releaseGrab {w} {
+proc ttk::releaseGrab {w} {
     grab release $w
     RestoreGrab $w
 }
@@ -162,7 +200,7 @@ proc tile::releaseGrab {w} {
 #  @@@ for -repeatdelay/-repeatinterval)
 #
 
-namespace eval tile {
+namespace eval ttk {
     variable Repeat
     array set Repeat {
 	delay		300
@@ -172,31 +210,31 @@ namespace eval tile {
     }
 }
 
-## tile::Repeatedly --
+## ttk::Repeatedly --
 #	Begin auto-repeat.
 #
-proc tile::Repeatedly {args} {
+proc ttk::Repeatedly {args} {
     variable Repeat
     after cancel $Repeat(timer)
     set script [uplevel 1 [list namespace code $args]]
     set Repeat(script) $script
     uplevel #0 $script
-    set Repeat(timer) [after $Repeat(delay) tile::Repeat]
+    set Repeat(timer) [after $Repeat(delay) ttk::Repeat]
 }
 
 ## Repeat --
 #	Continue auto-repeat
 #
-proc tile::Repeat {} {
+proc ttk::Repeat {} {
     variable Repeat
     uplevel #0 $Repeat(script)
-    set Repeat(timer) [after $Repeat(interval) tile::Repeat]
+    set Repeat(timer) [after $Repeat(interval) ttk::Repeat]
 }
 
-## tile::CancelRepeat --
+## ttk::CancelRepeat --
 #	Halt auto-repeat.
 #
-proc tile::CancelRepeat {} {
+proc ttk::CancelRepeat {} {
     variable Repeat
     after cancel $Repeat(timer)
 }
@@ -204,31 +242,13 @@ proc tile::CancelRepeat {} {
 ### Miscellaneous.
 #
 
-## tile::CopyBindings $from $to --
+## ttk::copyBindings $from $to --
 #	Utility routine; copies bindings from one bindtag onto another.
 #
-proc tile::CopyBindings {from to} {
+proc ttk::copyBindings {from to} {
     foreach event [bind $from] {
 	bind $to $event [bind $from $event]
     }
 }
 
-## tile::LoadImages $imgdir ?$patternList? --
-#	Utility routine for pixmap themes
-#
-#	Loads all image files in $imgdir matching $patternList.
-#	Returns: a paired list of filename/imagename pairs.
-#
-proc tile::LoadImages {imgdir {patterns {*.gif}}} {
-    foreach pattern $patterns {
-	foreach file [glob -directory $imgdir $pattern] {
-	    set img [file tail [file rootname $file]]
-	    if {![info exists images($img)]} {
-		set images($img) [image create photo -file $file]
-	    }
-	}
-    }
-    return [array get images]
-}
-
 #*EOF*
