@@ -13,7 +13,7 @@ bind TNotebook <Shift-KeyPress-F4>	{ destroy %W }
 # Argh.  Shift-KeyPress-Fn bindings don't work under XFree
 bind TNotebook <Control-KeyPress-F3>	{ destroy %W }
 
-set pw [ttk::paned .pw -orient vertical]
+set pw [ttk::panedwindow .pw -orient vertical]
 
 foreach nb {.pw.nb1 .pw.nb2} {
     $pw add [ttk::notebook $nb] -weight 1
