@@ -1,4 +1,4 @@
-# tvtest.tcl,v 1.8 2006/10/03 17:53:46 jenglish Exp
+# tvtest.tcl,v 1.11 2006/12/18 19:03:33 jenglish Exp
 # 
 # Sandbox test script for treeview testing 
 #
@@ -20,12 +20,15 @@ option add *tearOff 0
 set f [ttk::frame .f]
 set tv $f.tv
 ttk::scrollbar $f.vsb -orient vertical -command [list $tv yview]
+ttk::scrollbar $f.hsb -orient horizontal -command [list $tv xview]
 ttk::treeview $tv \
-    -columns [list X Y Z] -displaycolumns [list Z Y] \
+    -columns [list X Y Z] -displaycolumns [list X Y Z] \
     -yscrollcommand [list $f.vsb set] \
+    -xscrollcommand [list $f.hsb set] \
     ;
 
 grid $f.tv $f.vsb -sticky news
+grid $f.hsb -sticky news
 grid columnconfigure $f 0 -weight 1
 grid rowconfigure $f 0 -weight 1
 
@@ -57,7 +60,7 @@ proc setShow {tv} {
     variable Show
     set show [list]
     foreach {what} [list headings tree] {
-    	if {$Show($what} {
+    	if {$Show($what)} {
 	    lappend show $what 
 	}
     }
@@ -159,16 +162,27 @@ proc identify {tv x y} {
     }
 }
 
+## Add some nodes:
+#
 
 $tv insert {} end -id a -text "First node" -values [list a1 a2 a3]
 $tv insert {} end -id b -text "Second node" -values [list b1 b2 b3]
 $tv insert {} end -id c -text "Third node" -values [list c1 c2 c3]
 
+$tv insert {} end -id moveme -text "Press ^M to move me" -values {- - -}
+bind $tv <Control-KeyPress-m> {
+    %W move moveme [%W parent [%W focus]] [%W index [%W focus]]
+    ttk::treeview::BrowseTo %W moveme
+}
+bind $tv <Control-Shift-KeyPress-M> {
+    %W move moveme [%W focus] 0
+    ttk::treeview::BrowseTo %W moveme
+}
+
 $tv insert a end -id a1 -text "First subnode" -values [list d1 d2 d3]
 $tv insert a end -id a2 -text "Second subnode" -values [list e1 e2 e3]
 $tv insert a1 end -text "Subsubnode!!" -values [list X! Y! Z!]
 
-
 foreach label {a b c d e f g h i j k l m n o} {
     $tv insert b end -text $label
 }
@@ -257,7 +271,7 @@ proc selectColumns {tv} {
 # ...
 if {$haveinplace} {
 
-    bind $tv <ButtonPress-3> { clickToEdit %W %x %y }
+    bind $tv <ButtonPress-2> { clickToEdit %W %x %y }
 
     proc clickToEdit {tv x y} {
 	set row [$tv identify row $x $y]
@@ -267,3 +281,52 @@ if {$haveinplace} {
 	}
     }
 }
+
+menu $tv.m -tearoff false -borderwidth 1
+bind $tv <ButtonPress-3> [list column-menu/post $tv $tv.m %x %y]
+
+proc column-menu/post {tv m x y} {
+    upvar #0 $m M
+
+    set M(tv) $tv
+    set M(column) [$tv identify column $x $y]
+    set M(stretch) [$tv column $M(column) -stretch]
+
+    set displaycolumns [$tv cget -displaycolumns]
+    foreach column [$tv cget -columns] {
+	set M(show.$column) [expr {[lsearch $displaycolumns $column] >= 0}]
+    }
+
+    $m delete 0 end
+    $m add checkbutton -label "Stretch?" \
+    	-variable ${m}(stretch) \
+	-command [list column-menu/stretch-changed $m] \
+	;
+    $m add separator
+    foreach column [$tv cget -columns] {
+    	$m add checkbutton \
+	    -label [$tv heading $column -text] \
+	    -variable ${m}(show.$column) \
+	    -command [list column-menu/show-changed $m] \
+	    ;
+    }
+
+    tk_popup $m [winfo pointerx $m] [winfo pointery $m]
+}
+
+proc column-menu/show-changed {m} {
+    upvar #0 $m M
+    set tv [winfo parent $m]
+    set displaycolumns [list]
+    foreach column [$tv cget -columns] {
+	if {$M(show.$column)} {
+	    lappend displaycolumns $column
+	}
+    }
+    $tv configure -displaycolumns $displaycolumns
+}
+
+proc column-menu/stretch-changed {m} {
+    upvar #0 $m M
+    $M(tv) column $M(column) -stretch $M(stretch)
+}
