@@ -8,7 +8,7 @@
 # See the file "license.terms" for information on usage and redistribution
 # of this file, and for a DISCLAIMER OF ALL WARRANTIES.
 # 
-# genStubs.tcl,v 1.10 2004/12/31 01:05:25 jenglish Exp
+# genStubs.tcl,v 1.11 2005/07/13 17:47:39 jenglish Exp
 #
 # SOURCE: tcl/tools/genStubs.tcl, revision 1.17
 #
@@ -24,6 +24,7 @@
 #	  for unused stub entries, in case pointer-to-function and 
 #	  pointer-to-object are different sizes.
 #	+ Allow trailing semicolon in function declarations
+#	+ stubs table is const-qualified
 #
 
 package require Tcl 8
@@ -708,7 +709,7 @@ proc genStubs::emitHeader {name} {
     append text "} ${capName}Stubs;\n"
 
     append text "\n#ifdef __cplusplus\nextern \"C\" {\n#endif\n"
-    append text "extern ${capName}Stubs *${name}StubsPtr;\n"
+    append text "extern const ${capName}Stubs *${name}StubsPtr;\n"
     append text "#ifdef __cplusplus\n}\n#endif\n"
 
     emitMacros $name text
