@@ -1,6 +1,6 @@
 /* winTheme.c - Copyright (C) 2004 Pat Thoyts <patthoyts@users.sf.net>
  *
- * winTheme.c,v 1.36 2006/10/02 22:48:46 jenglish Exp
+ * winTheme.c,v 1.41 2007/12/02 04:34:31 jenglish Exp
  */
 
 #ifdef _MSC_VER
@@ -14,7 +14,7 @@
 #ifdef NO_PRIVATE_HEADERS
 #include <tkPlatDecls.h>
 #include "gunk.h"
-#else 
+#else
 #include <tkWinInt.h>
 #endif
 
@@ -45,7 +45,7 @@ static RECT BoxToRect(Ttk_Box b)
  *	which must be handled specially.
  *
  *	Passing the BF_FLAT flag to DrawEdge() yields something similar
- * 	to TK_RELIEF_SOLID. TK_RELIEF_FLAT can be implemented by not 
+ * 	to TK_RELIEF_SOLID. TK_RELIEF_FLAT can be implemented by not
  *	drawing anything.
  */
 static unsigned int ReliefToEdge(int relief)
@@ -81,7 +81,7 @@ static Ttk_StateTable pushbutton_statemap[] =
     { 0, 0, 0 }
 };
 
-static Ttk_StateTable arrow_statemap[] = 
+static Ttk_StateTable arrow_statemap[] =
 {
     { DFCS_INACTIVE,            TTK_STATE_DISABLED, 0 },
     { DFCS_PUSHED | DFCS_FLAT,  TTK_STATE_PRESSED,  0 },
@@ -100,12 +100,12 @@ typedef struct
     int cxId;			/* system metric id for size in x */
     int cyId;			/* system metric id for size in y */
     Ttk_StateTable *stateMap;	/* map Tk states to Win32 flags */
-    Ttk_Padding padding;	/* additional placement padding */
+    Ttk_Padding margins;	/* additional placement padding */
 } FrameControlElementData;
 
 static FrameControlElementData FrameControlElements[] =
 {
-    { "Checkbutton.indicator", 
+    { "Checkbutton.indicator",
 	DFC_BUTTON, DFCS_BUTTONCHECK, SM_CYMENUCHECK, SM_CYMENUCHECK,
 	checkbutton_statemap, {0,0,4,0} },
     { "Radiobutton.indicator",
@@ -114,10 +114,10 @@ static FrameControlElementData FrameControlElements[] =
     { "uparrow",
     	DFC_SCROLL, DFCS_SCROLLUP, SM_CXVSCROLL, SM_CYVSCROLL,
 	arrow_statemap, {0,0,0,0} },
-    { "downarrow", 
+    { "downarrow",
     	DFC_SCROLL, DFCS_SCROLLDOWN, SM_CXVSCROLL, SM_CYVSCROLL,
 	arrow_statemap, {0,0,0,0} },
-    { "leftarrow", 
+    { "leftarrow",
 	DFC_SCROLL, DFCS_SCROLLLEFT, SM_CXHSCROLL, SM_CYHSCROLL,
 	arrow_statemap, {0,0,0,0} },
     { "rightarrow",
@@ -132,14 +132,13 @@ static FrameControlElementData FrameControlElements[] =
 
 /* ---------------------------------------------------------------------- */
 
-static void FrameControlElementGeometry(
+static void FrameControlElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
-    FrameControlElementData *elementData = clientData;
-    *widthPtr = GetSystemMetrics(elementData->cxId);
-    *heightPtr = GetSystemMetrics(elementData->cyId);
-    *paddingPtr = elementData->padding;
+    FrameControlElementData *p = clientData;
+    *widthPtr = GetSystemMetrics(p->cxId) + Ttk_PaddingWidth(p->margins);
+    *heightPtr = GetSystemMetrics(p->cyId) + Ttk_PaddingHeight(p->margins);
 }
 
 static void FrameControlElementDraw(
@@ -157,12 +156,12 @@ static void FrameControlElementDraw(
     TkWinReleaseDrawableDC(d, hdc, &dcState);
 }
 
-static Ttk_ElementSpec FrameControlElementSpec = 
+static Ttk_ElementSpec FrameControlElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(NullElement),
-    NullElementOptions,
-    FrameControlElementGeometry,
+    TtkNullElementOptions,
+    FrameControlElementSize,
     FrameControlElementDraw
 };
 
@@ -179,7 +178,7 @@ static Ttk_ElementOptionSpec BorderElementOptions[] = {
     {NULL}
 };
 
-static void BorderElementGeometry(
+static void BorderElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -212,7 +211,7 @@ static Ttk_ElementSpec BorderElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(BorderElement),
     BorderElementOptions,
-    BorderElementGeometry,
+    BorderElementSize,
     BorderElementDraw
 };
 
@@ -228,13 +227,12 @@ typedef struct
 
 static Ttk_ElementOptionSpec FieldElementOptions[] =
 {
-    { "-fieldbackground", TK_OPTION_BORDER, 
+    { "-fieldbackground", TK_OPTION_BORDER,
     	Tk_Offset(FieldElement,backgroundObj), "white" },
     {NULL}
 };
 
-static void
-FieldElementGeometry(
+static void FieldElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -242,8 +240,7 @@ FieldElementGeometry(
     paddingPtr->top = paddingPtr->bottom = GetSystemMetrics(SM_CYEDGE);
 }
 
-static void
-FieldElementDraw(
+static void FieldElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     Drawable d, Ttk_Box b, unsigned int state)
 {
@@ -266,7 +263,7 @@ static Ttk_ElementSpec FieldElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(FieldElement),
     FieldElementOptions,
-    FieldElementGeometry,
+    FieldElementSize,
     FieldElementDraw
 };
 
@@ -286,12 +283,12 @@ static Ttk_ElementOptionSpec ButtonBorderElementOptions[] = {
 	Tk_Offset(ButtonBorderElement,reliefObj), "flat" },
     { "-highlightcolor",TK_OPTION_COLOR,
 	Tk_Offset(ButtonBorderElement,highlightColorObj), "black" },
-    { "-default", TK_OPTION_ANY, 
+    { "-default", TK_OPTION_ANY,
 	Tk_Offset(ButtonBorderElement,defaultStateObj), "disabled" },
     {NULL}
 };
 
-static void ButtonBorderElementGeometry(
+static void ButtonBorderElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -365,7 +362,7 @@ static Ttk_ElementSpec ButtonBorderElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(ButtonBorderElement),
     ButtonBorderElementOptions,
-    ButtonBorderElementGeometry,
+    ButtonBorderElementSize,
     ButtonBorderElementDraw
 };
 
@@ -374,7 +371,7 @@ static Ttk_ElementSpec ButtonBorderElementSpec =
  * 	Draw dashed focus rectangle.
  */
 
-static void FocusElementGeometry(
+static void FocusElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -398,8 +395,8 @@ static Ttk_ElementSpec FocusElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(NullElement),
-    NullElementOptions,
-    FocusElementGeometry,
+    TtkNullElementOptions,
+    FocusElementSize,
     FocusElementDraw
 };
 
@@ -455,7 +452,7 @@ static Ttk_ElementSpec ComboboxFocusElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(FillFocusElement),
     FillFocusElementOptions,
-    FocusElementGeometry,
+    FocusElementSize,
     ComboboxFocusElementDraw
 };
 
@@ -525,8 +522,8 @@ static Ttk_ElementSpec TroughElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(NullElement),
-    NullElementOptions,
-    NullElementGeometry,
+    TtkNullElementOptions,
+    TtkNullElementSize,
     TroughElementDraw
 };
 
@@ -545,7 +542,7 @@ static Ttk_ElementOptionSpec ThumbElementOptions[] =
     { NULL }
 };
 
-static void ThumbElementGeometry(
+static void ThumbElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -584,11 +581,11 @@ static Ttk_ElementSpec ThumbElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(ThumbElement),
     ThumbElementOptions,
-    ThumbElementGeometry,
+    ThumbElementSize,
     ThumbElementDraw
 };
 
-/* ---------------------------------------------------------------------- 
+/* ----------------------------------------------------------------------
  * The slider element is the shaped thumb used in the slider widget.
  * Windows likes to call this a trackbar.
  */
@@ -605,7 +602,7 @@ static Ttk_ElementOptionSpec SliderElementOptions[] =
     { NULL }
 };
 
-static void SliderElementGeometry(
+static void SliderElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -640,7 +637,7 @@ static Ttk_ElementSpec SliderElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(SliderElement),
     SliderElementOptions,
-    SliderElementGeometry,
+    SliderElementSize,
     SliderElementDraw
 };
 
@@ -648,7 +645,7 @@ static Ttk_ElementSpec SliderElementSpec =
  * +++ Notebook elements.
  */
 
-static void ClientElementGeometry(
+static void ClientElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -671,8 +668,8 @@ static Ttk_ElementSpec ClientElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(NullElement),
-    NullElementOptions,
-    ClientElementGeometry,
+    TtkNullElementOptions,
+    ClientElementSize,
     ClientElementDraw
 };
 
@@ -680,24 +677,26 @@ static Ttk_ElementSpec ClientElementSpec =
  * +++ Layouts.
  */
 
-TTK_BEGIN_LAYOUT(ButtonLayout)
+TTK_BEGIN_LAYOUT_TABLE(LayoutTable)
+
+TTK_LAYOUT("TButton",
     TTK_GROUP("Button.border", TTK_FILL_BOTH,
 	TTK_GROUP("Button.padding", TTK_FILL_BOTH,
-	    TTK_NODE("Button.label", TTK_FILL_BOTH)))
-TTK_END_LAYOUT
+	    TTK_NODE("Button.label", TTK_FILL_BOTH))))
 
-TTK_BEGIN_LAYOUT(ComboboxLayout)
+TTK_LAYOUT("TCombobox",
     TTK_GROUP("Combobox.field", TTK_FILL_BOTH,
 	TTK_NODE("Combobox.downarrow", TTK_PACK_RIGHT|TTK_FILL_Y)
 	TTK_GROUP("Combobox.padding", TTK_PACK_LEFT|TTK_EXPAND|TTK_FILL_BOTH,
-	    TTK_GROUP("Combobox.focus", TTK_PACK_LEFT|TTK_EXPAND|TTK_FILL_BOTH, 
-		TTK_NODE("Combobox.textarea", TTK_FILL_BOTH))))
-TTK_END_LAYOUT
+	    TTK_GROUP("Combobox.focus", TTK_PACK_LEFT|TTK_EXPAND|TTK_FILL_BOTH,
+		TTK_NODE("Combobox.textarea", TTK_FILL_BOTH)))))
 
+TTK_END_LAYOUT_TABLE
 
 /* ---------------------------------------------------------------------- */
 
-int WinTheme_Init(Tcl_Interp *interp, HWND hwnd)
+MODULE_SCOPE
+int TtkWinTheme_Init(Tcl_Interp *interp, HWND hwnd)
 {
     Ttk_Theme themePtr, parentPtr;
     FrameControlElementData *fce = FrameControlElements;
@@ -709,28 +708,27 @@ int WinTheme_Init(Tcl_Interp *interp, HWND hwnd)
     }
 
     Ttk_RegisterElementSpec(themePtr, "border", &BorderElementSpec, NULL);
-    Ttk_RegisterElementSpec(themePtr, "Button.border", 
+    Ttk_RegisterElementSpec(themePtr, "Button.border",
 	&ButtonBorderElementSpec, NULL);
     Ttk_RegisterElementSpec(themePtr, "field", &FieldElementSpec, NULL);
     Ttk_RegisterElementSpec(themePtr, "focus", &FocusElementSpec, NULL);
-    Ttk_RegisterElementSpec(themePtr, "Combobox.focus", 
+    Ttk_RegisterElementSpec(themePtr, "Combobox.focus",
     	&ComboboxFocusElementSpec, NULL);
     Ttk_RegisterElementSpec(themePtr, "thumb", &ThumbElementSpec, NULL);
     Ttk_RegisterElementSpec(themePtr, "slider", &SliderElementSpec, NULL);
-    Ttk_RegisterElementSpec(themePtr, "Scrollbar.trough", &TroughElementSpec, 
+    Ttk_RegisterElementSpec(themePtr, "Scrollbar.trough", &TroughElementSpec,
     	TroughClientDataInit(interp));
 
     Ttk_RegisterElementSpec(themePtr, "client", &ClientElementSpec, NULL);
 
     for (fce = FrameControlElements; fce->name != 0; ++fce) {
-	Ttk_RegisterElementSpec(themePtr, fce->name, 
+	Ttk_RegisterElementSpec(themePtr, fce->name,
 		&FrameControlElementSpec, fce);
     }
 
-    Ttk_RegisterLayout(themePtr, "TButton", ButtonLayout);
-    Ttk_RegisterLayout(themePtr, "TCombobox", ComboboxLayout);
+    Ttk_RegisterLayouts(themePtr, LayoutTable);
 
-    Tcl_PkgProvide(interp, "tile::theme::winnative", TILE_VERSION);
-    return TCL_OK; 
+    Tcl_PkgProvide(interp, "ttk::theme::winnative", TILE_VERSION);
+    return TCL_OK;
 }
 
