@@ -1,5 +1,5 @@
 /*
- * xpTheme.c,v 1.82 2006/02/05 21:29:18 jenglish Exp
+ * xpTheme.c,v 1.94 2007/12/02 04:34:31 jenglish Exp
  *
  * Tk theme engine which uses the Windows XP "Visual Styles" API
  * Adapted from Georgios Petasis' XP theme patch.
@@ -45,36 +45,31 @@ typedef HRESULT (STDAPICALLTYPE CloseThemeDataProc)(HTHEME hTheme);
 typedef HRESULT (STDAPICALLTYPE DrawThemeBackgroundProc)(HTHEME hTheme,
                  HDC hdc, int iPartId, int iStateId, const RECT *pRect,
                  OPTIONAL const RECT *pClipRect);
-typedef HRESULT (STDAPICALLTYPE DrawThemeParentBackgroundProc)(HWND hwnd,
-                 HDC hdc, OPTIONAL const RECT *prc);
-typedef HRESULT (STDAPICALLTYPE DrawThemeEdgeProc)(HTHEME hTheme, HDC hdc,
-		 int iPartId, int iStateId, const RECT *pDestRect,
-		 UINT uEdge, UINT uFlags, RECT *pContentRect);
-typedef HRESULT (STDAPICALLTYPE DrawThemeTextProc)(HTHEME hTheme, HDC hdc,
-		 int iPartId, int iStateId, LPCWSTR pszText, int iCharCount,
-		 DWORD dwTextFlags, DWORD dwTextFlags2, const RECT *pRect);
-typedef HRESULT	(STDAPICALLTYPE GetThemeMarginsProc)(HTHEME, HDC,
-		 int iPartId, int iStateId, int iPropId,
-		 OPTIONAL RECT *prc, MARGINS *pMargins);
 typedef HRESULT	(STDAPICALLTYPE GetThemePartSizeProc)(HTHEME,HDC,
 		 int iPartId, int iStateId,
 		 RECT *prc, enum THEMESIZE eSize, SIZE *psz);
+typedef int     (STDAPICALLTYPE GetThemeSysSizeProc)(HTHEME,int);
+/* GetThemeTextExtent and DrawThemeText only used with BROKEN_TEXT_ELEMENT */ 
 typedef HRESULT (STDAPICALLTYPE GetThemeTextExtentProc)(HTHEME hTheme, HDC hdc,
 		 int iPartId, int iStateId, LPCWSTR pszText, int iCharCount,
-		 DWORD dwTextFlags, const RECT *pBoundingRect, RECT *pExtentRect);
+		 DWORD dwTextFlags, const RECT *pBoundingRect, RECT *pExtent);
+typedef HRESULT (STDAPICALLTYPE DrawThemeTextProc)(HTHEME hTheme, HDC hdc,
+		 int iPartId, int iStateId, LPCWSTR pszText, int iCharCount,
+		 DWORD dwTextFlags, DWORD dwTextFlags2, const RECT *pRect);
 typedef BOOL    (STDAPICALLTYPE IsThemeActiveProc)(VOID);
+typedef BOOL    (STDAPICALLTYPE IsAppThemedProc)(VOID);
 
 typedef struct
 {
     OpenThemeDataProc			*OpenThemeData;
     CloseThemeDataProc			*CloseThemeData;
-    DrawThemeBackgroundProc		*DrawThemeBackground;
-    DrawThemeParentBackgroundProc	*DrawThemeParentBackground;
-    DrawThemeEdgeProc			*DrawThemeEdge;
-    DrawThemeTextProc		        *DrawThemeText;
     GetThemePartSizeProc		*GetThemePartSize;
+    GetThemeSysSizeProc			*GetThemeSysSize;
+    DrawThemeBackgroundProc		*DrawThemeBackground;
+    DrawThemeTextProc		        *DrawThemeText;
     GetThemeTextExtentProc		*GetThemeTextExtent;
-    IsThemeActiveProc                   *IsThemeActive;
+    IsThemeActiveProc			*IsThemeActive;
+    IsAppThemedProc			*IsAppThemed;
 
     HWND                                stubWindow;
 } XPThemeProcs;
@@ -102,51 +97,38 @@ typedef struct
 static XPThemeProcs *
 LoadXPThemeProcs(HINSTANCE *phlib)
 {
-    OSVERSIONINFO os;
-
     /*
-     * We have to check whether we are running at least on Windows XP.
-     * In order to determine this we call GetVersionEx directly, although
-     * it would be a good idea to wrap it inside a function similar to
-     * TkWinGetPlatformId...
+     * Load the library "uxtheme.dll", where the native widget
+     * drawing routines are implemented.  This will only succeed 
+     * if we are running at least on Windows XP.
      */
-    ZeroMemory(&os, sizeof(os));
-    os.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
-    GetVersionEx(&os);
-    if (os.dwMajorVersion >= 5 && os.dwMinorVersion >= 1) {
+    HINSTANCE handle;
+    *phlib = handle = LoadLibrary("uxtheme.dll");
+    if (handle != 0)
+    {
 	/*
-	 * We are running under Windows XP or a newer version.
-	 * Load the library "uxtheme.dll", where the native widget
-	 * drawing routines are implemented.
+	 * We have successfully loaded the library. Proceed in storing the
+	 * addresses of the functions we want to use.
 	 */
-	HINSTANCE handle;
-	*phlib = handle = LoadLibrary("uxtheme.dll");
-	if (handle != 0)
-	{
-	    /*
-	     * We have successfully loaded the library. Proceed in storing the
-	     * addresses of the functions we want to use.
-	     */
-	    XPThemeProcs *procs = (XPThemeProcs*)ckalloc(sizeof(XPThemeProcs));
+	XPThemeProcs *procs = (XPThemeProcs*)ckalloc(sizeof(XPThemeProcs));
 #define LOADPROC(name) \
 	(0 != (procs->name = (name ## Proc *)GetProcAddress(handle, #name) ))
 
-	    if (   LOADPROC(OpenThemeData)
-		&& LOADPROC(CloseThemeData)
-		&& LOADPROC(DrawThemeBackground)
-		&& LOADPROC(DrawThemeParentBackground)
-		&& LOADPROC(DrawThemeEdge)
-		&& LOADPROC(DrawThemeText)
-		&& LOADPROC(GetThemePartSize)
-		&& LOADPROC(GetThemeTextExtent)
-		&& LOADPROC(IsThemeActive)
-	    )
-	    {
-		return procs;
-	    }
-#undef LOADPROC
-	    ckfree((char*)procs);
+	if (   LOADPROC(OpenThemeData)
+	    && LOADPROC(CloseThemeData)
+	    && LOADPROC(GetThemePartSize)
+	    && LOADPROC(GetThemeSysSize)
+	    && LOADPROC(DrawThemeBackground)
+	    && LOADPROC(GetThemeTextExtent)
+	    && LOADPROC(DrawThemeText)
+	    && LOADPROC(IsThemeActive)
+	    && LOADPROC(IsAppThemed)
+	)
+	{
+	    return procs;
 	}
+#undef LOADPROC
+	ckfree((char*)procs);
     }
     return 0;
 }
@@ -169,7 +151,9 @@ static int
 XPThemeEnabled(Ttk_Theme theme, void *clientData)
 {
     XPThemeData *themeData = clientData;
-    return themeData->procs->IsThemeActive();
+    int active = themeData->procs->IsThemeActive();
+    int themed = themeData->procs->IsAppThemed();
+    return (active && themed);
 }
 
 /*
@@ -206,18 +190,20 @@ static Ttk_StateTable pushbutton_statemap[] =
 
 /*
  * Checkboxes (Tk: "Checkbutton")
- *
- * Missing: CBS_MIXEDDISABLED CBS_MIXEDHOT CBS_MIXEDNORMAL CBS_MIXEDPRESSED
  */
 static Ttk_StateTable checkbox_statemap[] =
 {
-{CBS_CHECKEDDISABLED,	TTK_STATE_DISABLED|TTK_STATE_SELECTED, 0},
-{CBS_CHECKEDPRESSED,	TTK_STATE_PRESSED|TTK_STATE_SELECTED, 0},
-{CBS_CHECKEDHOT,	TTK_STATE_ACTIVE|TTK_STATE_SELECTED, 0},
+{CBS_MIXEDDISABLED, 	TTK_STATE_ALTERNATE|TTK_STATE_DISABLED, 0},
+{CBS_MIXEDPRESSED, 	TTK_STATE_ALTERNATE|TTK_STATE_PRESSED, 0},
+{CBS_MIXEDHOT,  	TTK_STATE_ALTERNATE|TTK_STATE_ACTIVE, 0},
+{CBS_MIXEDNORMAL, 	TTK_STATE_ALTERNATE, 0},
+{CBS_CHECKEDDISABLED,	TTK_STATE_SELECTED|TTK_STATE_DISABLED, 0},
+{CBS_CHECKEDPRESSED,	TTK_STATE_SELECTED|TTK_STATE_PRESSED, 0},
+{CBS_CHECKEDHOT,	TTK_STATE_SELECTED|TTK_STATE_ACTIVE, 0},
 {CBS_CHECKEDNORMAL,	TTK_STATE_SELECTED, 0},
-{CBS_UNCHECKEDDISABLED,	TTK_STATE_DISABLED, TTK_STATE_SELECTED},
-{CBS_UNCHECKEDPRESSED,	TTK_STATE_PRESSED, TTK_STATE_SELECTED},
-{CBS_UNCHECKEDHOT,	TTK_STATE_ACTIVE, TTK_STATE_SELECTED},
+{CBS_UNCHECKEDDISABLED,	TTK_STATE_DISABLED, 0},
+{CBS_UNCHECKEDPRESSED,	TTK_STATE_PRESSED, 0},
+{CBS_UNCHECKEDHOT,	TTK_STATE_ACTIVE, 0},
 {CBS_UNCHECKEDNORMAL,	0,0 }
 };
 
@@ -226,13 +212,13 @@ static Ttk_StateTable checkbox_statemap[] =
  */
 static Ttk_StateTable radiobutton_statemap[] =
 {
-{RBS_CHECKEDDISABLED,	TTK_STATE_DISABLED|TTK_STATE_SELECTED, 0},
-{RBS_CHECKEDPRESSED,	TTK_STATE_PRESSED|TTK_STATE_SELECTED, 0},
-{RBS_CHECKEDHOT,	TTK_STATE_ACTIVE|TTK_STATE_SELECTED, 0},
+{RBS_CHECKEDDISABLED,	TTK_STATE_SELECTED|TTK_STATE_DISABLED, 0},
+{RBS_CHECKEDPRESSED,	TTK_STATE_SELECTED|TTK_STATE_PRESSED, 0},
+{RBS_CHECKEDHOT,	TTK_STATE_SELECTED|TTK_STATE_ACTIVE, 0},
 {RBS_CHECKEDNORMAL,	TTK_STATE_SELECTED, 0},
-{RBS_UNCHECKEDDISABLED,	TTK_STATE_DISABLED, TTK_STATE_SELECTED},
-{RBS_UNCHECKEDPRESSED,	TTK_STATE_PRESSED, TTK_STATE_SELECTED},
-{RBS_UNCHECKEDHOT,	TTK_STATE_ACTIVE, TTK_STATE_SELECTED},
+{RBS_UNCHECKEDDISABLED,	TTK_STATE_DISABLED, 0},
+{RBS_UNCHECKEDPRESSED,	TTK_STATE_PRESSED, 0},
+{RBS_UNCHECKEDHOT,	TTK_STATE_ACTIVE, 0},
 {RBS_UNCHECKEDNORMAL,	0,0 }
 };
 
@@ -372,6 +358,9 @@ static Ttk_StateTable tabitem_statemap[] =
  *	only seems to work properly for BP_PUSHBUTTON.  So we hardcode
  *	the required padding at element registration time instead.
  *
+ *	The PAD_MARGINS flag bit determines whether the padding
+ *	should be added on the inside (0) or outside (1) of the element.
+ *
  * <<NOTE-GetThemePartSize>>: 
  *	This gives bogus metrics for some parts (in particular,
  *	BP_PUSHBUTTONS).  Set the IGNORE_THEMESIZE flag to skip this call.
@@ -387,7 +376,8 @@ typedef struct 	/* XP element specifications */
     Ttk_StateTable *statemap;	/* Map Tk states to XP states */
     Ttk_Padding	padding;	/* See NOTE-GetThemeMargins */
     int  	flags;		
-#   define 	IGNORE_THEMESIZE 0x1	/* See NOTE-GetThemePartSize */
+#   define 	IGNORE_THEMESIZE 0x80000000 /* See NOTE-GetThemePartSize */
+#   define 	PAD_MARGINS	 0x40000000 /* See NOTE-GetThemeMargins */
 } ElementInfo;
 
 typedef struct
@@ -481,8 +471,7 @@ FreeElementData(ElementData *elementData)
  * such as radiobutton and checkbutton indicators, scrollbar arrows, etc.
  */
 
-static void
-GenericElementGeometry(
+static void GenericElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -512,18 +501,27 @@ GenericElementGeometry(
     /* See NOTE-GetThemeMargins 
      */
     *paddingPtr = elementData->info->padding;
+    if (elementData->info->flags & PAD_MARGINS) {
+	*widthPtr += Ttk_PaddingWidth(elementData->info->padding);
+	*heightPtr += Ttk_PaddingHeight(elementData->info->padding);
+    }
 }
 
-static void
-GenericElementDraw(
+static void GenericElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     Drawable d, Ttk_Box b, unsigned int state)
 {
     ElementData *elementData = clientData;
-    RECT rc = BoxToRect(b);
+    RECT rc;
 
-    if (!InitElementData(elementData, tkwin, d))
+    if (!InitElementData(elementData, tkwin, d)) {
 	return;
+    }
+
+    if (elementData->info->flags & PAD_MARGINS) {
+    	b = Ttk_PadBox(b, elementData->info->padding);
+    }
+    rc = BoxToRect(b);
 
     elementData->procs->DrawThemeBackground(
 	elementData->hTheme,
@@ -540,8 +538,43 @@ static Ttk_ElementSpec GenericElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(NullElement),
-    NullElementOptions,
-    GenericElementGeometry,
+    TtkNullElementOptions,
+    GenericElementSize,
+    GenericElementDraw
+};
+
+/*----------------------------------------------------------------------
+ * +++ Sized element implementation.
+ * 
+ * Used for elements which are handled entirely by the XP Theme API,
+ * but that require a fixed size adjustment.
+ * Note that GetThemeSysSize calls through to GetSystemMetrics
+ */
+
+static void
+GenericSizedElementSize(
+    void *clientData, void *elementRecord, Tk_Window tkwin,
+    int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
+{
+    ElementData *elementData = clientData;
+
+    if (!InitElementData(elementData, tkwin, 0))
+	return;
+
+    GenericElementSize(clientData, elementRecord, tkwin,
+	widthPtr, heightPtr, paddingPtr);
+
+    *widthPtr = elementData->procs->GetThemeSysSize(NULL,
+	(elementData->info->flags >> 8) & 0xff);
+    *heightPtr = elementData->procs->GetThemeSysSize(NULL,
+	elementData->info->flags & 0xff);
+}
+
+static Ttk_ElementSpec GenericSizedElementSpec = {
+    TK_STYLE_VERSION_2,
+    sizeof(NullElement),
+    TtkNullElementOptions,
+    GenericSizedElementSize,
     GenericElementDraw
 };
 
@@ -550,8 +583,7 @@ static Ttk_ElementSpec GenericElementSpec =
  *     Same as a GenericElement, but don't draw in the disabled state.
  */
 
-static void
-ThumbElementDraw(
+static void ThumbElementDraw(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     Drawable d, Ttk_Box b, unsigned int state)
 {
@@ -579,8 +611,8 @@ static Ttk_ElementSpec ThumbElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(NullElement),
-    NullElementOptions,
-    GenericElementGeometry,
+    TtkNullElementOptions,
+    GenericElementSize,
     ThumbElementDraw
 };
 
@@ -590,14 +622,14 @@ static Ttk_ElementSpec ThumbElementSpec =
  *	so that indeterminate progress bars show 3 bars instead of 1.   
  */
 
-static void PbarElementGeometry(
+static void PbarElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
     ElementData *elementData = clientData;
     int nBars = 3;
 
-    GenericElementGeometry(clientData, elementRecord, tkwin,
+    GenericElementSize(clientData, elementRecord, tkwin,
     	widthPtr, heightPtr, paddingPtr);
 
     if (elementData->info->partId == PP_CHUNK) {
@@ -611,8 +643,8 @@ static Ttk_ElementSpec PbarElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(NullElement),
-    NullElementOptions,
-    PbarElementGeometry,
+    TtkNullElementOptions,
+    PbarElementSize,
     GenericElementDraw
 };
 
@@ -652,8 +684,8 @@ static Ttk_ElementSpec TabElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(NullElement),
-    NullElementOptions,
-    GenericElementGeometry,
+    TtkNullElementOptions,
+    GenericElementSize,
     TabElementDraw
 };
 
@@ -692,8 +724,8 @@ static Ttk_ElementSpec TreeIndicatorElementSpec =
 {
     TK_STYLE_VERSION_2,
     sizeof(NullElement),
-    NullElementOptions,
-    GenericElementGeometry,
+    TtkNullElementOptions,
+    GenericElementSize,
     TreeIndicatorElementDraw
 };
 
@@ -725,8 +757,7 @@ static Ttk_ElementOptionSpec TextElementOptions[] =
     { NULL }
 };
 
-static void
-TextElementGeometry(
+static void TextElementSize(
     void *clientData, void *elementRecord, Tk_Window tkwin,
     int *widthPtr, int *heightPtr, Ttk_Padding *paddingPtr)
 {
@@ -759,8 +790,7 @@ TextElementGeometry(
     FreeElementData(elementData);
 }
 
-static void 
-TextElementDraw(
+static void TextElementDraw(
     ClientData clientData, void *elementRecord, Tk_Window tkwin,
     Drawable d, Ttk_Box b, unsigned int state)
 {
@@ -790,7 +820,7 @@ static Ttk_ElementSpec TextElementSpec =
     TK_STYLE_VERSION_2,
     sizeof(TextElement),
     TextElementOptions,
-    TextElementGeometry,
+    TextElementSize,
     TextElementDraw
 };
 
@@ -800,49 +830,47 @@ static Ttk_ElementSpec TextElementSpec =
  * +++ Widget layouts:
  */
 
-TTK_BEGIN_LAYOUT(ButtonLayout)
+TTK_BEGIN_LAYOUT_TABLE(LayoutTable)
+
+TTK_LAYOUT("TButton",
     TTK_GROUP("Button.button", TTK_FILL_BOTH,
 	TTK_GROUP("Button.focus", TTK_FILL_BOTH, 
 	    TTK_GROUP("Button.padding", TTK_FILL_BOTH,
-		TTK_NODE("Button.label", TTK_FILL_BOTH))))
-TTK_END_LAYOUT
+		TTK_NODE("Button.label", TTK_FILL_BOTH)))))
 
-TTK_BEGIN_LAYOUT(MenubuttonLayout)
+TTK_LAYOUT("TMenubutton",
     TTK_NODE("Menubutton.dropdown", TTK_PACK_RIGHT|TTK_FILL_Y)
     TTK_GROUP("Menubutton.button", TTK_PACK_RIGHT|TTK_EXPAND|TTK_FILL_BOTH,
 	    TTK_GROUP("Menubutton.padding", TTK_PACK_LEFT|TTK_EXPAND|TTK_FILL_X,
-	        TTK_NODE("Menubutton.label", 0)))
-TTK_END_LAYOUT
+	        TTK_NODE("Menubutton.label", 0))))
 
-TTK_BEGIN_LAYOUT(HorizontalScrollbarLayout)
+TTK_LAYOUT("Horizontal.TScrollbar",
     TTK_GROUP("Horizontal.Scrollbar.trough", TTK_FILL_X,
 	TTK_NODE("Horizontal.Scrollbar.leftarrow", TTK_PACK_LEFT)
 	TTK_NODE("Horizontal.Scrollbar.rightarrow", TTK_PACK_RIGHT)
 	TTK_GROUP("Horizontal.Scrollbar.thumb", TTK_FILL_BOTH|TTK_UNIT,
-	    TTK_NODE("Horizontal.Scrollbar.grip", 0)))
-TTK_END_LAYOUT
+	    TTK_NODE("Horizontal.Scrollbar.grip", 0))))
 
-TTK_BEGIN_LAYOUT(VerticalScrollbarLayout)
+TTK_LAYOUT("Vertical.TScrollbar",
     TTK_GROUP("Vertical.Scrollbar.trough", TTK_FILL_Y,
 	TTK_NODE("Vertical.Scrollbar.uparrow", TTK_PACK_TOP)
 	TTK_NODE("Vertical.Scrollbar.downarrow", TTK_PACK_BOTTOM)
 	TTK_GROUP("Vertical.Scrollbar.thumb", TTK_FILL_BOTH|TTK_UNIT,
-	    TTK_NODE("Vertical.Scrollbar.grip", 0)))
-TTK_END_LAYOUT
+	    TTK_NODE("Vertical.Scrollbar.grip", 0))))
 
-TTK_BEGIN_LAYOUT(VerticalScaleLayout)
-    TTK_GROUP("Scale.focus", TTK_EXPAND|TTK_FILL_BOTH,
-	TTK_GROUP("Vertical.Scale.trough", TTK_EXPAND|TTK_FILL_BOTH,
-	    TTK_NODE("Vertical.Scale.track", TTK_FILL_Y)
-	    TTK_NODE("Vertical.Scale.slider", TTK_PACK_TOP) ))
-TTK_END_LAYOUT
-
-TTK_BEGIN_LAYOUT(HorizontalScaleLayout)
+TTK_LAYOUT("Horizontal.TScale",
     TTK_GROUP("Scale.focus", TTK_EXPAND|TTK_FILL_BOTH,
 	TTK_GROUP("Horizontal.Scale.trough", TTK_EXPAND|TTK_FILL_BOTH,
 	    TTK_NODE("Horizontal.Scale.track", TTK_FILL_X) 
-	    TTK_NODE("Horizontal.Scale.slider", TTK_PACK_LEFT) ))
-TTK_END_LAYOUT
+	    TTK_NODE("Horizontal.Scale.slider", TTK_PACK_LEFT) )))
+
+TTK_LAYOUT("Vertical.TScale",
+    TTK_GROUP("Scale.focus", TTK_EXPAND|TTK_FILL_BOTH,
+	TTK_GROUP("Vertical.Scale.trough", TTK_EXPAND|TTK_FILL_BOTH,
+	    TTK_NODE("Vertical.Scale.track", TTK_FILL_Y)
+	    TTK_NODE("Vertical.Scale.slider", TTK_PACK_TOP) )))
+
+TTK_END_LAYOUT_TABLE
 
 /*----------------------------------------------------------------------
  * +++ XP element info table: 
@@ -855,9 +883,9 @@ TTK_END_LAYOUT
 
 static ElementInfo ElementInfoTable[] = {
     { "Checkbutton.indicator", &GenericElementSpec, L"BUTTON",
-    	BP_CHECKBOX, checkbox_statemap, PAD(0, 0, 4, 0), 0 },
+    	BP_CHECKBOX, checkbox_statemap, PAD(0, 0, 4, 0), PAD_MARGINS },
     { "Radiobutton.indicator", &GenericElementSpec, L"BUTTON",
-    	BP_RADIOBUTTON, radiobutton_statemap, PAD(0, 0, 4, 0), 0 },
+    	BP_RADIOBUTTON, radiobutton_statemap, PAD(0, 0, 4, 0), PAD_MARGINS },
     { "Button.button", &GenericElementSpec, L"BUTTON",
     	BP_PUSHBUTTON, pushbutton_statemap, PAD(3, 3, 3, 3), IGNORE_THEMESIZE },
     { "Labelframe.border", &GenericElementSpec, L"BUTTON",
@@ -866,8 +894,9 @@ static ElementInfo ElementInfoTable[] = {
     	edittext_statemap, PAD(1, 1, 1, 1), 0 },
     { "Combobox.field", &GenericElementSpec, L"EDIT",
 	EP_EDITTEXT, combotext_statemap, PAD(1, 1, 1, 1), 0 },
-    { "Combobox.downarrow", &GenericElementSpec, L"COMBOBOX",
-	CP_DROPDOWNBUTTON, combobox_statemap, NOPAD, 0 },
+    { "Combobox.downarrow", &GenericSizedElementSpec, L"COMBOBOX",
+	CP_DROPDOWNBUTTON, combobox_statemap, NOPAD,
+	(SM_CXVSCROLL << 8) | SM_CYVSCROLL },
     { "Vertical.Scrollbar.trough", &GenericElementSpec, L"SCROLLBAR",
     	SBP_UPPERTRACKVERT, scrollbar_statemap, NOPAD, 0 },
     { "Vertical.Scrollbar.thumb", &ThumbElementSpec, L"SCROLLBAR",
@@ -880,14 +909,18 @@ static ElementInfo ElementInfoTable[] = {
    	SBP_THUMBBTNHORZ, scrollbar_statemap, NOPAD, 0 },
     { "Horizontal.Scrollbar.grip", &GenericElementSpec, L"SCROLLBAR",
     	SBP_GRIPPERHORZ, scrollbar_statemap, NOPAD, 0 },
-    { "Scrollbar.uparrow", &GenericElementSpec, L"SCROLLBAR",
-    	SBP_ARROWBTN, uparrow_statemap, NOPAD, 0 },
-    { "Scrollbar.downarrow", &GenericElementSpec, L"SCROLLBAR",
-    	SBP_ARROWBTN, downarrow_statemap, NOPAD, 0 },
-    { "Scrollbar.leftarrow", &GenericElementSpec, L"SCROLLBAR",
-    	SBP_ARROWBTN, leftarrow_statemap, NOPAD, 0 },
-    { "Scrollbar.rightarrow", &GenericElementSpec, L"SCROLLBAR",
-    	SBP_ARROWBTN, rightarrow_statemap, NOPAD, 0 },
+    { "Scrollbar.uparrow", &GenericSizedElementSpec, L"SCROLLBAR",
+    	SBP_ARROWBTN, uparrow_statemap, NOPAD,
+	(SM_CXVSCROLL << 8) | SM_CYVSCROLL },
+    { "Scrollbar.downarrow", &GenericSizedElementSpec, L"SCROLLBAR",
+    	SBP_ARROWBTN, downarrow_statemap, NOPAD,
+	(SM_CXVSCROLL << 8) | SM_CYVSCROLL },
+    { "Scrollbar.leftarrow", &GenericSizedElementSpec, L"SCROLLBAR",
+    	SBP_ARROWBTN, leftarrow_statemap, NOPAD,
+	(SM_CXHSCROLL << 8) | SM_CYHSCROLL },
+    { "Scrollbar.rightarrow", &GenericSizedElementSpec, L"SCROLLBAR",
+    	SBP_ARROWBTN, rightarrow_statemap, NOPAD,
+	(SM_CXHSCROLL << 8) | SM_CYHSCROLL },
     { "Horizontal.Scale.slider", &GenericElementSpec, L"TRACKBAR",
     	TKP_THUMB, scale_statemap, NOPAD, 0 },
     { "Vertical.Scale.slider", &GenericElementSpec, L"TRACKBAR",
@@ -919,7 +952,7 @@ static ElementInfo ElementInfoTable[] = {
     { "Menubutton.dropdown", &GenericElementSpec, L"TOOLBAR",
     	TP_SPLITBUTTONDROPDOWN,toolbutton_statemap, NOPAD,0 },
     { "Treeitem.indicator", &TreeIndicatorElementSpec, L"TREEVIEW",
-    	TVP_GLYPH, tvpglyph_statemap, PAD(1,1,6,0), 0 },
+    	TVP_GLYPH, tvpglyph_statemap, PAD(1,1,6,0), PAD_MARGINS },
     { "Treeheading.border", &GenericElementSpec, L"HEADER", 
     	HP_HEADERITEM, header_statemap, PAD(4,0,4,0),0 },
     { "sizegrip", &GenericElementSpec, L"STATUS", 
@@ -938,7 +971,7 @@ static ElementInfo ElementInfoTable[] = {
  * +++ Initialization routine:
  */
 
-int XPTheme_Init(Tcl_Interp *interp, HWND hwnd)
+int TtkXPTheme_Init(Tcl_Interp *interp, HWND hwnd)
 {
     XPThemeData *themeData;
     XPThemeProcs *procs;
@@ -981,21 +1014,14 @@ int XPTheme_Init(Tcl_Interp *interp, HWND hwnd)
 	Ttk_RegisterCleanup(interp, clientData, DestroyElementData);
     }
 
-    Ttk_RegisterElementSpec(themePtr, "Scale.trough", &NullElementSpec, 0);
+    Ttk_RegisterElementSpec(themePtr, "Scale.trough", &ttkNullElementSpec, 0);
 
     /*
      * Layouts:
      */
-    Ttk_RegisterLayout(themePtr, "TButton", ButtonLayout);
-    Ttk_RegisterLayout(themePtr, "TMenubutton", MenubuttonLayout);
-    Ttk_RegisterLayout(themePtr, "Vertical.TScrollbar",
-    	VerticalScrollbarLayout);
-    Ttk_RegisterLayout(themePtr, "Horizontal.TScrollbar",
-    	HorizontalScrollbarLayout);
-    Ttk_RegisterLayout(themePtr, "Vertical.TScale", VerticalScaleLayout);
-    Ttk_RegisterLayout(themePtr, "Horizontal.TScale", HorizontalScaleLayout);
+    Ttk_RegisterLayouts(themePtr, LayoutTable);
 
-    Tcl_PkgProvide(interp, "tile::theme::xpnative", TILE_VERSION);
+    Tcl_PkgProvide(interp, "ttk::theme::xpnative", TILE_VERSION);
 
     return TCL_OK;
 }
